/********************************************************************
 * OAbnd
 * lib to provide services for browse, disco and feature negotation
 *
 * Copyright (C) 2002 Justin Kirby
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Justin Kirby <justink@rochester.rr.com>
 *
 *******************************************************************/
#include "framework/first.h"
#include "openaether.hpp"
#include <algorithm>
#include "framework/map_utils.hpp"
#include "bnd/disconode.hpp"

namespace jabber
{
  void DiscoNode::Assign(const JabJid& jid)
  {
    if(!jid.IsNull() || !jid.IsValid()){
    }
    m_uri.Jid(jid);
  }
  void DiscoNode::NodeName(const std::string &name)
  {
    if(!name.empty()){
      m_uri.Parameter("node",name);
    }/**\todo should this throw?*/
  }

  void DiscoNode::AddFeature(const DiscoFeature &feature)
  {
    feature_cont::iterator fnd=std::find(m_features.begin(),
					 m_features.end(),
					 feature);
    if(fnd==m_features.end()){
      m_features.push_back(feature);
    }
  }
  void DiscoNode::AddIdentity(const DiscoIdentity &ident)
  {
    identity_cont::iterator fnd=std::find(m_identities.begin(),
					  m_identities.end(),
					  ident);
    if(fnd==m_identities.end()){
      m_identities.push_back(ident);
    }
  }


  JabDiscoInfo* DiscoNode::GetInfo()const
  {
    std::auto_ptr<JabDiscoInfo> jinfo(new JabDiscoInfo);
    /* do features list*/
    feature_cont::const_iterator fpos=m_features.begin();
    for(;fpos!=m_features.end();++fpos){
      jinfo->Insert(*fpos);
    }

    /* do identities list*/
    identity_cont::const_iterator ipos=m_identities.begin();
    for(;ipos!=m_identities.end();++ipos){
      jinfo->Insert(*ipos);
    }

    if(m_uri.HasParameter("node")){
      jinfo->Node(m_uri.Parameter("node"));
    }
    return jinfo.release();
  }

};
