/********************************************************************
 * OAbnd
 * lib to provide services for browse, disco and feature negotation
 *
 * Copyright (C) 2002 Justin Kirby
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Justin Kirby <justink@rochester.rr.com>
 *
 *******************************************************************/
#ifndef OABND_DISCONODE_HPP
#define OABND_DISCONODE_HPP
#include <list>
#include <vector>
#include "jabber/jaburi.h"
#include "bnd/oabnd_exports.h"
#include "bnd/ijabdiscoitem.hpp"
#include "bnd/ijabdiscoinfo.hpp"

namespace jabber
{
  /**
   * Represents a node in the disco framework.
   * Use this as a primary to configure the responses
   * your application will provide to disco queries
   */
  class OABND_API DiscoNode;
  class DiscoNode
  {
  public:
    typedef JabDiscoInfo::Identity DiscoIdentity;
    typedef JabDiscoInfo::Feature DiscoFeature;
    typedef std::list<DiscoNode> child_cont;
  public:
    DiscoNode(){
      m_uri.Query("disco");/*making sure that htis is disco*/
    }
    DiscoNode(const DiscoNode& dn)
      : m_identities(dn.m_identities),
	m_features(dn.m_features),
	m_uri(dn.m_uri),
	m_display(dn.m_display){}
    ~DiscoNode(){}

    /**
     * Get the URI of the node.
     * this is a specific extension to the the jabber uri
     * The uri is created by the assigned jid and its name
     * @return a JabUri
     */
    JabUri Uri()const{return m_uri;}


    /**
     * Assign this node to a specific jid
     * Alternatively you can apply this assignment to
     * all children of this node.
     * @param jid THe jabber id to assign this node to.
     */
    void Assign(const JabJid& jid);
    /**
     * Assign the node a specific name. Until there is a
     * name, this node is invalid.
     * @param name The name to assign this node
     */
    void NodeName(const std::string &name);
    void DisplayName(const std::string &name){m_display=name;}
    std::string DisplayName()const{return m_display;}

    void AddFeature(const DiscoFeature &feature);
    void AddIdentity(const DiscoIdentity &ident);

    void RemoveFeature(const DiscoFeature &feature){
      m_features.remove(feature);
    }
    void RemoveIdentity(const DiscoIdentity &ident){
      m_identities.remove(ident);
    }

    JabDiscoInfo* GetInfo()const;

  private:
    typedef std::list<DiscoIdentity> identity_cont;
    typedef std::list<DiscoFeature> feature_cont;

    identity_cont m_identities;
    feature_cont m_features;

    JabUri m_uri;
    std::string m_display;
  };
};



#endif
