/********************************************************************
 * OAbnd
 * lib to provide services for browse, disco and feature negotation
 *
 * Copyright (C) 2002 Justin Kirby
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Justin Kirby <justink@rochester.rr.com>
 *
 *******************************************************************/
#ifndef OABND_DISCOSRV_HPP
#define OABND_DISCOSRV_HPP
#include <boost/smart_ptr.hpp>
#include "bnd/oabnd_exports.h"
#include "bnd/disconode.hpp"

namespace jabber
{
  /**
   * designed to be used with oa::Singleton<>
   * A note about jaburi
   * the jaburi must be a disco uri (this is a proprietary extension for
   * internal use only. Feel free to adopt it though :)
   * - the Query MUST be disco.
   * - the values used are; node, name
   */
  class OABND_API DiscoSrv;
  class DiscoSrv
  {
  public:
    typedef JabDiscoInfo::Identity DiscoIdentity;
    typedef JabDiscoInfo::Feature DiscoFeature;
  public:

    void CreateNode(const JabUri& uri, const JabUri& parent=JabUri());
    DiscoNode*const GetNode(const JabUri& uri)const;
    void Add(const JabUri& uri, const DiscoIdentity& ident);
    void Add(const JabUri& uri, const DiscoFeature& feature);

    void Remove(const JabUri& uri, const DiscoIdentity& ident);
    void Remove(const JabUri& uri, const DiscoFeature& feature);

    JabDiscoItem* Response(const JabDiscoItem& jdi)const;
    JabDiscoInfo* Response(const JabDiscoInfo& jdi)const;

  protected:
    DiscoSrv(){}
    DiscoSrv(const DiscoSrv& ds){}
    ~DiscoSrv(){}

  private:
    typedef boost::shared_ptr<DiscoNode> node_elem;
    typedef std::multimap<JabUri,node_elem> child_cont;
    typedef std::map<JabUri,node_elem> node_cont;

    child_cont m_children;
    node_cont m_nodes;
  };
};
#endif
