/********************************************************************
 * OAbnd
 * lib to provide services for browse, disco and feature negotation
 *
 * Copyright (C) 2002 Justin Kirby
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Justin Kirby <justink@rochester.rr.com>
 *
 *******************************************************************/
#ifndef OABND_IJABDISCOINFO_HPP
#define OABND_IJABDISCOINFO_HPP

#include <list>
#include <vector>
#include "jabber/jaburi.h"
#include "jabber/ijabiq.h"
#include "bnd/oabnd_exports.h"

namespace jabber
{
  /** @ingroup gJabPacket
   */
  class OABND_API JabDiscoInfo;
  class JabDiscoInfo : public JabIq
  {
  public:
    class Identity
    {
    public:
      Identity(){}
      Identity(const Identity& id)
	: m_category(id.m_category),
	  m_type(id.m_type),
	  m_name(id.m_name){}
      bool operator==(const Identity &id)const{
	if(&id==this){return true;}
	return (m_category==id.m_category &&
		m_type == id.m_type &&
		m_name == id.m_name);
      }
      bool IsEmpty()const{
	/*|| is because the all have to exist */
	return (m_category.empty() ||
		m_type.empty() ||
		m_name.empty());
      }
      std::string Category()const{return m_category;}
      std::string Type()const{return m_type;}
      std::string Name()const{return m_name;}
      void Category(const std::string &category){m_category=category;}
      void Type(const std::string &type){m_type=type;}
      void Name(const std::string &name){m_name=name;}
    private:
      std::string m_category;
      std::string m_type;
      std::string m_name;
    };
    class Feature
    {
    public:
      Feature(){}
      Feature(const Feature& f)
	: m_var(f.m_var){}
      bool operator==(const Feature& f)const{
	if(&f==this){return true;}
	return (m_var==f.m_var);
      }
      bool IsEmpty()const{return m_var.empty();}
      std::string Var()const{return m_var;}
      void Var(const std::string &var){m_var=var;}
    private:
      std::string m_var;
    };
    typedef std::list<Identity> identity_list;
    typedef std::list<Feature> feature_list;
  public:
    JabDiscoInfo()
      : JabIq("jabdiscoinfo"){}
    JabDiscoInfo(const JabDiscoInfo& jdi)
      : JabIq(jdi),
	m_identities(jdi.m_identities),
	m_features(jdi.m_features),
	m_nodeName(jdi.m_nodeName),
	m_name(jdi.m_name){}
    virtual ~JabDiscoInfo(){}

    virtual const char*const Uri()const{
      return "xmpp:xmlns=http://jabber.org/protocol/disco#info";
    }
    JabUri NodeUri()const;
    const char*const InfoUri()const{
      return "http://jabber.org/protocol/disco#info";
    }
    void Node(const std::string &node){m_nodeName=node;}
    std::string Node()const{return m_nodeName;}
    
    identity_list Identities()const;
    feature_list Features()const;
    void Insert(const Identity& id);
    void Insert(const Feature& fe);
    void Remove(const Identity& id);
    void Remove(const Feature& fe);
    virtual xercesc::DOMNode*const BuildXml();
  private:
    bool Exists(const Identity& id)const;
    bool Exists(const Feature& fe)const;

    void BuildIdentities(xercesc::DOMNode*const qry);
    void BuildFeatures(xercesc::DOMNode*const qry);
  private:
    typedef std::vector<Identity> identity_cont;
    typedef std::vector<Feature> feature_cont;

    identity_cont m_identities;
    feature_cont m_features;

    std::string m_nodeName;
    std::string m_name;
  };
};

#endif
