/********************************************************************
 * OAbnd
 * lib to provide services for browse, disco and feature negotation
 *
 * Copyright (C) 2002 Justin Kirby
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Justin Kirby <justink@rochester.rr.com>
 *
 *******************************************************************/
#include "framework/first.h"
#include "openaether.hpp"
#include "framework/map_utils.hpp"
#include "framework/xerces_xmlns.hpp"
#include "framework/xmlutil.hpp"
#include "framework/xmlstr.hpp"
#include "bnd/ijabdiscoitem.hpp"

XERCES_CPP_NAMESPACE_USE
namespace jabber
{
  JabUri JabDiscoItem::NodeUri()const
  {
    JabUri rv;
    JabJid jid;
    if(Type()=="get"){
      /* the uri is constructed via the to jid*/
      jid=To();
    }else if(Type()=="result"){
      jid=From();
    }else{
      return rv;//return a null uri
    }
    rv.Jid(jid);
    rv.Query("disco");
    if(!m_nodeName.empty()){
      rv.Parameter("node",m_nodeName);
    }
    return rv;
  }
  JabDiscoItem::item_list JabDiscoItem::GetItems()const
  {
    JabDiscoItem::item_list templist;
    item_cont::const_iterator pos=m_items.begin();
    for(;pos!=m_items.end();++pos){
      templist.push_back(pos->second);
    }
    return templist;
  }
  JabDiscoItem::item_list JabDiscoItem::GetItems(const JabJid& jj)const
  {
    JabDiscoItem::item_list templist;
    item_cont::const_iterator pos = m_items.lower_bound(jj);
    for(;pos!=m_items.upper_bound(jj);++pos){
      templist.push_back(pos->second);
    }
    return templist;
  }
  JabDiscoItem::Item JabDiscoItem::GetItem(const JabJid& jj,
					   const std::string &node)const
  {
    JabDiscoItem::Item rv;
    item_cont::const_iterator pos=m_items.lower_bound(jj);
    item_cont::const_iterator up=m_items.upper_bound(jj);
    for(;pos!=up;++pos){
      if(pos->second.Node()==node){
	return pos->second;
      }
    }
    return rv;//return the empty node
  }

  void JabDiscoItem::Insert(const Item& item)
  {
    JabDiscoItem::Item rv = GetItem(item.Jid(),item.Node());
    if(!rv.IsEmpty()){
      /**\todo throw?*/
      return;
    }
    m_items.insert(std::make_pair(item.Jid(),item));
  }
  void JabDiscoItem::Remove(const JabJid& jj, const std::string &node)
  {
    item_cont::iterator pos=m_items.lower_bound(jj);
    item_cont::iterator up=m_items.upper_bound(jj);
    for(;pos!=up;++pos){
      if(pos->second.Node()==node){
	m_items.erase(pos);
	return;
      }
    }
  }
  DOMNode*const JabDiscoItem::BuildXml()
  {
    DOMDocument *doc = oa::XMLUtils().Document(DocUri(),"oa");
    oa::dom_ptr<DOMElement> queryNode;
    /* create the query in xmlns disco_items*/
    {
      /* scoping so these don't live forever*/
      oa::xmlstr<XMLCh,char> queryName("query"),
	queryURI(Xmlns().c_str()),
	xmlnsName("xmlns");

      /*create the query element in the ns */
      queryNode=doc->createElementNS(queryURI.get(),queryName.get());

      /* add the xmlns attribute because of the lameness of the formatter*/
      oa::XercesXmlns()(queryNode.get(),queryURI);
      //      queryNode->setAttribute(xmlnsName.get(),queryURI.get());
    }//end of anon-scope
    /* is there a node name? if so add it as an attr*/
    if(!m_nodeName.empty()){
      oa::xmlstr<XMLCh,char> nodeName("node"),
	nodeValue(m_nodeName.c_str());
      queryNode->setAttribute(nodeName.get(),nodeValue.get());
    }

    /* is this a result type? if so add the items*/
    if(Type()=="result"){
      BuildItems(queryNode.get());
    }
    oa::dom_ptr<DOMNode> iq(BuildIq());

    iq->appendChild(queryNode.release());
    return iq.release();
  }

  void JabDiscoItem::BuildItems(DOMNode*const qry)
  {
    DOMDocument *doc = qry->getOwnerDocument();
    oa::xmlstr<XMLCh,char> itemName("item"),
      jidName("jid"),
      nameName("name"),
      nodeName("node"),
      jidValue,nameValue,nodeValue;

    item_cont::iterator pos=m_items.begin();
    for(;pos!=m_items.end();++pos){
      DOMElement *itemNode = doc->createElement(itemName.get());
      JabDiscoItem::Item discoItem = pos->second;
      if(!discoItem.Jid().IsNull()){
	jidValue.set(discoItem.Jid().Jid().c_str());
	itemNode->setAttribute(jidName.get(),jidValue.get());

	if(!discoItem.Name().empty()){
	  nameValue.set(discoItem.Name().c_str());
	  itemNode->setAttribute(nameName.get(),nameValue.get());
	}
	if(!discoItem.Node().empty()){
	  nodeValue.set(discoItem.Node().c_str());
	  itemNode->setAttribute(nodeName.get(),nodeValue.get());
	}
	qry->appendChild(itemNode);
      }
    }
  }
};
