/********************************************************************
 * OAbnd
 * lib to provide services for browse, disco and feature negotation
 *
 * Copyright (C) 2002 Justin Kirby
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Justin Kirby <justink@rochester.rr.com>
 *
 *******************************************************************/
#ifndef OABND_IJABDISCOITEM_HPP
#define OABND_IJABDISCOITEM_HPP
#include <map>
#include <list>
#include "xstrm/jabjid.h"
#include "jabber/jaburi.h"
#include "jabber/ijabiq.h"
#include "bnd/oabnd_exports.h"

namespace jabber
{
  /** @ingroup gJabPacket
   */
  class OABND_API JabDiscoItem;
  class JabDiscoItem : public JabIq
  {
  public:
    class Item
    {
    public:
      Item(){}
      Item(const Item& i)
	: m_jid(i.m_jid),
	  m_node(i.m_node),
	  m_name(i.m_name){}
      ~Item(){}

      bool operator==(const Item& i)const{
	if(&i==this){return true;}
	return(i.m_jid==m_jid &&
	       i.m_node==m_node &&
	       i.m_name==m_name);
      }
      bool IsEmpty()const{
	return (m_jid.IsNull() &&
		m_node.empty() &&
		m_name.empty());
      }
      JabJid Jid()const{return m_jid;}
      std::string Node()const{return m_node;}
      std::string Name()const{return m_name;}
      void Jid(const JabJid& jj){m_jid=jj;}
      void Node(const std::string &nd){m_node=nd;}
      void Name(const std::string &nm){m_name=nm;}
    private:

    private:
      JabJid m_jid;
      std::string m_node;
      std::string m_name;
    };
    typedef std::list<Item> item_list;
  public:
    JabDiscoItem()
      : JabIq("jabdiscoitem"){}
    JabDiscoItem(const JabDiscoItem& jdi)
      : JabIq(jdi),
	m_items(jdi.m_items),
	m_nodeName(jdi.m_nodeName){}
    virtual ~JabDiscoItem(){}


    virtual const char*const Uri()const{
      return "xmpp:xmlns=http://jabber.org/protocol/disco#items";
    }
    virtual void Xmlns(const std::string& xmlns){}//does nothing for a reason
    virtual std::string Xmlns()const{
      return "http://jabber.org/protocol/disco#items";
    }

    JabUri NodeUri()const;

    std::string Node()const{return m_nodeName;}
    std::string Name()const{return m_name;}
    
    void Node(const std::string &node){m_nodeName=node;}
    void Name(const std::string &name){m_name=name;}

    item_list GetItems()const;
    item_list GetItems(const JabJid& jj)const;
    Item GetItem(const JabJid& jj, const std::string &node)const;

    void Insert(const Item& item);
    void Remove(const JabJid& jj, const std::string &node);
    virtual xercesc::DOMNode*const BuildXml();
  protected:
    void BuildItems(xercesc::DOMNode*const qry);
  private:
    typedef std::multimap<JabJid,Item> item_cont;
    item_cont m_items;
    std::string m_nodeName;
    std::string m_name;
  };
};
#endif
