/********************************************************************
 * OAbnd
 * lib to provide services for browse, disco and feature negotation
 *
 * Copyright (C) 2002 Justin Kirby
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Justin Kirby <justink@rochester.rr.com>
 *
 *******************************************************************/
#include "framework/first.h"
#include "openaether.hpp"
#include "bnd/jabblddiscoinfo.hpp"
#include <xercesc/util/XMLString.hpp>
#include <xercesc/dom/DOMElement.hpp>
#include <xercesc/dom/DOMNodeList.hpp>
#include "framework/xmlstr.hpp"

#define URI_INFO "http://jabber.org/protocol/disco#info"

XERCES_CPP_NAMESPACE_USE
namespace jabber
{
  bool DiscoInfoBuild::CanBuild(DOMNode *const nd)const
  {
    /* The req is quite simple, if the query is
     * xmlns="http://jabber.org/protocol/disco#info
     */
    oa::xmlstr<XMLCh,char> queryName("query"),
      infoUri(URI_INFO);
    DOMNode*query=nd->getFirstChild();
    if(query &&
       query->getNodeType()==DOMNode::ELEMENT_NODE &&
       !XMLString::compareString(query->getNodeName(),queryName.get()) &&
       !XMLString::compareString(query->getNamespaceURI(),infoUri.get())){
      return true;
    }

    return false;
  }

  JabPacket* DiscoInfoBuild::Build(DOMNode *const nd)
  {
    /* and off we go to build the info, hi ho, hi ho*/

    std::string to,from,id,type;
    GetTo(to,nd);
    GetFrom(from,nd);
    GetId(id,nd);
    GetType(type,nd);

    DOMNode*queryNode=nd->getFirstChild();
    if(!queryNode){
      return 0;/* aaaahhhH!!!! die horribly or something*/
    }

    /* verify its a domelement and called "query"...
     * then cast it to a domelement and build
     */
    oa::xmlstr<XMLCh,char> queryName("query");
    if(queryNode->getNodeType()==DOMNode::ELEMENT_NODE &&
       !XMLString::compareString(queryNode->getNodeName(),queryName.get())){
      DOMElement*queryElem=((DOMElement*)queryNode);

      std::auto_ptr<JabDiscoInfo> jabInfo(new JabDiscoInfo);
      
      /* if we ahve a node attr get it*/
      oa::xmlstr<XMLCh,char> nodeAttr("node");
      oa::xmlstr<char,XMLCh> nodeValue;
      if(queryElem->hasAttribute(nodeAttr.get())){
	nodeValue.set(queryElem->getAttribute(nodeAttr.get()));
	jabInfo->Node(nodeValue.get());
      }

      /* now we ahve to alternate between feature n identity*/
      if(queryElem->hasChildNodes()){
	JabDiscoInfo::Identity ident;
	JabDiscoInfo::Feature feature;
	oa::xmlstr<XMLCh,char> identName("identity"),
	  featureName("feature");

	DOMNodeList* infoList = queryElem->getChildNodes();
	for(unsigned int i=0;i<infoList->getLength();++i){
	  DOMNode*infoNode=infoList->item(i);
	  /* if this is a identity, do that*/
	  if(!XMLString::compareString(infoNode->getNodeName(),
				       identName.get())){
	    if(BuildIdentity(ident,infoNode)){
	      jabInfo->Insert(ident);
	    }
	    
	  }else if(!XMLString::compareString(infoNode->getNodeName(),
					     featureName.get())){
	    if(BuildFeature(feature,infoNode)){
	      jabInfo->Insert(feature);
	    }
	  }
	}
      }
      JabJid toJid,fromJid;
      toJid.Jid(to);
      fromJid.Jid(from);
      jabInfo->To(toJid);
      jabInfo->From(fromJid);
      jabInfo->Id(id);
      jabInfo->Type(type);
      return jabInfo.release();
    }
    return 0;
  }
  bool DiscoInfoBuild::BuildIdentity(JabDiscoInfo::Identity &jdii,
				     DOMNode*const nd)
  {
    if(nd && nd->getNodeType()==DOMNode::ELEMENT_NODE){
      oa::xmlstr<XMLCh,char> catName("category"),
	typeName("type"),
	nameName("name");

      DOMElement*identElem=((DOMElement*)nd);
      if(identElem->hasAttribute(catName.get()) &&
	 identElem->hasAttribute(typeName.get()) &&
	 identElem->hasAttribute(nameName.get())){
	oa::xmlstr<char,XMLCh> catVal,typeVal,nameVal;
	catVal.set(identElem->getAttribute(catName.get()));
	typeVal.set(identElem->getAttribute(typeName.get()));
	nameVal.set(identElem->getAttribute(nameName.get()));
	if(catVal.get()){
	  jdii.Category(catVal.get());
	}
	if(typeVal.get()){
	  jdii.Type(typeVal.get());
	}
	if(nameVal.get()){
	  jdii.Name(nameVal.get());
	}
	return true;
      }
    }
    return false;
  }
  bool DiscoInfoBuild::BuildFeature(JabDiscoInfo::Feature &jdif,
				    DOMNode*const nd)
  {
    if(nd && nd->getNodeType()==DOMNode::ELEMENT_NODE){
      DOMElement*featElem=((DOMElement*)nd);

      oa::xmlstr<XMLCh,char> varName("var");
      if(featElem->hasAttribute(varName.get())){
	oa::xmlstr<char,XMLCh> varVal;
	varVal.set(featElem->getAttribute(varName.get()));
	if(varVal.get()){
	  jdif.Var(varVal.get());
	}
	return true;
      }
    }
    return false;
  }
};
