/********************************************************************
 * OAbnd
 * lib to provide services for browse, disco and feature negotation
 *
 * Copyright (C) 2002 Justin Kirby
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Justin Kirby <justink@rochester.rr.com>
 *
 *******************************************************************/
#include "framework/first.h"
#include "openaether.hpp"
#include "bnd/jabblddiscoitem.hpp"
#include "framework/xmlstr.hpp"
#include "oapu/oapr_logstream.hpp"
#include "framework/xmlutil.hpp"
#include <xercesc/util/XMLString.hpp>
#include <xercesc/dom/DOMElement.hpp>
#include <xercesc/dom/DOMNodeList.hpp>

#define DISCO_ITEM \
"http://jabber.org/protocol/disco#items"

XERCES_CPP_NAMESPACE_USE
namespace jabber
{
  bool DiscoItemBuild::CanBuild(DOMNode *const nd)const
  {
    /* the question is: does it have aquery element
     * in the disco namespace of #items?
     * if so, then this should handle the building of it
     */
    if(!nd){return false;}//and there was much paranoia
    oa::xmlstr<XMLCh,char> itemuri,queryName;
    itemuri.set(DISCO_ITEM);
    queryName.set("query");

    //get the first child of this node
    DOMNode *query=nd->getFirstChild();
    if(!query){return false;}

    const XMLCh* qryuri=query->getNamespaceURI();
    if(!XMLString::compareString(qryuri,itemuri.get()) &&
       !XMLString::compareString(query->getNodeName(),queryName.get())){
      return true;
    }
    return false;
  }

  JabPacket* DiscoItemBuild::Build(DOMNode *const nd)
  {
    /* several possiblities of this
     * - a type=get with empty query w/o node
     * - a type=get with empty query w/ node
     * - a type=result with items w/o node
     * - a type=result with items w/ node
     */
    std::string to,from,type,id, node;

    /* process the 'header' first*/
    GetTo(to,nd);
    GetFrom(from,nd);
    GetType(type,nd);
    GetId(type,nd);

    /* obtain the query elem */
    DOMNode* queryNode = nd->getFirstChild();
    if(!queryNode){
      /* no query element... blank <iq/> huh?*/
      return 0;
    }

    /* cast queryNode to DOMElement and get node attr if exists*/
    /* be paranoid and check type*/
    if(queryNode->getNodeType()==DOMNode::ELEMENT_NODE){

      std::auto_ptr<JabDiscoItem> jabItem(new JabDiscoItem);

      DOMElement *queryElem = (DOMElement*)queryNode;
      /* if we ahve a node get it*/
      oa::xmlstr<XMLCh,char> nodeAttr("node");
      oa::xmlstr<char,XMLCh> valNode;
      if(queryElem->hasAttribute(nodeAttr.get())){
	valNode.set(queryElem->getAttribute(nodeAttr.get()));
	jabItem->Node(valNode.get());
      }

      /* if we have items, get them too*/
      JabDiscoItem::Item item;
      if(queryElem->hasChildNodes()){
	DOMNodeList *items = queryElem->getChildNodes();
	for(unsigned int i=0; i<items->getLength(); ++i){
	  if(BuildItem(item,items->item(i))){
	    jabItem->Insert(item);
	  }
	}
      }

      /* set all the misc crap*/
      JabJid toJid,fromJid;
      toJid.Jid(to);
      fromJid.Jid(from);
      jabItem->To(toJid);
      jabItem->From(fromJid);
      jabItem->Type(type);
      jabItem->Id(id);      

      return jabItem.release();
    }else{
      return 0;/* query isn't a node, throw or do something more useful*/
    }
  }
  bool DiscoItemBuild::BuildItem(JabDiscoItem::Item &jdi, DOMNode*const nd)
  {
    /* for an item we get the attributes and build the item*/
    oa::xmlstr<XMLCh,char> attrJid("jid"),attrNode("node"),attrName("name");
    oa::xmlstr<char,XMLCh> valJid,valNode,valName;
    /* only jid and name are required*/
    DOMElement*const itemElem=(DOMElement*const)nd;
    if(itemElem->hasAttribute(attrJid.get())){
      valJid.set(itemElem->getAttribute(attrJid.get()));

      if(itemElem->hasAttribute(attrName.get())){
	valName.set(itemElem->getAttribute(attrName.get()));
	jdi.Name(valName.get());
      }

      if(itemElem->hasAttribute(attrNode.get())){
	valNode.set(itemElem->getAttribute(attrNode.get()));
	jdi.Node(valNode.get());
      }

      JabJid jid;
      jid.Jid(valJid.get());
      jdi.Jid(jid);

      return true;
    }else{
      /* throw here or not?*/
      jdi.Name("");
      jdi.Node("");
      return false;
    }
  }
};
