#!/bin/sh
####################################
#                                  #
# EDIT boostrap.conf file frist!!! #
#                                  #
####################################
if test ! -e $(pwd)/config; then
    mkdir -p $(pwd)/config
fi
BOOTSTRAP_CONF=$(pwd)/config/bootstrap.conf

if test ! -f $BOOTSTRAP_CONF; then
    cat <<__EOF__;
You appear to be running bootstrap for the first time.

A build configuration file has been created: $BOOTSTRAP_CONF
 
View this file and change it to suit your system if necessary.
Especially the paths to required programs, and install prefix.

Once you have done this, run $0 to configure the build.

For advanced build options, try $0 -h

If you encounter problems, check your settings, make sure everything
required is installed, and if you still have no luck, consult documentation.

__EOF__
cat >$BOOTSTRAP_CONF <<__EOF__;
#OpenAether build configuration
#this file will not be overwritten when checking out from cvs
# 0=no, 1=yes comment out or leave blank to disable
PREFIX=           # Build prefix for configure (same as --prefix=). Suggest /usr for most distros or /usr/local (default)
CONF_DAILY=1      # Turn on daily build number, based on time
CONF_DEBUG=1      # Do debug build
CONF_EFENCE=0     # use electric fence if its available

CONF_XERCESINC=/usr/local/include # xerces-c include dir
CONF_XERCESLIB=/usr/local/lib     # xerces-c lib dir
CONF_XERCESINC_D=                 # xerces-c debug include dir
CONF_XERCESLIB_D=                 # xerces-c debug lib dir


CONF_APR=/usr/local/bin           # Location of apr-config
CONF_APU=/usr/local/bin           # Location of apu-config
CONF_OAPR=/usr/local/bin          # Location of oapr-config
CONF_OAPU=/usr/local/bin          # Location of oapu-config

CONF_BOOSTINC=                    # Boost include dir


#---------------#
# Cross options #
#---------------#
TARGET=
CPREFIX=
CBUILD=
__EOF__
  exit 4;
fi

# usage function
usage()
{
    echo 'Usage: $0 [options]'
    echo '';
    echo ' -e display proposed configure arguments'
    echo ' -a run autotools; libtoolize, aclocal, autoconf, automake'
    echo ' -c run configure '
    echo " -d force debug build (overrides $BOOSTRAP_CONF)"
    echo " -x perform a cross compile"
    echo ' -h display this message'
    echo 'Default behavior is to run autools and configure then create a release build.'
};


build_confargs()
{
    PRE=
    if test "$cross" = "on"; then
	PRE=$CPREFIX
    fi

    if test $PREFIX; then
	CONF_ARGS="$CONF_ARGS --prefix=$PREFIX "
    fi
    if test $CONF_DAILY = 1; then
	CONF_ARGS="$CONF_ARGS --enable-daily ";
    fi

    if test $CONF_DEBUG = 1; then
	CONF_ARGS="$CONF_ARGS --enable-debug --enable-maintainer-mode ";
    fi
    if test $CONF_EFENCE = 1; then
	CONF_ARGS="$CONF_ARGS --enable-efence ";
    fi

    if test $CONF_XERCESINC; then
	CONF_ARGS="$CONF_ARGS --with-xercescinc=$PRE$CONF_XERCESINC ";
    fi
    if test $CONF_XERCESLIB; then
	CONF_ARGS="$CONF_ARGS --with-xercesclib=$PRE$CONF_XERCESLIB ";
    fi
    if test $CONF_XERCESINC_D; then
	CONF_ARGS="$CONF_ARGS --with-xercescinc-debug=$PRE$CONF_XERCESINC_D ";
    fi
    if test $CONF_XERCESLIB_D; then
	CONF_ARGS="$CONF_ARGS --with-xercesclib-debug=$PRE$CONF_XERCESLIB_D ";
    fi

    if test $CONF_BOOSTINC; then
	CONF_ARGS="$CONF_ARGS --with-boostinc=$PRE$CONF_BOOSTINC ";
    fi
    if test $CONF_APR; then
	CONF_ARGS="$CONF_ARGS --with-apr=$PRE$CONF_APR ";
    fi
    if test $CONF_APU; then
	CONF_ARGS="$CONF_ARGS --with-apr-util=$PRE$CONF_APU ";
    fi
    if test $CONF_OAPR; then
	CONF_ARGS="$CONF_ARGS --with-oapr=$PRE$CONF_OAPR ";
    fi
    if test $CONF_OAPU; then
	CONF_ARGS="$CONF_ARGS --with-oapu=$PRE$CONF_OAPU ";
    fi

    return 0;
};


do_autoconf()
{
    old_pwd=$PWD
    srcdir=$(dirname $0)

    cd $srcdir
    echo "Running autotools..."
    #checking for config dir... if not exist make it
    if ! test -d config ; then
	mkdir config || exit $?
    fi
    if test -e config.status; then
	rm -f config.status
    fi

    rm -f config.cache
    rm -f acconfig.h
    echo "..libtoolizing"

    libtoolize --force --copy || exit $?
    echo "..aclocal."
    aclocal -I m4 || exit $?
    echo "..autoconf."
    autoconf || exit $?
    echo "..autoheader."
    autoheader || exit $?
    echo "..automake."
    automake -a || exit $?
    cd $old_pwd
};


do_configure()
{
    configure=$(dirname $0)/configure
    eval $configure $CONF_ARGS || exit $?
}

do_cross()
{
    configure=$(dirname $0)/configure
    PATH="$CPREFIX/bin:$CPREFIX/$TARGET/bin:$PATH"
    export PATH
    eval $configure --target=$TARGET --host=$TARGET --build=$CBUILD \
	$CONF_ARGS || exit $?
    cat <<__EOF__;
WARNING!!  do not run make. instead run cross-make.sh like:
__EOF__
  cat >$PWD/cross-make.sh <<__EOF__;
    #setting vars from bootstrap.conf
    PATH="$CPREFIX/bin:$CPREFIX/$TARGET/bin:$PATH"
    export PATH
    exec make $*
__EOF__


}
### MAIN CODE

# Parse command-line options

set -- `getopt am:cm:em:dm:xm:h $*`;
if test $? != 0; then
  usage;
  exit 1;
fi

# see if we are in a cvs repository and set maintainer mode 
MAINTAINER=0
if test -d CVS; then
    MAINTAINER=1;
else
    MAINTAINER=0;
fi


echo_confargs=off;
run_autoconf=off;
run_configure=off;
debug=off;
cross=off;
user_class=expert;

# No args, assume -ac and don't assume expert user
if test $# = 1; then
  run_autoconf=on;
  run_configure=on;
  user_class=unknown;
fi


set -- `getopt am:cm:em:dm:xm:h $*`;
if [ $? != 0 ]
    then
    usage;
    exit 1
fi


# Parse options, if any
for i in $*
do
    case $i in
      -e) echo_confargs=on; shift;;
      -a) run_autoconf=on; shift;;
      -c) run_configure=on; shift;;
      -d) debug=on; shift;;
      -x) cross=on; shift;;
      -h) usage; exit 1;;
      --) shift; break;;      
    esac
done;
# Read bootstrap.conf

if test $user_class != "expert"; then
  echo "Reading $BOOTSTRAP_CONF.";
fi
. $BOOTSTRAP_CONF

# -d: Force debug build
if test $debug = "on"; then
    CONF_DEBUG=1;
fi

build_confargs;
#do we just build the conf args and exit
if test $echo_confargs = "on"; then
    echo ${CONF_ARGS};
fi


#do we run autoconf?
if test $run_autoconf = "on"; then
    do_autoconf;
fi
if test $cross = "on"; then
    do_cross
    exit 0
fi
if test $run_configure = "on"; then
    do_configure;
fi

#just exit here for tests and such
exit 0;

