/********************************************************************
 * OpenAether Framework
 * Contains objects to aid in the development of applications
 * mainly focusing on test applications and daemons
 *
 * Copyright (C) 2002 Justin Kirby
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Justin Kirby <justink@rochester.rr.com>
 *
 *******************************************************************/
#ifdef WIN32
#include "first.h"
#endif
#include "framework/config.hpp"
#include "framework/config_algorithm.hpp"

namespace oa
{
  bool Config::Has(std::string param)
  {
    if(m_config.find(param) != m_config.end()){
      return true;
    }else{
      return false;
    }
  }
  bool Config::Get(std::string param, std::string &val)
  {
    strmap::iterator pos = m_config.find(param);
    if(pos != m_config.end()){
      val = pos->second;
      return true;
    }else{
       return false;
    }
  }
  bool Config::Get(int i, std::string &param, std::string &val)
  {
    if(m_config.size()<=0){
      return false;
    }
    strmap::iterator pos=m_config.begin();
    for(int ipos=0;ipos<i && pos!=m_config.end();++ipos){
      pos++;
    }
    if(pos == m_config.end()){
      return false;
    }
    param = pos->first;
    val = pos->second;
    return true;
  }
  bool Config::Set(std::string param, std::string val)
  {
    strmap::iterator found = m_config.find(param);
    if(found == m_config.end()){
      m_config.insert(std::make_pair(param,val));
      return true;
    }else{
      return false;
    }
  }

  ConfigChain::~ConfigChain()
  {
    /* boost::shared_ptr handles freeing*/
  }
  Config* ConfigChain::Key(std::string key)
  {
    cfgmap::iterator pos;
    for(pos=m_configs.begin(); pos != m_configs.end(); ++pos){
      if((*pos)->Key() == key){
	return pos->get();
      }
    }
    return 0;
  }
  Config* ConfigChain::Key(int it)
  {
    if(m_configs.size()<=0){
      return 0;
    }
    if(it < 0 || static_cast<unsigned int>(it) >= m_configs.size()){
      return 0;
    }
    return m_configs[it].get();
  }

  int ConfigChain::Insert(Config* cfg)
  {
    if(cfg){
      m_configs.push_back(CONFIGPtr(cfg));
    }
    return m_configs.size();
  }
};
