/********************************************************************
 * OpenAether Framework
 * Contains objects to aid in the development of applications
 * mainly focusing on test applications and daemons
 *
 * Copyright (C) 2002 Justin Kirby
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Justin Kirby <justink@rochester.rr.com>
 *
 *******************************************************************/
/*******************************************************************
 * CHANGE LOG
 * 2003-03-10
 * - removed Config from being and AppNode.
 * - Config is its own type, had to make some code changes to accomodate
 * 2003-03-09
 * - added copy ctors to Config and ConfigChain
 * - then I removed the ctors
 *******************************************************************/
#ifndef FRAMEWORK_CONFIG_HPP
#define FRAMEWORK_CONFIG_HPP
#include "framework/framework_exports.h"
#include <string>
#include <map>
#include <vector>
#include "boost/smart_ptr.hpp"

namespace oa
{
  class FW_API Config;
  class FW_API ConfigChain;
  class Config
  {
  public:
    typedef std::map<std::string,std::string> strmap;
  public:
    Config(std::string key) : m_key(key){}
    virtual ~Config(){}
    virtual std::string Key(){
      return m_key;
    }
    virtual Config* Key(int it){return 0;}
    virtual Config* Key(std::string key){return 0;}
    virtual bool Has(std::string param);
    virtual bool Get(int i,std::string &param,std::string &val);
    virtual bool Get(std::string param, std::string &val);
    virtual bool Set(std::string param, std::string val);
  protected:
    strmap m_config;
    std::string m_key;

  };

  class ConfigChain : public Config
  {
  public:
    typedef boost::shared_ptr<Config> CONFIGPtr;
    typedef std::vector<CONFIGPtr> cfgmap;
  public:
    ConfigChain(std::string key) : Config(key){}
    virtual ~ConfigChain();
    
    virtual Config* Key(int it);
    virtual Config* Key(std::string key);
    virtual int Insert(Config *cfg);
  protected:
    cfgmap m_configs;
  };
};
#endif
