/********************************************************************
 * OpenAether Framework
 * Contains objects to aid in the development of applications
 * mainly focusing on test applications and daemons
 *
 * Copyright (C) 2002 Justin Kirby
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Justin Kirby <justink@rochester.rr.com>
 *
 *******************************************************************/
#include "framework/first.h"
#include "framework/config_algorithm.hpp"
#include "framework/parser_cmdline.hpp"

namespace oa
{
  Config* ConfigMain::operator()()
  {
    std::vector<std::string> args;
    for(int i=1;i<m_argc;++i){
      args.push_back(m_argv[i]);
    }
    CmdLine line;
    line.Source(args);
    return line.Parse();
  }
  Config* ConfigFind::FirstKey(std::string &key){
    return FirstKey(p_config,key);
  }
  Config* ConfigFind::LastKey(std::string &key)
  {
    return 0;
  }

  Config* ConfigFind::FirstKey(Config *child,std::string &key)
  {
    if(!child){return 0;}
    Config *found=0,*pos= child->Key(key);
    if(pos){
      return pos;
    }

    for(int i=0;(pos=p_config->Key(i));++i){
      found = FirstKey(pos,key);
      if(found){
	return found;
      }
    }
    return 0;
  }
  void ConfigPrint::Print()
  {
    if(!p_config || !m_ostrm){
      return;
    }
    Print(p_config);
  }
  void ConfigPrint::Print(Config *cfg)
  {

    PrintWs();
    m_ostrm << cfg->Key() << std::endl;
    m_ws++;

    std::string param,value;
    Config *child=0;
    for(int p=0;(cfg->Get(p,param,value));++p){
      PrintWs();
      m_ostrm << param << "=" << value << std::endl;
    }
    for(int i=0;(child=cfg->Key(i));++i){
      Print(child);
    }
    m_ws--;
  }
  void ConfigPrint::PrintWs()
  {
    for(int i=0;i<m_ws;++i){
      m_ostrm<<"  ";
    }
  }
};
