/********************************************************************
 * OpenAether Framework
 * Contains objects to aid in the development of applications
 * mainly focusing on test applications and daemons
 *
 * Copyright (C) 2002 Justin Kirby
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Justin Kirby <justink@rochester.rr.com>
 *
 *******************************************************************/
#ifndef FRAMEWORK_CONFIG_ALGORITHM_HPP
#define FRAMEWORK_CONFIG_ALGORITHM_HPP
#include "framework/framework_exports.h"
#include <iostream>

#include "framework/config.hpp"

namespace oa
{
#ifdef WIN32
  class FW_API ConfigFind;
  class FW_API ConfigPrint;
#endif

  class ConfigMain
  {
  public:
    ConfigMain(int argc,char *argv[])
      : m_argc(argc),m_argv(argv){}
    virtual ~ConfigMain(){}

    Config* operator()();
  private:
    int m_argc;
    char** m_argv;
  };
  class ConfigFind
  {
  public:
    ConfigFind(Config *cfg) : p_config(cfg){}
    virtual ~ConfigFind(){}

    virtual Config* FirstKey(std::string &key);
    virtual Config* LastKey(std::string &key);
  private:
    virtual Config* FirstKey(Config *child,std::string &key);
  private:

    Config *p_config;
  };

  class ConfigPrint
  {
  public:
    ConfigPrint(Config *cfg, std::streambuf *strm)
      : p_config(cfg), m_ostrm(strm), m_ws(0){}
    virtual ~ConfigPrint(){}

    virtual void Print();
  protected:
    virtual void Print(Config *cfg);
    virtual void PrintWs();
  protected:
    Config *p_config;
    std::iostream m_ostrm;

    std::string m_source;
    std::string m_destination;

    int m_ws;
  };

};

#endif
