/********************************************************************
 * OpenAether Framework
 * Contains objects to aid in the development of applications
 * mainly focusing on test applications and daemons
 *
 * Copyright (C) 2002 Justin Kirby
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Justin Kirby <justink@rochester.rr.com>
 *
 *******************************************************************/
#ifndef FRAMEWORK_DOM_PTR_HPP
#define FRAMEWORK_DOM_PTR_HPP

namespace oa
{
  template<typename Dom_t>
  Dom_t* checked_delete_dom(Dom_t *p){
    if(p){p->release();}
    return 0;
  }

  template<typename Dom_t>
  class dom_ptr
  {
  public:
    typedef Dom_t dom_type;

    explicit dom_ptr(Dom_t *p=0)
      : p_ptr(p){}
    dom_ptr(const dom_ptr<Dom_t>& dp)
      : p_ptr(dp.release()){}
    ~dom_ptr(){
      checked_delete_dom(p_ptr);
    }
    void reset(Dom_t *p=0){
      if(p_ptr != p){
	p_ptr=checked_delete_dom(p_ptr);
	p_ptr=p;
      }
    }

    dom_ptr& operator=(Dom_t *p){
      reset(p);
      return *this;
    }

    dom_ptr& operator=(dom_ptr &dp){
      reset(dp.release());
      return *this;
    }


    Dom_t& operator*()const { return *p_ptr;}
    Dom_t *const operator->()const { return p_ptr;}



    Dom_t *const get()const {return p_ptr;}
    Dom_t * release(){
      Dom_t* tmp=p_ptr;
      p_ptr=0;
      return tmp;
    }

  private:
    Dom_t *p_ptr;
  };


  /**\todo implement shared pointer */
};
#endif
