/********************************************************************
 * OpenAether Framework
 * Contains objects to aid in the development of applications
 * mainly focusing on test applications and daemons
 *
 * Copyright (C) 2002 Justin Kirby
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Justin Kirby <justink@rochester.rr.com>
 *
 *******************************************************************/
#ifndef FRAMEWORK_DOMBUILDER_H
#define FRAMEWORK_DOMBUILDER_H
#include <string>
#include "boost/scoped_ptr.hpp"
#include "framework/framework_exports.h"
#include "framework/dom_ptr.hpp"
#include <xercesc/util/XMLUniDefs.hpp>
#include <xercesc/dom/DOMDocument.hpp>
#include <xercesc/dom/DOMNode.hpp>
#include <xercesc/dom/DOMText.hpp>
#include <xercesc/dom/DOMElement.hpp>
#include <xercesc/sax2/Attributes.hpp>
#include <xercesc/sax/Locator.hpp>
#include <xercesc/sax/InputSource.hpp>
#include <xercesc/sax/SAXParseException.hpp>
#include <xercesc/sax/EntityResolver.hpp>
#include <xercesc/sax/DTDHandler.hpp>
#include <xercesc/sax/ErrorHandler.hpp>
#include <xercesc/sax2/ContentHandler.hpp>
#include <xercesc/sax2/LexicalHandler.hpp>
#include <xercesc/sax2/DeclHandler.hpp>
namespace oa
{
  class FW_API DomBuilder;
  class DomBuilder : public xercesc::EntityResolver,
		     public xercesc::DTDHandler,
		     public xercesc::ContentHandler,
		     public xercesc::ErrorHandler,
		     public xercesc::LexicalHandler
		     /**\todo implement DeclHandler*/
  {
  public:
    typedef std::basic_string<XMLCh> xmlstring;
  public:
    DomBuilder()
      : m_current(0),
	m_position(0),
	m_text(0),
	m_defaultNs(0),
	m_append(false){}
    virtual ~DomBuilder(){}


    xercesc::DOMElement* getElement()const{//do proper downcast
      if(!m_doc.get()){return 0;}
      return m_doc->getDocumentElement();
    }

    // --------------------------------------
    // Handlers for the SAX DocumentHandler interface
    //--------------------------------------
    virtual void startDocument();
    virtual void startElement(const XMLCh *const uri, 
			      const XMLCh *const localname,
			      const XMLCh *const qname,
			      const xercesc::Attributes& attributes);
    virtual void startPrefixMapping(const XMLCh *const prefix,
				    const XMLCh *const uri);
    virtual void characters(const XMLCh* const chars,
			    const unsigned int length);
    virtual void ignorableWhitespace(const XMLCh *const chars,
				     const unsigned int length);
    virtual void processingInstruction(const XMLCh *const target,
				       const XMLCh *const data);
    virtual void setDocumentLocator(const xercesc::Locator *const locator);

    virtual void endPrefixMapping(const XMLCh *const prefix);
    virtual void endElement(const XMLCh *const uri,
			    const XMLCh *const localname,
			    const XMLCh *const qname);
    virtual void endDocument();
    virtual void skippedEntity(const XMLCh *const name);
    // --------------------------------------
    // Functions from LexicalHandler interface
    // --------------------------------------
    virtual void startCDATA();
    virtual void startEntity(const XMLCh* const name);
    virtual void startDTD(const XMLCh *const name, 
			  const XMLCh *const publicId,
			  const XMLCh *const systemId);
    virtual void comment(const XMLCh *const chars, const unsigned int length);
    virtual void endDTD();
    virtual void endEntity(const XMLCh *const name);
    virtual void endCDATA();

    // --------------------------------------
    // Functions for the SAX DTDHandler interface
    //--------------------------------------
    virtual void notationDecl(const XMLCh *const name,
			      const XMLCh *const publicId,
			      const XMLCh *const systemId);
    virtual void unparsedEntityDecl(const XMLCh *const name,
				    const XMLCh *const publicId,
				    const XMLCh *const systemId,
				    const XMLCh *const notationName);
    virtual void resetDocType();

    // --------------------------------------
    // Functions for the SAX EntifyResolver interface
    //--------------------------------------
    virtual xercesc::InputSource* resolveEntity(const XMLCh *const publicId,
				       const XMLCh *const systemId);

    // --------------------------------------
    // Functions for the SAX ErrorHandler interface
    //--------------------------------------
    virtual void error(const xercesc::SAXParseException& exception);
    virtual void fatalError(const xercesc::SAXParseException& exception);
    virtual void warning(const xercesc::SAXParseException& exception);
    virtual void resetErrors();

  private:
    oa::dom_ptr<xercesc::DOMDocument> m_doc;
    xercesc::DOMNode *m_root;
    xercesc::DOMNode *m_current;
    xercesc::DOMNode *m_position;
    xercesc::DOMText *m_text;
    boost::scoped_ptr<XMLCh> m_defaultNs;

    bool m_append;
  };
};

#endif
