/********************************************************************
 * OpenAether Framework
 * Contains objects to aid in the development of applications
 * mainly focusing on test applications and daemons
 *
 * Copyright (C) 2002 Justin Kirby
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Justin Kirby <justink@rochester.rr.com>
 *
 *******************************************************************/
#ifndef FRAMEWORK_FIFO_QUEUE_HPP
#define FRAMEWORK_FIFO_QUEUE_HPP


#include <list>
#include <boost/utility.hpp>

namespace oa
{
  template<typename Obj,
	   typename ThreadPol,
	   typename Cont=std::list<Obj> >
  class Fifo : private boost::noncopyable
  {
  public:
    typedef typename ThreadPol::lock_type lock_type;
  public:
    Fifo()
      :m_sigd(false){}
    ~Fifo(){}

    void signal(){
      lock_type lock(m_monitor);
      m_sigd=true;
      m_bufferEmpty.Broadcast();
    }
    void push(const Obj& o){
      lock_type lock(m_monitor);
      m_cont.push_back(o);
      m_bufferEmpty.Broadcast();
    }
    bool get(Obj& o){
      lock_type lock(m_monitor);
      while(m_cont.empty() && !m_sigd){
	m_bufferEmpty.Wait(m_monitor);
      }

      if(!m_cont.empty()){
	o = m_cont.front();
	m_cont.pop_front();
	return true;
      }else{
	return false;
      }
    }

  private:
    Cont m_cont;
    bool m_sigd;
    typename ThreadPol::condition m_bufferEmpty;
    typename ThreadPol::mutex m_monitor;
  };
};
#endif
