/********************************************************************
 * OpenAether Framework
 * Contains objects to aid in the development of applications
 * mainly focusing on test applications and daemons
 *
 * Copyright (C) 2002 Justin Kirby
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Justin Kirby <justink@rochester.rr.com>
 *
 *******************************************************************/
#ifndef FRAMEWORK_FWIS_IDLER_HPP
#define FRAMEWORK_FWIS_IDLER_HPP
#include <signal.h>
#include <pthread.h>

namespace oa
{
  class Idler
  {
  public:
    typedef pthread_mutex_t mutex_type;;
    typedef pthread_cond_t  cond_type;
  public:
    Idler(){}
    ~Idler(){}

    int operator()(int sc,int *sv){
      /* register signal callback*/
      for(int i=0; i<sc;++i){
	signal(sv[i],&Idler::SigCallback);
      }
      pthread_mutex_init(&Idler::m_monitor,NULL);
      pthread_cond_init(&Idler::m_killsig,NULL);
      Idler::Idle();
      pthread_mutex_destroy(&Idler::m_monitor);
      pthread_cond_destroy(&Idler::m_killsig);
      return 0;      
    }

  private:
    Idler(const Idler& i){}

    static void SigCallback(int sig){
      pthread_cond_broadcast(&Idler::m_killsig);
    }

    static void Idle(){
      pthread_mutex_lock(&Idler::m_monitor);
      pthread_cond_wait(&Idler::m_killsig,&Idler::m_monitor);
      pthread_mutex_unlock(&Idler::m_monitor);
    }
  private:
    static mutex_type m_monitor;
    static cond_type  m_killsig;

  };
};

#endif
