/********************************************************************
 * OpenAether Framework
 * Contains objects to aid in the development of applications
 * mainly focusing on test applications and daemons
 *
 * Copyright (C) 2002 Justin Kirby
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Justin Kirby <justink@rochester.rr.com>
 *
 *******************************************************************/
#ifndef FWIS_MED_NODE_HPP
#define FWIS_MED_NODE_HPP
#include "framework/config.hpp"
#include <boost/smart_ptr.hpp>
#include "framework/fwis/med_sender.hpp"

namespace oa
{
  template<typename event_t>
  class MedNode
  {
  public:
    typedef oa::Config conf_type;
    typedef boost::shared_ptr<conf_type> conf_ptr;
    typedef boost::shared_ptr<event_t> event_ptr;
    typedef event_t event_type;

    typedef MedSender<event_t> sender_type;
    typedef boost::shared_ptr<sender_type> sender_ptr;
  public:
    MedNode(const std::string &id)
      : m_id(id){}
    MedNode(const MedNode& mn)
      : m_id(mn.m_id){}
    virtual ~MedNode(){}
    const std::string Id()const{return m_id;}

    virtual int Conf(conf_type*const cfg)=0;
    virtual void SetSender(sender_ptr sp){
      m_sender=sp;
    }
    virtual int Start()=0;
    virtual int Exec(event_ptr evnt)=0;
    virtual int Term()=0;
  protected:
    sender_ptr m_sender;
  private:
    std::string m_id;
  };
};


#endif
