/********************************************************************
 * OpenAether Framework
 * Contains objects to aid in the development of applications
 * mainly focusing on test applications and daemons
 *
 * Copyright (C) 2002 Justin Kirby
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Justin Kirby <justink@rochester.rr.com>
 *
 *******************************************************************/
#ifndef FRAMEWORK_MEDIATOR_HPP
#define FRAMEWORK_MEDIATOR_HPP
#include "framework/parser_cmdline.hpp"
#include "framework/fwis/router.hpp"
namespace oa
{
  template<typename event_t,
	   typename node_t,
	   typename factory_t>
  class AppMed
  {
  public:
    enum SendResult{SR_Handled=0,SR_NotHandled,SR_Unknown,SR_Error};
    typedef boost::shared_ptr<node_t> node_ptr;
  public:
    /**
     * Initializes the mediator
     * This parses the arguments passed in from main
     * and creates a config
     * \post A Config (m_cmdLinde) object containing the arguments
     * @param argc The argc from main, or number of elements in argv
     * @param argv The argv from main, or array of params
     */
    void Initialize(int argc,char*argv[]){
      CmdLine cmdLine;
      cmdLine.Source(argc,argv);
      m_cmdLine.reset(cmdLine.Parse());
    }

    /**
     * 
     */
    void Run()
    {
      /* create a factory and use it*/
      factory_t localFactory;
      localFactory.Create(m_cmdLine);
      /* get the router from factory*/
      m_router = localFactory.GetRouter();
      m_router->Start();
    }
    void Send(typename factory_t::event_ptr ep, const std::string &ndid){
      m_router->Send(ep,ndid);
    }
    void Terminate(){
      m_router->Term();
    }

  private:
    boost::shared_ptr<Config> m_cmdLine;
    typename factory_t::router_ptr m_router;
  };
};


#endif
