/********************************************************************
 * OpenAether Framework
 * Contains objects to aid in the development of applications
 * mainly focusing on test applications and daemons
 *
 * Copyright (C) 2002 Justin Kirby
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Justin Kirby <justink@rochester.rr.com>
 *
 *******************************************************************/
#ifndef FWIS_ROUTER_DIRECT_HPP
#define FWIS_ROUTER_DIRECT_HPP
#include "framework/fwis/router.hpp"
#include <map>
namespace oa
{
  template<typename event_t,
	   typename node_t>
  class RouterDirect : public Router<event_t,node_t>
  {
  public:
    typedef typename Router<event_t,node_t>::event_type event_type;
    typedef typename Router<event_t,node_t>::event_ptr event_ptr;
    typedef typename Router<event_t,node_t>::node_type node_type;
    typedef typename Router<event_t,node_t>::node_ptr node_ptr;
    typedef std::map<std::string,node_ptr> node_cont;
  public:
    RouterDirect(){}
    ~RouterDirect(){}
    void AddNode(node_ptr nd, bool primary=false){
      m_nodes.insert(std::make_pair(nd->Id(),nd));
      if(primary){
	m_primary=nd;
      }
    }
    void Send(event_ptr ep, const std::string& str){
      typename node_cont::iterator fnd=m_nodes.find(str);
      if(fnd!=m_nodes.end()){
	fnd->second->Exec(ep);
      }
    }
    node_ptr Primary()const{
      return m_primary;
    }
    void Start(){
      typename node_cont::iterator pos=m_nodes.begin();
      for(;pos!=m_nodes.end();++pos){
	pos->second->Start();
      }
    }
    void Term(){
      typename node_cont::iterator pos=m_nodes.begin();
      for(;pos!=m_nodes.end();++pos){
	pos->second->Term();
      }
    }
  private:
    node_cont m_nodes;
    node_ptr m_primary;
  };
};


#endif
