/********************************************************************
 * OpenAether Framework
 * Contains objects to aid in the development of applications
 * mainly focusing on test applications and daemons
 *
 * Copyright (C) 2002 Justin Kirby
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Justin Kirby <justink@rochester.rr.com>
 *
 *******************************************************************/
#ifndef FWIS_THREAD_NODE_HPP
#define FWIS_THREAD_NODE_HPP
#include <boost/smart_ptr.hpp>
#include <boost/thread/thread.hpp>
#include "framework/fwis/med_node.hpp"
#include "framework/fwis/thread_proxy.hpp"

//iostream include is temporary
#include <iostream>
namespace oa
{
  template<typename event_t>
  class ThreadNode : public MedNode<event_t>
  {
  public:
    typedef typename MedNode<event_t>::event_type event_type;
    typedef typename MedNode<event_t>::event_ptr event_ptr;
    typedef MedNode<event_t> base_type;
    typedef boost::shared_ptr<base_type> base_ptr;

  public:
    ThreadNode(base_ptr bs)
      : MedNode<event_t>(bs->Id()),
	m_base(bs){}
    ThreadNode(const ThreadNode& tn)
      : MedNode<event_t>(tn),
	m_base(tn.m_base),
	m_proxy(tn.m_proxy),
	m_thread(tn.m_thread){}
    virtual ~ThreadNode(){}

    virtual int Conf(typename base_type::conf_ptr cfg){return 0;}
    virtual int Exec(event_ptr evnt){
      if(!m_proxy.get()){
	m_proxy.reset(new oa::ThreadProxy<event_ptr>);
      }
      if(!m_thread.get()){
	std::cout<<"starting thread"<<std::endl;
	m_thread.reset(new boost::thread(*this));
      }
      m_proxy->Push(evnt);
      return 0;
    }
    virtual int Term(){return 0;}
    void operator()(){
      event_ptr evnt;
      while(m_proxy->Get(evnt)){
	m_base->Exec(evnt);
      }
    }
  private:
    base_ptr m_base;
    boost::shared_ptr<oa::ThreadProxy<event_ptr> > m_proxy;
    boost::shared_ptr<boost::thread> m_thread;

  };
};
#endif
