/********************************************************************
 * OpenAether Framework
 * Contains objects to aid in the development of applications
 * mainly focusing on test applications and daemons
 *
 * Copyright (C) 2002 Justin Kirby
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Justin Kirby <justink@rochester.rr.com>
 *
 *******************************************************************/
/*******************************************************************
 * CHANGELOG
 * 2003-03-09
 * - added Source(argc,argv)
 *******************************************************************/
#ifdef WIN32
#include "framework/first.h"
#endif
#include "framework/stringutil.hpp"
#include "framework/parser_cmdline.hpp"

namespace oa
{
  int CmdLine::Source(int argc, char*argv[]){
    CmdLine::strvec args;
    for(int i=0;i<argc;++i){
      args.push_back(argv[i]);
    }
    return Source(args);
  }
  int CmdLine::Source(const std::string &src){
    m_source = src;
    return 0;
  }
  int CmdLine::Source(CmdLine::strvec &src)
  {
    m_vecsrc = src;
    return 0;
  }
  Config* CmdLine::Parse()
  {
    std::auto_ptr<Config> pcfg(new Config("cmdline"));
    if(!m_source.empty()){
      oa::StringUtil().split(m_source," ",m_vecsrc);
    }

    oa::StringUtil::vecstr::iterator pos;
    for(pos=m_vecsrc.begin();pos!=m_vecsrc.end();++pos){
      CmdLine::strpr sp = Tokenize(*pos);
      pcfg->Set(sp.first,sp.second);
    }
    return pcfg.release();
  }
 CmdLine::strpr CmdLine::Tokenize(const std::string &tok)
 {
   //is it a parameter?
   if(tok.find(m_paramToken)){
     oa::StringUtil::vecstr toked;
     oa::StringUtil().split(tok,m_paramToken,toked);
     return std::make_pair(*toked.begin(),(*(toked.end()-1)));
   }else{
       return std::make_pair(tok,std::string(""));
   }
 }
};
