/********************************************************************
 * OpenAether Framework
 * Contains objects to aid in the development of applications
 * mainly focusing on test applications and daemons
 *
 * Copyright (C) 2002 Justin Kirby
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Justin Kirby <justink@rochester.rr.com>
 *
 *******************************************************************/
#ifndef FRAMEWORK_PARSER_XML_HPP
#define FRAMEWORK_PARSER_XML_HPP
#include "framework/framework_exports.h"
#include "framework/config_parser.hpp"
#include <xercesc/dom/DOMNode.hpp>
#include <xercesc/sax/InputSource.hpp>

namespace oa
{
#ifdef WIN32
    class FW_API XMLParser;
#endif
  /**
   * Parses and xml file designed for configuration purposes
   *
   * 
   */
  class XMLParser : public Parser
  {
  public:
    XMLParser(){}
    virtual ~XMLParser(){}
    /**
     * Set the filename to open and parse
     * @param src Filename to parse
     * @return always returns zero
     */
    virtual int Source(const std::string &src);
    /**
     * Parse the source (xml file)
     * This uses xercesC to parse the xml file 
     * It builds the ConfigChain from key/values in the xml
     *
     * @return A pointer to the Config generated
     */
    virtual Config* Parse();
  protected:
    typedef std::pair<std::string,std::string> strpr;

    virtual ConfigChain* Parse(xercesc::DOMNode *const nd);
    virtual bool IsKey(xercesc::DOMNode *const nd);
    virtual bool IsParam(xercesc::DOMNode *const nd);
    virtual void DoAttrs(Config *cfg, xercesc::DOMNode *const nd);
    virtual strpr GetParam(xercesc::DOMNode *const nd);
    virtual int XercesHack(xercesc::DOMNode *const nd);
  protected:
    std::string m_tempFile;
    std::string m_file;
  };
};
#endif
