/********************************************************************
 * OpenAether Framework
 * Contains objects to aid in the development of applications
 * mainly focusing on test applications and daemons
 *
 * Copyright (C) 2002 Justin Kirby
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Justin Kirby <justink@rochester.rr.com>
 *
 *******************************************************************/
#ifndef FRAMEWORK_SIGSYNC_HPP
#define FRAMEWORK_SIGSYNC_HPP
#include "oapr/oapr_pool_mgr.hpp"
namespace oa
{
  template<typename ThreadPol>
  class SigSync
  {
  public:
    SigSync()
      : m_signalled(false),
	m_received(false),
	m_mutex(oapr::pool_mgr::instance()->unique()),
	m_cond(oapr::pool_mgr::instance()->unique()){}

    ~SigSync(){}

    void Signal(){
      typename ThreadPol::lock_type lock(m_mutex);
      m_signalled=true;
      while(!m_received){
	m_cond.wait(m_mutex);
      }
    }
    bool Signalled()const{return m_signalled;}
    void Receive(){
      typename ThreadPol::lock_type lock(m_mutex);
      m_received=true;
      m_cond.signal();
    }
    bool Received()const;

  private:
    bool m_signalled;
    bool m_received;
    typename ThreadPol::mutex m_mutex;
    typename ThreadPol::condition m_cond;
  };
};
#endif
