/********************************************************************
 * OpenAether Framework
 * Contains objects to aid in the development of applications
 * mainly focusing on test applications and daemons
 *
 * Copyright (C) 2002 Justin Kirby
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Justin Kirby <justink@rochester.rr.com>
 *
 *******************************************************************/
#ifdef WIN32
#include "framework/first.h"
#endif
#include "framework/stringutil.hpp"

namespace oa
{
  int StringUtil::split(const std::string &src, 
					const std::string &delim,
					StringUtil::vecstr &lines)
  {
    std::string line;
    std::string::size_type begI,endI;

    begI = src.find_first_not_of(delim);
    endI = src.find_first_of(delim);
    while(begI != std::string::npos){
      if(endI == std::string::npos){
	endI = src.length();
      }
      line = src.substr(begI,(endI-begI));
      begI = src.find_first_not_of(delim,endI);
      endI = src.find_first_of(delim,begI);
      if(!line.empty()){
	lines.push_back(line);
      }
    }
    return lines.size();
  }
  int StringUtil::split(const std::string &src,
			const std::string &delim,
			vecstr &lines,
			int max)
  {
    std::string line;
    std::string::size_type begI,endI;
    begI = src.find_first_not_of(delim);
    endI = src.find_first_of(delim);
    for(int idx=0; idx<max && begI != std::string::npos; idx++){
      if(endI == std::string::npos){
	endI = src.length();
      }
      line = src.substr(begI,(endI-begI));
      begI = src.find_first_not_of(delim,endI);
      endI = src.find_first_of(delim,begI);
      if(!line.empty()){
	lines.push_back(line);
      }
    }
    return lines.size();
  }
  bool StringUtil::wtos(const std::wstring &wstr, std::string &str)
  {
      std::wstring::const_iterator wpos;
      str.clear();
      for(wpos=wstr.begin();wpos!=wstr.end(); ++wpos){
          str+=static_cast<char>(*wpos);
      }
      return true;
  }
  bool StringUtil::stow(const std::string &str, std::wstring &wstr)
  {
      std::string::const_iterator spos;
      wstr.clear();
      for(spos=str.begin();spos!=str.end(); ++spos){
          wstr+=static_cast<wchar_t>(*spos);
      }
      return true;
  }
};
