/********************************************************************
 * OpenAether Framework
 * Contains objects to aid in the development of applications
 * mainly focusing on test applications and daemons
 *
 * Copyright (C) 2002 Justin Kirby
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Justin Kirby <justink@rochester.rr.com>
 *
 *******************************************************************/
#include "framework/first.h"
#include "openaether.hpp"
#include <iostream>
#include <boost/smart_ptr.hpp>
#include <xercesc/util/PlatformUtils.hpp>
#include <xercesc/sax2/XMLReaderFactory.hpp>
#include "framework/xmlutil.hpp"
#include "framework/dombuilder.hpp"
#include <xercesc/util/XMLString.hpp>

XERCES_CPP_NAMESPACE_USE

class StrX
{
public:
  StrX(const XMLCh* const toTranscode){
    m_local=XMLString::transcode(toTranscode);
  }
  ~StrX(){
    delete [] m_local;
  }
  const char* local()const{
    return m_local;
  }
private:
  char* m_local;
};
std::ostream& operator<<(std::ostream& target, const StrX& toDump){
  target<<toDump.local();
  return target;
}


const char* xmlFile=0;

int main(int argc, char* argv[])
{
  try{
    XMLPlatformUtils::Initialize();
  }catch(const XMLException &e){
    std::cerr<<"ERROR during init:\n"
	     << StrX(e.getMessage())<<std::endl;
  }
  if(argc<2){
    std::cerr<<"ERROR, need file\n";
    XMLPlatformUtils::Terminate();
    return 1;
  }
  xmlFile=argv[1];


  boost::scoped_ptr<SAX2XMLReader> parser(XMLReaderFactory::createXMLReader());
  /*
  parser->setValidationScheme(SAXParser::Val_Never);
  parser->setDoNamespaces(true);
  parser->setDoSchema(false);
  */

  oa::DomBuilder builder;
  try{
    parser->setContentHandler(&builder);
    parser->setErrorHandler(&builder);
    parser->parse(xmlFile);


    std::string xmlout;
    oa::XMLUtils().Print(xmlout,builder.getElement());
    std::cout<<xmlout.c_str()<<std::endl;
  }catch(const XMLException& e){
    std::cerr<< "\nERROR: "<<StrX(e.getMessage())<<std::endl;
  }
  XMLPlatformUtils::Terminate();
  return 0;
}
