/********************************************************************
 * OpenAether Framework
 * Contains objects to aid in the development of applications
 * mainly focusing on test applications and daemons
 *
 * Copyright (C) 2002 Justin Kirby
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Justin Kirby <justink@rochester.rr.com>
 *
 *******************************************************************/
#include "framework/fifo_queue.hpp"
#include "framework/thread_pthread.hpp"
#include <boost/smart_ptr.hpp>
#include <iostream>


typedef oa::Fifo<int,oa::Pthread> fifo_type;
typedef boost::shared_ptr<fifo_type> fifo_ptr;

class pusher
{
public:
  pusher(fifo_ptr& fifo, int cnt)
    : m_fifo(fifo),
      m_count(cnt){}
  pusher(const pusher& p)
    : m_fifo(p.m_fifo),
      m_count(p.m_count){}
  void operator()(){

    for(int i=0;i<m_count;++i){
      std::cout<<"IN: "<<i<<std::endl;
      m_fifo->push(i);
    }
  }
private:
  fifo_ptr m_fifo;
  int m_count;
};

class getter
{
public:
  getter(fifo_ptr& fifo, const std::string &nm)
    : m_fifo(fifo),m_name(nm){}
  getter(const getter& g)
    : m_fifo(g.m_fifo),m_name(g.m_name){}
  void operator()(){
    std::cout<<"NAME: "<<m_name<<std::endl;
    int o;
    while(m_fifo->get(o)){
      std::cout<<m_name<<":"<<o<<std::endl;
    }
    std::cout<<"EXITING from "<<m_name<<std::endl;
  }
private:
  fifo_ptr m_fifo;
  std::string m_name;
};
struct pusher_arg{
  fifo_ptr fifo;
  int cnt;
};
void* pusher_func(void* arg){
  pusher_arg* pa = (pusher_arg*)arg;
  pusher psh(pa->fifo,pa->cnt);
  psh();
  return 0;
}
struct getter_arg{
  fifo_ptr fifo;
  std::string nm;
};
void* getter_func(void* arg){
  getter_arg* ga=(getter_arg*)arg;
  getter get(ga->fifo,ga->nm);
  get();
  return 0;
}
int main(int argc,char*argv[])
{
  fifo_ptr fifo(new fifo_type);
  pusher_arg psh;
  psh.fifo=fifo;
  psh.cnt=100;

  getter_arg get1,get2;
  get1.fifo=fifo;
  get1.nm="one";
  get2.fifo=fifo;
  get2.nm="two";


  pthread_t psh1t,psh2t,get1t,get2t;
  pthread_create(&psh1t,NULL,pusher_func,(void*)&psh);
  pthread_create(&get1t,NULL,getter_func,(void*)&get1);
  pthread_create(&get2t,NULL,getter_func,(void*)&get2);
  pthread_create(&psh2t,NULL,pusher_func,(void*)&psh);
  std::cerr<<"join psh1t\n";
  pthread_join(psh1t,NULL);
  std::cerr<<"join psh2t\n";
  pthread_join(psh2t,NULL);
  fifo->signal();
  std::cerr<<"join get1t\n";
  pthread_join(get1t,NULL);
  std::cerr<<"join get2t\n";
  pthread_join(get2t,NULL);
  return 0;
}

