/********************************************************************
 * OpenAether Framework
 * Contains objects to aid in the development of applications
 * mainly focusing on test applications and daemons
 *
 * Copyright (C) 2004 Justin Kirby
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Justin Kirby <justink@rochester.rr.com>
 *
 *******************************************************************/
#include "framework/xmlfilters.hpp"
#include <xercesc/dom/DOMNodeFilter.hpp>
#include <xercesc/dom/DOMTreeWalker.hpp>
#include <xercesc/util/XMLString.hpp>
#include "framework/dom_ptr.hpp"

namespace oa
{
  class NsFilter : public xercesc::DOMNodeFilter
  {
  public:
    NsFilter(const XMLCh*const ns)
      : m_ns(ns){}
    ~NsFilter(){}
    short acceptNode(const xercesc::DOMNode* node)const
    {
      const XMLCh* ns=node->getNamespaceURI();
      if(xercesc::XMLString::compareString(ns,m_ns) == 0){
	return FILTER_ACCEPT;
      }else{
	return FILTER_REJECT;
      }
    }
  private:
    const XMLCh*const m_ns;
  };

  bool DomFilterNs::HasNs()
  {
    if(m_doc == 0  || m_root == 0 || m_ns == 0){
      return false;
    }
    NsFilter nsf(m_ns);

    oa::dom_ptr<xercesc::DOMTreeWalker> walker(m_doc->createTreeWalker(m_root,xercesc::DOMNodeFilter::SHOW_ELEMENT,&nsf,false));

    if(walker->firstChild() == 0){
      return true;
    }else{
      return false;
    }

  }
};
