/********************************************************************
 * OAjabber library
 * Contains objects to aid in Jabber client development and extend the
 * basic functionality of jabber.
 *
 * Copyright (C) 2001 Justin Kirby
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Justin Kirby <justink@rochester.rr.com>
 *
 *******************************************************************/
#include "framework/first.h"
#include "openaether.hpp"
#include <xercesc/util/XMLString.hpp>
#include <xercesc/dom/DOMDocumentFragment.hpp>
#include <xercesc/dom/DOMElement.hpp>
#include "framework/xerces_xmlns.hpp"
#include "framework/xmlutil.hpp"
#include "framework/dom_ptr.hpp"
#include "framework/xmlstr.hpp"
#include "xstrm/jabjid.h"
#include "jabber/ijabagent.h"

XERCES_CPP_NAMESPACE_USE
namespace jabber
{
  DOMNode*const JabAgent::BuildXml()
  {
    //    boost::scoped_array<XMLCh> tagName,tagValue,attrName,attrValue;
    std::string type,id,to,from;
    std::string jid,name,desc,service;

    oa::xmlstr<XMLCh,char> tagName, tagValue, attrName,attrValue;

    DOMDocument *doc=oa::XMLUtils().Document(DocUri(),"oa");
    oa::dom_ptr<DOMNode> iqElem(BuildIq());

    /* create the main query element
     * putting it in default namespace of Xmlns()
     */
    tagName.set("query");
    tagValue.set(Xmlns().c_str());
    oa::dom_ptr<DOMElement> queryElem(doc->createElementNS(tagValue,
							   tagName));

    /* adding the xmlns attribute
     * cause otherwise things have a hissy fit
     */
    oa::XercesXmlns()(queryElem.get(),tagValue);
    //    queryElem->setAttribute(attrName.get(),attrValue.get());

    /**\todo move this out somewhere to someplace for someone*/
    tagValue.set("http://www.w3.org/XML/1998/namespace");


    if(Type()=="result"){
      queryElem->appendChild(BuildProperties());
    }


    /* appending the now possibly huge query elem*/
    iqElem->appendChild(queryElem.release());
    return iqElem.release();
  }
  DOMNode*const JabAgent::BuildProperties()
  {
    std::string jid,name,desc,service;
    //    boost::scoped_array<XMLCh> tagName,tagValue,attrName,attrValue;
    oa::xmlstr<XMLCh,char> tagName,tagValue,attrName,attrValue;

    DOMDocument *doc=oa::XMLUtils().Document(DocUri(),"oa");
    oa::dom_ptr<DOMDocumentFragment> fdoc(doc->createDocumentFragment());

    /* get the name, and set it if there*/
    name=Name();
    if(!name.empty()){
      tagName.set("name");
      tagValue.set(name.c_str());
      oa::dom_ptr<DOMElement> nameElem(doc->createElement(tagName));
      oa::dom_ptr<DOMText> nameValue(doc->createTextNode(tagValue));
      nameElem->appendChild(nameValue.release());
      fdoc->appendChild(nameElem.release());				       
    }
      
    /* descripting... get and set if there*/
    desc=Description();
    if(!desc.empty()){
      tagName.set("description");
      tagValue.set(desc.c_str());
      oa::dom_ptr<DOMElement> descElem(doc->createElement(tagName));
      oa::dom_ptr<DOMText> descValue(doc->createTextNode(tagValue));
      descElem->appendChild(descValue.release());
      fdoc->appendChild(descElem.release());
    }
      
    /* transport is a bool flag 
     * represented by empty elem <transport/>
     */
    if(Transport()){
      tagName.set("transport");
      oa::dom_ptr<DOMElement> transElem(doc->createElement(tagName));
      fdoc->appendChild(transElem.release());
    }
    
    /* groupchat is a bool flag
     * true is represented by a elem <groupchat/>
     */
    if(GroupChat()){
      tagName.set("groupchat");
      oa::dom_ptr<DOMElement> gcElem(doc->createElement(tagName));
      fdoc->appendChild(gcElem.release());
    }
    
    /* service... get and set*/
    service=Service();
    if(!service.empty()){
      tagName.set("service");
      tagValue.set(service.c_str());
      oa::dom_ptr<DOMElement> servElem(doc->createElement(tagName));
      oa::dom_ptr<DOMText> servValue(doc->createTextNode(tagValue));
      servElem->appendChild(servValue.release());
      fdoc->appendChild(servElem.release());
    }
      
    /* register is a bool flag
     * indicated by <register/> == true
     */
    if(Register()){
      tagName.set("register");
      oa::dom_ptr<DOMElement> regElem(doc->createElement(tagName));
      fdoc->appendChild(regElem.release());
    }
      
    /* search is a bool flag
     * indicated by <search/> ==true
     */
    if(Search()){
      tagName.set("search");
      oa::dom_ptr<DOMElement> srchElem(doc->createElement(tagName));
      fdoc->appendChild(srchElem.release());
    }
    return fdoc.release();
  }

  /**************************************
   **************************************/
  JabAgents::JabAgents()
    : JabIq("agents")
  {}
  JabAgents::~JabAgents(){}

  int JabAgents::Length(){
    return m_agents.size();
  }
  JabAgent*const JabAgents::Item(int i){
      return m_agents[i].get();
  }
  int JabAgents::Insert(JabAgent *agent){
    if(!agent){
      return 1;
    }
    agent_ptr eagent(agent);
    m_agents.push_back(eagent);
    return 0;
  }

  DOMNode*const JabAgents::BuildXml()
  {

    //boost::scoped_array<XMLCh> tagName,tagValue,attrName,attrValue;
    oa::xmlstr<XMLCh,char> tagName,tagValue,attrName,attrValue;
    DOMDocument *doc = oa::XMLUtils().Document(DocUri(),"oa");
    
    oa::dom_ptr<DOMNode> iqElem(BuildIq());

    tagName.set("query");
    tagValue.set(Xmlns().c_str());
    oa::dom_ptr<DOMElement> queryElem(doc->createElementNS(tagValue,tagName));

    oa::XercesXmlns()(queryElem.get(),tagValue);
    //    queryElem->setAttribute(attrName,attrValue);
    /**\todo move this out somewhere to someplace for someone*/
    tagValue.set("http://www.w3.org/XML/1998/namespace");



    oa::dom_ptr<DOMElement> agentElem;
    
    vec_agents::iterator pos=m_agents.begin();
    for(;pos!=m_agents.end();++pos){
      tagName.set("agent");
      agentElem.reset(doc->createElement(tagName));

      attrName.set("jid");
      attrValue.set((*pos)->Jid().c_str());
      agentElem->setAttribute(attrName,attrValue);

      agentElem->appendChild((*pos)->BuildProperties());
      queryElem->appendChild(agentElem.release());
    }

    iqElem->appendChild(queryElem.release());
    /*
    m_node=iqElem.release();
    return m_node.get();
    */
    return iqElem.release();
  }
};
