/* -*- Mode: c++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 * OAjabber library
 * Contains objects to aid in Jabber client development and extend the
 * basic functionality of jabber.
 *
 * Copyright (C) 2001 Justin Kirby
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Justin Kirby <justink@rochester.rr.com>
 *
 *******************************************************************/
#ifndef OAJABBER_IJABIQ_H
#define OAJABBER_IJABIQ_H
#include "jabber/oajabber_exports.h"
#include "xstrm/jabpacket.h"

namespace jabber
{
  class OAJABBER_API JabIq;
  /** @ingroup gJabPacket
   */

  /**
   * JabIq is the primary object created by 
   * receiving an <iq> node from the jabber
   * server. It has minimal functionality.
   * It should be derived in order to
   * add any functionality required.
   * @author Justin Kirby <justink@rochester.rr.com>
   */
  class JabIq : public JabPacket
  {
  public:
    JabIq(std::string classid);
    JabIq(const JabIq &ji)
      : JabPacket(ji){}
    virtual ~JabIq();

    virtual const char*const Uri()const{
      return "xmpp:iq";
    }
    using JabPacket::Id;
    virtual std::string Id()const;

    virtual std::string xml();
    virtual xercesc::DOMNode*const BuildXml(){
      return BuildIq();
    }
  protected:
    virtual xercesc::DOMNode* BuildIq();
    virtual xercesc::DOMNode* BuildError();
  };
};
#endif
