/********************************************************************
 * OAjabber library
 * Contains objects to aid in Jabber client development and extend the
 * basic functionality of jabber.
 *
 * Copyright (C) 2002 Justin Kirby
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Justin Kirby <justink@rochester.rr.com>
 *
 *******************************************************************/
#include "framework/first.h"
#include "openaether.hpp"
#include <boost/smart_ptr.hpp>
#include <xercesc/util/XMLString.hpp>
#include <xercesc/dom/DOMElement.hpp>
#include "oapr/oapr_exception.hpp"
#include "oapr/oapr_exc_utils.hpp"
#include "framework/dom_ptr.hpp"
#include "framework/xmlstr.hpp"
#include "framework/xmlutil.hpp"
#include "framework/xerces_xmlns.hpp"
#include "jabber/ijablast.h"

XERCES_CPP_NAMESPACE_USE
namespace jabber
{


  DOMNode*const JabLast::BuildXml()
  {
    //    boost::scoped_array<XMLCh> tagName,tagValue,attrName,attrValue;
    oa::xmlstr<XMLCh,char> tagName,tagValue,attrName,attrValue;
    DOMDocument *doc=oa::XMLUtils().Document(DocUri(),"oa");

    /* create the query element in the namespace
     */
    tagName.set("query");
    tagValue.set(Xmlns().c_str());
    oa::dom_ptr<DOMElement> queryElem(doc->createElementNS(tagValue,
							   tagName));

    /* force the xmlns attribute to exist
     */
    //    attrName.reset(XMLString::transcode("xmlns"));
    attrValue.set(Xmlns().c_str());
    oa::XercesXmlns()(queryElem.get(),attrValue);
    //    queryElem->setAttribute(attrName.get(),attrValue.get());

    std::string type=Type();
    if(type == "result"){
      /* only if the type is result
       * seconds is required to have a value
       */
      std::string seconds=Seconds();
      if(seconds.empty()){
	OAPR_THROW(oapr::logic_error,"Required query attribute seconds is empty");
      }

      attrName.set("seconds");
      attrValue.set(seconds.c_str());
      queryElem->setAttribute(attrName.get(),attrValue.get());
    }

    /* now we build the iq
     */
    oa::dom_ptr<DOMNode> iqElem(BuildIq());
    iqElem->appendChild(queryElem.release());

    return iqElem.release();
  }



};
