/* -*- Mode: c++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 * OAjabber library
 * Contains objects to aid in Jabber client development and extend the
 * basic functionality of jabber.
 *
 * Copyright (C) 2002 Justin Kirby
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Justin Kirby <justink@rochester.rr.com>
 *
 *******************************************************************/
#ifndef OAJABBER_IJABLAST_H
#define OAJABBER_IJABLAST_H
#include "jabber/ijabiq.h"

namespace jabber
{
  class OAJABBER_API JabLast;

  /** @ingroup gJabPacket
   */

  ///Implementation of jabber:iq:last
  /**
   * Basically a really simple implementation
   * of the jabber:iq:last
   */
  class JabLast : public JabIq
  {
  public:
    ///ctor
    JabLast()
      : JabIq("last"){}
    ///copy ctor
    JabLast(const JabLast& jl)
      : JabIq(jl),
	m_seconds(jl.m_seconds){}
    ///dtor
    virtual ~JabLast(){}

    virtual const char*const Uri()const{
      return "xmpp:xmlns=jabber:iq:last";
    }
    ///Get the xml namespace
    /**
     * Used to get the xml namespace that this implements
     * this is jabber:iq:last
     * @return xml namespace of jabber:iq:last
     */
    virtual std::string Xmlns()const{
      return "jabber:iq:last";
    }
    ///Set the seconds
    /**
     * Set the value which is currently only a seconds
     * attribute.
     * @param sec val seconds
     */
    virtual void Seconds(const std::string& sec){m_seconds=sec;}
    ///Get the value
    /**
     * @return the value
     */
    virtual std::string Seconds()const{return m_seconds;}


    ///Get the xml in DOM form
    virtual xercesc::DOMNode*const BuildXml();
  protected:
    std::string m_seconds;
  };
};


#endif
