/********************************************************************
 * OAjabber library
 * Contains objects to aid in Jabber client development and extend the
 * basic functionality of jabber.
 *
 * Copyright (C) 2002 Justin Kirby
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Justin Kirby <justink@rochester.rr.com>
 *
 *******************************************************************/
#include "framework/first.h"
#include "openaether.hpp"

#include <boost/smart_ptr.hpp>
#include "framework/xmlutil.hpp"
#include "jabber/ijabpres.h"

#include <xercesc/util/XMLString.hpp>
#include <xercesc/dom/DOMElement.hpp>

XERCES_CPP_NAMESPACE_USE
namespace jabber
{
  std::string JabPresence::xml()
  {
    std::string ret;
    oa::dom_ptr<DOMNode> built(BuildXml());
    if(built.get()){
      oa::XMLUtils().Print(ret,built.get());
    }
    return ret;
  }
  DOMNode*const JabPresence::BuildXml()
  {
    boost::scoped_array<XMLCh> tagName,tagValue,attrName,attrValue;
    JabJid to,from;
    std::string id,type,show,status,priority;
    DOMDocument *doc = oa::XMLUtils().Document(DocUri(),"oa");

    tagName.reset(XMLString::transcode("presence"));
    DOMElement *presElem = doc->createElement(tagName.get());

    /* now we only build elements that are there
     * The protocol does not require ANY other attributes or elements
     */
    /* deal with the to jid*/
    to=To();
    if(!to.IsNull()){
      attrName.reset(XMLString::transcode("to"));
      attrValue.reset(XMLString::transcode(to.Jid().c_str()));
      DOMAttr *toAttr = doc->createAttribute(attrName.get());
      toAttr->setValue(attrValue.get());
      presElem->setAttributeNode(toAttr);
    }
    /* deal with the from jid*/
    /*
    from=From();
    if(!from.empty()){
      attrName.reset(XMLString::transcode("from"));
      attrValue.reset(XMLString::transcode(from.c_str()));
      DOMAttr *fromAttr = doc->createAttribute(attrName.get());
      fromAttr->setValue(attrValue.get());
      presElem->setAttributeNode(fromAttr);
    }
    */
    /* deal with the id attribute */
    id=Id();
    if(!id.empty()){
      attrName.reset(XMLString::transcode("id"));
      attrValue.reset(XMLString::transcode(id.c_str()));
      DOMAttr *idAttr = doc->createAttribute(attrName.get());
      idAttr->setValue(attrValue.get());
      presElem->setAttributeNode(idAttr);
    }
    /* deal with the type attribute */
    type=Type();
    if(!type.empty()){
      /* if type is of error then add the error element*/
      if(HasError()){
	type="error";

	JabError error=Error();
	tagName.reset(XMLString::transcode("error"));	
	tagValue.reset(XMLString::transcode(error.Message().c_str()));
	attrName.reset(XMLString::transcode("code"));
	attrValue.reset(XMLString::transcode(error.Code().c_str()));
	DOMElement *errorElem = doc->createElement(tagName.get());
	DOMText *errorValue = doc->createTextNode(tagValue.get());
	errorElem->appendChild(errorValue);

	DOMAttr *codeAttr = doc->createAttribute(attrName.get());
	codeAttr->setValue(attrValue.get());
	errorElem->setAttributeNode(codeAttr);
	presElem->appendChild(errorElem);
      }
      attrName.reset(XMLString::transcode("type"));
      attrValue.reset(XMLString::transcode(type.c_str()));      
      DOMAttr *typeAttr = doc->createAttribute(attrName.get());
      typeAttr->setValue(attrValue.get());
      presElem->setAttributeNode(typeAttr);
    }

    /* Now we have to deal with the child elements*/
    show=Show();
    if(!show.empty()){
      tagName.reset(XMLString::transcode("show"));
      DOMElement *showElem = doc->createElement(tagName.get());

      tagValue.reset(XMLString::transcode(show.c_str()));
      DOMText *showValue = doc->createTextNode(tagValue.get());

      showElem->appendChild(showValue);
      presElem->appendChild(showElem);
    }

    status=Status();
    if(!status.empty()){
      tagName.reset(XMLString::transcode("status"));
      DOMElement *statusElem=doc->createElement(tagName.get());
      
      tagValue.reset(XMLString::transcode(status.c_str()));
      DOMText *statusValue = doc->createTextNode(tagValue.get());

      statusElem->appendChild(statusValue);
      presElem->appendChild(statusElem);
    }
    priority=Priority();
    if(!priority.empty()){
      tagName.reset(XMLString::transcode("priority"));
      DOMElement *priElem=doc->createElement(tagName.get());

      tagValue.reset(XMLString::transcode(priority.c_str()));
      DOMText *priValue = doc->createTextNode(tagValue.get());

      priElem->appendChild(priValue);
      presElem->appendChild(priElem);
    }
    return presElem;
  }
};
