/********************************************************************
 * OAjabber library
 * Contains objects to aid in Jabber client development and extend the
 * basic functionality of jabber.
 *
 * Copyright (C) 2001 Justin Kirby
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Justin Kirby <justink@rochester.rr.com>
 *
 *******************************************************************/
#ifndef OAJABBER_IJABPRES_H
#define OAJABBER_IJABPRES_H
#include "jabber/oajabber_exports.h"
#include "xstrm/jabpacket.h"

namespace jabber
{
  class OAJABBER_API JabPresence;

  /** @ingroup gJabPacket
   */
  ///Presence management
  /**
   * JabPresence manages the resource's
   * presence on the jabber server. It
   * handles basic subscription and reception
   * of subscription events.
   * @author Justin Kirby <justink@rochester.rr.com>
   */
  class JabPresence : public JabPacket
  {
  public:
    JabPresence()
      : JabPacket("presence","presence")
    {}
    JabPresence(const JabPresence &jp)
      : JabPacket(jp),
      m_priority(jp.m_priority),
      m_status(jp.m_status),
      m_show(jp.m_show)
      {}
    virtual ~JabPresence(){}
    virtual const char*const Uri()const{
      return "xmpp:presence";
    }
    virtual std::string Show()const{
      return m_show;
    }
    virtual std::string Status()const{
      return m_status;
    }
    virtual std::string Priority()const{
      return m_priority;
    }
    virtual void Show(const std::string& show){
      m_show=show;
    }
    virtual void Status(const std::string& status){
      m_status=status;
    }
    virtual void Priority(const std::string& pri){
      m_priority=pri;
    }
    /**
     * Generate the xml from values
     * @return the xml
     */
    virtual std::string xml();
    virtual xercesc::DOMNode*const BuildXml();    
  protected:
    std::string m_priority; //not needed all the time
    std::string m_status;
    std::string m_show;
  };
};
#endif
