/********************************************************************
 * OAjabber library
 * Contains objects to aid in Jabber client development and extend the
 * basic functionality of jabber.
 *
 * Copyright (C) 2001 Justin Kirby
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Justin Kirby <justink@rochester.rr.com>
 *
 *******************************************************************/
#include "framework/first.h"
#include "openaether.hpp"
#include <algorithm>
#include "oapr/oapr_exception.hpp"
#include "framework/dom_ptr.hpp"
#include "framework/xmlstr.hpp"
#include "framework/xmlutil.hpp"
#include "jabber/ijabregister.h"

XERCES_CPP_NAMESPACE_USE
namespace jabber
{
  JabRegister::JabRegister()
    : JabIq("register")
  {}
  JabRegister::~JabRegister(){ }
    
  std::string JabRegister::Instructions(){ 
    return m_instructions;
  }
  bool JabRegister::Remove(){ 
    return m_remove;
  }
  
  void JabRegister::Instructions(const std::string &inst){ 
    m_instructions = inst;
  }
  bool JabRegister::SetValue(const std::string &elem,
			     const std::string &val)
  {
    //is this required? if not should we send it?
    /**\todo answer the above question*/
    //for now just inserting into the map
    m_values.insert(std::make_pair(elem,val));
    return true;
  }

  void JabRegister::Remove(bool rmv){ 
    m_remove = rmv;
  }
  void JabRegister::AddRequired(const std::string &rq)
  {
    //is it in here?
    required_type::iterator pos = std::find(m_required.begin(),
					    m_required.end(),
					    rq);
    if(pos == m_required.end()){
      //if its not there add it
      m_required.push_back(rq);
    }
  }
  bool JabRegister::Required(required_type &rt)
  {
    std::copy(m_required.begin(),
	      m_required.end(),
          std::back_inserter(rt));
    return true;
  }


  DOMNode*const JabRegister::BuildXml()
  {
    oa::dom_ptr<DOMNode> frag;
    if(m_type=="get"){
      frag.reset(BuildGet());
    }else if(m_type=="set"){
      frag.reset(BuildSet());
    }else if(m_type=="result"){
    }else if(m_type=="error"){
    }else{
      OAPR_THROW(oapr::invalid_arg,"iq type is not of get,set,result, or error");
    }

    oa::dom_ptr<DOMNode> iq(BuildIq());
    if(iq.get() && frag.get()){
      iq->appendChild(frag.release());
      return iq.release();
    }else{
      /**@todo throw?*/
      return 0;
    }
  }
  DOMNode*const JabRegister::BuildGet()
  {
    /* build the <query/> and optionally the values*/
    oa::xmlstr<XMLCh,char> xmlnsName("xmlns"),
      xmlnsValue(Xmlns().c_str()),
      queryName("query");
    DOMDocument *doc=oa::XMLUtils().Document(DocUri(),"oa");
    oa::dom_ptr<DOMDocumentFragment> rv(doc->createDocumentFragment());
    oa::dom_ptr<DOMElement> query(doc->createElementNS(xmlnsValue.get(),
						       queryName.get()));
    /* if we have values, translate them into xml element value pairs*/
    rv->appendChild(query.release());
    return rv.release();
  }
  DOMNode*const JabRegister::BuildSet()
  {
    oa::xmlstr<XMLCh,char> xmlnsName("xmlns"),
      xmlnsValue(Xmlns().c_str()),
      queryName("query"),
      keyName,keyValue;

    DOMDocument *doc=oa::XMLUtils().Document(DocUri(),"oa");
    oa::dom_ptr<DOMDocumentFragment> rv(doc->createDocumentFragment());
    oa::dom_ptr<DOMElement> query(doc->createElementNS(xmlnsValue.get(),
						       queryName.get()));
    /* if we have values, translate them into xml element value pairs*/

    value_type::iterator pos=m_values.begin();
    for(;pos!=m_values.end();++pos){
      keyName.set(pos->first.c_str());
      keyValue.set(pos->second.c_str());
      oa::dom_ptr<DOMText> val(doc->createTextNode(keyValue));
      oa::dom_ptr<DOMElement> key(doc->createElement(keyName));
      key->appendChild(val.release());
      query->appendChild(key.release());
    }
    rv->appendChild(query.release());
    return rv.release();
  }
}
