/* -*- Mode: c++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 * OAjabber library
 * Contains objects to aid in Jabber client development and extend the
 * basic functionality of jabber.
 *
 * Copyright (C) 2001 Justin Kirby
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Justin Kirby <justink@rochester.rr.com>
 *
 *******************************************************************/
#ifndef OAJABBER_IJABREGISTER_H
#define OAJABBER_IJABREGISTER_H
#include <vector>
#include <map>
#include "jabber/oajabber_exports.h"
#include "jabber/ijabiq.h"

namespace jabber
{
  class OAJABBER_API JabRegister;

  /** @ingroup gJabPacket
   */

  /**
   * This is to register, e.g. create a account
   * However, its kind of fubarred atm
   */
  class JabRegister : public JabIq
  {
  public:
    typedef std::vector<std::string> required_type;
    typedef std::map<std::string,std::string> value_type;
  public:
    JabRegister();
    JabRegister(const JabRegister &jr)
      : JabIq(jr),
        m_instructions(jr.m_instructions),
        m_remove(jr.m_remove),
        m_required(jr.m_required),
        m_values(jr.m_values){}
    virtual ~JabRegister();

    /**\todo fix this*/
    virtual const char*const Uri()const{
      return "xmpp:xmlns=jabber:iq:register";
    }
    virtual std::string Xmlns()const{
      return "jabber:iq:register";
    }    


    virtual std::string Instructions();
    virtual void Instructions(const std::string &inst);

    virtual bool Remove();
    virtual void Remove(bool rmv);

    virtual bool SetValue(const std::string &type,
			  const std::string &val);

    virtual void AddRequired(const std::string &rq);
    virtual bool Required(required_type &get);


    virtual xercesc::DOMNode*const BuildXml();
  protected:
    virtual xercesc::DOMNode*const BuildGet();
    virtual xercesc::DOMNode*const BuildSet();
  protected:
    std::string m_instructions;
    bool m_remove;
    required_type m_required;
    value_type m_values;
  };
};

#endif
