/********************************************************************
 * OAjabber library
 * Contains objects to aid in Jabber client development and extend the
 * basic functionality of jabber.
 *
 * Copyright (C) 2002 Justin Kirby
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Justin Kirby <justink@rochester.rr.com>
 *
 *******************************************************************/
#include "framework/first.h"
#include "openaether.hpp"
#include <xercesc/util/XMLString.hpp>
#include <xercesc/dom/DOMElement.hpp>
#include "framework/xmlutil.hpp"
#include "framework/dom_ptr.hpp"
#include "framework/xmlstr.hpp"
#include "framework/xerces_xmlns.hpp"
#include "jabber/ijabroster.h"

XERCES_CPP_NAMESPACE_USE
namespace jabber
{

  JabRosterItem*const JabRoster::FindId(std::string &jid)
  {
    cont_type::iterator pos;
    for(pos=m_items.begin(); pos!=m_items.end(); ++pos){
      if((*pos)->Jid() == jid){
	return (*pos).get();
      }
    }
    return 0;
  }
  JabRosterItem*const JabRoster::FindName(std::string &name)
  {
    cont_type::iterator pos;
    for(pos=m_items.begin(); pos!=m_items.end(); ++pos){
      if((*pos)->Name() == name){
	return (*pos).get();
      }
    }
    return 0;
  }

  int JabRoster::Remove(std::string &jid)
  {
    return 0;
  }
  int JabRoster::Add(JabRosterItem *item)
  {
    if(!item){//<-- paranoia
      return 1;
    }
    elem_type pitem(item);
    m_items.push_back(pitem);
    return 0;
  }

  DOMNode*const JabRoster::BuildXml()
  {
    /**\todo move the scoped_arry -> xmlstr*/
    std::string type;
    boost::scoped_array<XMLCh> tagName,tagValue,attrName,attrValue;

    DOMDocument *doc = oa::XMLUtils().Document(DocUri(),"oa");

    oa::dom_ptr<DOMNode> iqElem(BuildIq());

    /* this is type sensitive
     * - get = just <query/> 
     * - set = remove
     * - result = nothing, just iq
     */

    type=Type();
    if(type=="get"){
      /* just build the query*/
      tagName.reset(XMLString::transcode("query"));
      tagValue.reset(XMLString::transcode(Xmlns().c_str()));
      oa::dom_ptr<DOMElement> queryElem(doc->createElementNS(tagValue.get(),
							     tagName.get()));
      //      attrName.reset(XMLString::transcode("xmlns"));
      attrValue.reset(XMLString::transcode(Xmlns().c_str()));
      /**\todo move this out somewhere to someplace for someone*/
      tagValue.reset(XMLString::transcode("http://www.w3.org/2000/xmlns/"));
      //      queryElem->setAttributeNS(tagValue.get(),attrName.get(),attrValue.get());
      oa::XercesXmlns()(queryElem.get(),attrValue.get());

      iqElem->appendChild(queryElem.release());
    }else if(type!="result"){
      /* just build the query*/
      tagName.reset(XMLString::transcode("query"));
      tagValue.reset(XMLString::transcode(Xmlns().c_str()));
      oa::dom_ptr<DOMElement> queryElem(doc->createElementNS(tagValue.get(),
							     tagName.get()));
      attrName.reset(XMLString::transcode("xmlns"));
      attrValue.reset(XMLString::transcode(Xmlns().c_str()));
      /**\todo move this out somewhere to someplace for someone*/
      tagValue.reset(XMLString::transcode("http://www.w3.org/XML/1998/namespace"));
      queryElem->setAttribute(attrName.get(),attrValue.get());



      /* ok now this is the one that uses the item list
       * loop through the vector and buildItem
       * append to the queryElem each item
       */
      cont_type::iterator pos=m_items.begin();
      for(;pos!=m_items.end(); ++pos){
	queryElem->appendChild((*pos)->BuildItem());
      }
      iqElem->appendChild(queryElem.release());
    }
    if(HasError()){
      oa::dom_ptr<DOMNode> errorElem(BuildError());
      if(errorElem.get()){
	iqElem->appendChild(errorElem.release());
      }
    }

    /*
    m_node=iqElem.release();
    return m_node.get();
    */
    return iqElem.release();
  }  
};
