/* -*- Mode: c++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 * OAjabber library
 * Contains objects to aid in Jabber client development and extend the
 * basic functionality of jabber.
 *
 * Copyright (C) 2001 Justin Kirby
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Justin Kirby <justink@rochester.rr.com>
 *
 *******************************************************************/
#ifndef OAJABBER_IJABROSTER_H
#define OAJABBER_IJABROSTER_H
#include <vector>
#include <boost/smart_ptr.hpp>
#include "jabber/ijabrosteritem.h"

namespace jabber
{
  class OAJABBER_API JabRoster;
  /** @ingroup gJabPacket
   */

  /**
   *
   */
  class JabRoster : public JabIq
  {
  public:
    typedef boost::shared_ptr<JabRosterItem> elem_type;
    typedef std::vector<elem_type> cont_type;
  public:
    JabRoster()
      : JabIq("roster"){}
    JabRoster(const JabRoster &jr)
      : JabIq(jr),
        m_items(jr.m_items){}
    virtual ~JabRoster(){}
    
    virtual const char*const Uri()const{
      return "xmpp:xmlns=jabber:iq:roster";
    }
    virtual std::string Xmlns()const{
      return "jabber:iq:roster";
    }


    virtual int Length()const{
      return m_items.size();
    }
    virtual JabRosterItem*const Get(int i){
      return m_items[i].get();
    }
    virtual int Remove(std::string &jid);
    virtual int Add(JabRosterItem *item);


    virtual JabRosterItem*const FindId(std::string &jid);
    virtual JabRosterItem*const FindName(std::string &name);

    virtual xercesc::DOMNode*const BuildXml();
  protected:    
    cont_type m_items;
  };
};
#endif
