/********************************************************************
 * OAjabber library
 * Contains objects to aid in Jabber client development and extend the
 * basic functionality of jabber.
 *
 * Copyright (C) 2002 Justin Kirby
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Justin Kirby <justink@rochester.rr.com>
 *
 *******************************************************************/
#include "framework/first.h"
#include "openaether.hpp"
#include <boost/smart_ptr.hpp>
#include <xercesc/util/XMLString.hpp>
#include <xercesc/dom/DOMElement.hpp>
#include "oapr/oapr_exception.hpp"
#include "oapr/oapr_exc_utils.hpp"
#include "framework/xmlutil.hpp"
#include "framework/xmlstr.hpp"
#include "framework/xerces_xmlns.hpp"
#include "jabber/ijabrosteritem.h"

XERCES_CPP_NAMESPACE_USE
namespace jabber
{
  DOMNode*const JabRosterItem::BuildItem()
  {
    std::string jid,name,sub,group;
    boost::scoped_array<XMLCh> tagName,tagValue,attrName,attrValue;

    DOMDocument *doc = oa::XMLUtils().Document(DocUri(),"oa");

    tagName.reset(XMLString::transcode("item"));
    oa::dom_ptr<DOMElement> itemElem(doc->createElement(tagName.get()));

    jid=Jid();
    if(jid.empty()){
      OAPR_THROW(oapr::logic_error,"Required attribute, jid, is empty");
    }

    attrName.reset(XMLString::transcode("jid"));
    attrValue.reset(XMLString::transcode(jid.c_str()));
    itemElem->setAttribute(attrName.get(),attrValue.get());

    name=Name();
    if(name.empty()){
      OAPR_THROW(oapr::logic_error,"Required attribute,name,is empty");
    }
    attrName.reset(XMLString::transcode("name"));
    attrValue.reset(XMLString::transcode(name.c_str()));
    itemElem->setAttribute(attrName.get(),attrValue.get());

    sub=SubType();
    if(sub.empty()){
      OAPR_THROW(oapr::logic_error,"Required attribute, subscription, is empty");
    }
    attrName.reset(XMLString::transcode("sub"));
    attrValue.reset(XMLString::transcode(sub.c_str()));
    itemElem->setAttribute(attrName.get(),attrValue.get());

    group=Group();
    if(!group.empty()){
      tagName.reset(XMLString::transcode("group"));
      tagValue.reset(XMLString::transcode(group.c_str()));
      oa::dom_ptr<DOMElement> groupElem(doc->createElement(tagName.get()));
      oa::dom_ptr<DOMText> groupValue(doc->createTextNode(tagValue.get()));
      groupElem->appendChild(groupValue.release());
      itemElem->appendChild(groupElem.release());
    }
    return itemElem.release();
  }

  DOMNode*const JabRosterItem::BuildXml()
  {
    boost::scoped_array<XMLCh> tagName,tagValue,attrName,attrValue;
    DOMDocument *doc = oa::XMLUtils().Document(DocUri(),"oa");

    oa::dom_ptr<DOMNode> iqElem(BuildIq());

    /* build the query element*/
    tagName.reset(XMLString::transcode("item"));
    tagValue.reset(XMLString::transcode(Xmlns().c_str()));
    oa::dom_ptr<DOMElement> queryElem(doc->createElementNS(tagValue.get(),tagName.get()));

    //    attrName.reset(XMLString::transcode("xmlns"));
    attrValue.reset(XMLString::transcode(Xmlns().c_str()));
    /**\todo move this out somewhere to someplace for someone*/
    tagValue.reset(XMLString::transcode("http://www.w3.org/XML/1998/namespace"));
    //    queryElem->setAttribute(attrName.get(),attrValue.get());
    oa::XercesXmlns()(queryElem.get(),attrValue.get());

    /* if this is being called its not for a roster request
     * at least it better not be
     * so the assumption is that its an item get/set/result
     */
    oa::dom_ptr<DOMNode> itemElem(BuildItem());
    
    queryElem->appendChild(itemElem.release());

    iqElem->appendChild(queryElem.release());
    return iqElem.release();
  }  
};
