/* -*- Mode: c++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 * OAjabber library
 * Contains objects to aid in Jabber client development and extend the
 * basic functionality of jabber.
 *
 * Copyright (C) 2001 Justin Kirby
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Justin Kirby <justink@rochester.rr.com>
 *
 *******************************************************************/
#ifndef OAJABBER_IJABROSTERITEM_H
#define OAJABBER_IJABROSTERITEM_H

#include "jabber/oajabber_exports.h"
#include "jabber/ijabiq.h"

namespace jabber
{
  /** @ingroup gJabPacket
   */
  class OAJABBER_API JabRosterItem;
  class JabRosterItem : public JabIq
  {
  public:        
    JabRosterItem()
      : JabIq("rosteritem"){}
    JabRosterItem(const JabRosterItem &jr)
      : JabIq(jr),
      m_group(jr.m_group),
      m_name(jr.m_name),
      m_subType(jr.m_subType),
      m_jid(jr.m_jid),
      m_subscribing(jr.m_subscribing){}
    virtual ~JabRosterItem(){}

    virtual const char*const Uri()const{
      return "xmpp:xmlns=jabber:iq:roster?item";
    }
    virtual std::string Xmlns()const{
      return "jabber:iq:roster";
    }

    virtual void Name(std::string name){m_name=name;}
    virtual void Jid(std::string jid){m_jid=jid;}
    virtual void SubType(std::string sub){m_subType=sub;}
    virtual void Group(std::string group){m_group=group;}
    virtual std::string Name()const{return m_name;}
    virtual std::string Jid()const{return m_jid;}
    virtual std::string SubType()const{return m_subType;}
    virtual std::string Group()const{return m_group;}

    
    virtual xercesc::DOMNode*const BuildXml();
  protected:
    virtual xercesc::DOMNode*const BuildItem();
  protected:
    std::string m_group;
    std::string m_name;
    std::string m_subType;
    std::string m_jid;
    bool m_subscribing;

    friend class JabRoster;
  };
};
#endif
