/* -*- Mode: c++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 * OAjabber library
 * Contains objects to aid in Jabber client development and extend the
 * basic functionality of jabber.
 *
 * Copyright (C) 2004 Justin Kirby
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Justin Kirby <justink@rochester.rr.com>
 *
 *******************************************************************/
#ifndef OAJABBER_IJABSTREAM_H
#define OAJABBER_IJABSTREAM_H
#include <string>
#include "xstrm/jabpacket.h"
#include "jabber/oajabber_exports.h"

namespace jabber
{
	class OAJABBER_API JabStream;
  class JabStream : public JabPacket
  {
  public:
    JabStream(const std::string& xmlns)
      : JabPacket("stream","stream")
        ,m_xmlns(xmlns)
        ,m_node(0)
    {
    }
    JabStream(const JabStream& js)
      : JabPacket(js)
        ,m_host(js.m_host)
        ,m_xmlns(js.m_xmlns)
        ,m_node(js.m_node)
    {
    }
    virtual ~JabStream()
    {
    }

    virtual const char*const Uri()const{
      return "xmpp:stream";
    }
    virtual std::string Xmlns()const{
      return m_xmlns;
    }
    virtual void Xmlns(const std::string& xmlns){}

    virtual void Node(xercesc::DOMNode* nd){m_node=nd;}

    virtual std::string Host()const{return m_host;}
    virtual void Host(const std::string& hs){m_host=hs;}

    virtual std::string xml();
    virtual xercesc::DOMNode*const BuildXml(){
      return m_node;
    }
  private:
    std::string m_host;
    std::string m_xmlns;
    xercesc::DOMNode* m_node;
  };
};
#endif
