/********************************************************************
 * OAjabber library
 * Contains objects to aid in Jabber client development and extend the
 * basic functionality of jabber.
 *
 * Copyright (C) 2001 Justin Kirby
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Justin Kirby <justink@rochester.rr.com>
 *
 *******************************************************************/
#include "framework/first.h"
#include "openaether.hpp"
#include <boost/smart_ptr.hpp>
#include <xercesc/util/XMLString.hpp>
#include <xercesc/dom/DOMElement.hpp>
#include "oapr/oapr_exception.hpp"
#include "oapr/oapr_exc_utils.hpp"
#include "framework/xmlutil.hpp"
#include "framework/xerces_xmlns.hpp"
#include "framework/xmlstr.hpp"
#include "framework/dom_ptr.hpp"
#include "xstrm/jabjid.h"
#include "jabber/ijabtime.h"

XERCES_CPP_NAMESPACE_USE
namespace jabber
{

  DOMNode*const JabTime::BuildXml()
  {
    /**\todo move scoped_array -> xmlstr */
    JabJid to,from;
    std::string type,id;
    std::string utc,tz,display;
    boost::scoped_array<XMLCh> tagName,tagValue,attrName,attrValue;

    DOMDocument *doc = oa::XMLUtils().Document(DocUri(),"oa");
    tagName.reset(XMLString::transcode("iq"));
    oa::dom_ptr<DOMElement> iqElem(doc->createElement(tagName.get()));

    /* type is required attribute
     * setting default to get if not there
     */
    type=Type();
    if(type.empty()){
      type="get";
    }
    attrName.reset(XMLString::transcode("type"));
    attrValue.reset(XMLString::transcode(type.c_str()));
    iqElem->setAttribute(attrName.get(),attrValue.get());

    /* to is a required attribute
     * however, there is no reasonable default
     * throwing logic error
     */
    to=To();
    if(to.IsNull()){
      OAPR_THROW(oapr::logic_error,"Required attribute to missing");
    }
    attrName.reset(XMLString::transcode("to"));
    attrValue.reset(XMLString::transcode(to.Jid().c_str()));
    iqElem->setAttribute(attrName.get(),attrValue.get());

    /* from is optional attribute
     * the jabberd takes care of this for us
     */
    from=From();
    if(!from.IsNull()){
      attrName.reset(XMLString::transcode("from"));
      attrValue.reset(XMLString::transcode(from.Jid().c_str()));
      iqElem->setAttribute(attrName.get(),attrValue.get());
    }

    /* id is a required attribute
     * throwing logic exception if not there
     */
    id=Id();
    if(id.empty()){
      OAPR_THROW(oapr::logic_error,"Required attribute id missing");
    }
    attrName.reset(XMLString::transcode("id"));
    attrValue.reset(XMLString::transcode(id.c_str()));
    iqElem->setAttribute(attrName.get(),attrValue.get());


    /* ok now on to the query element */
    tagName.reset(XMLString::transcode("query"));
    tagValue.reset(XMLString::transcode(Xmlns().c_str()));
    oa::dom_ptr<DOMElement> queryElem(doc->createElementNS(tagValue.get(),tagName.get()));

    //    attrName.reset(XMLString::transcode("xmlns"));
    attrValue.reset(XMLString::transcode(Xmlns().c_str()));
    /**\todo move this out somewhere to someplace for someone*/
    tagValue.reset(XMLString::transcode("http://www.w3.org/XML/1998/namespace"));
    //    queryElem->setAttribute(attrName.get(),attrValue.get());
    oa::XercesXmlns()(queryElem.get(),attrValue.get());


    utc=Utc();
    if(!utc.empty()){
      tagName.reset(XMLString::transcode("utc"));
      tagValue.reset(XMLString::transcode(utc.c_str()));
      oa::dom_ptr<DOMElement> utcElem(doc->createElement(tagName.get()));
      oa::dom_ptr<DOMText> utcValue(doc->createTextNode(tagValue.get()));
      utcElem->appendChild(utcValue.release());
      queryElem->appendChild(utcElem.release());
    }
    tz=TimeZone();
    if(!tz.empty()){
      tagName.reset(XMLString::transcode("tz"));
      tagValue.reset(XMLString::transcode(tz.c_str()));
      oa::dom_ptr<DOMElement> tzElem(doc->createElement(tagName.get()));
      oa::dom_ptr<DOMText> tzValue(doc->createTextNode(tagValue.get()));
      tzElem->appendChild(tzValue.release());
      queryElem->appendChild(tzElem.release());

    }
    display=Display();
    if(!display.empty()){
      tagName.reset(XMLString::transcode("display"));
      tagValue.reset(XMLString::transcode(display.c_str()));
      oa::dom_ptr<DOMElement> displayElem(doc->createElement(tagName.get()));
      oa::dom_ptr<DOMText> displayValue(doc->createTextNode(tagValue.get()));
      displayElem->appendChild(displayValue.release());
      queryElem->appendChild(displayElem.release());
    }
    iqElem->appendChild(queryElem.release());
    /*
    m_node=iqElem.release();
    return m_node.get();
    */
    return iqElem.release();
  }
};
