/* -*- Mode: c++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 * OAjabber library
 * Contains objects to aid in Jabber client development and extend the
 * basic functionality of jabber.
 *
 * Copyright (C) 2001 Justin Kirby
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Justin Kirby <justink@rochester.rr.com>
 *
 *******************************************************************/
#ifndef OAJABBER_IJABTIME_H
#define OAJABBER_IJABTIME_H
#include "jabber/ijabiq.h"

namespace jabber
{
  class OAJABBER_API JabTime;

  /** @ingroup gJabPacket
   */

  /// Time packet
  /** 
   * Get request time from a jabber id
   */
  class JabTime : public JabIq
  {
  public:
    JabTime()
      : JabIq("time"){}
    JabTime(const JabTime &jt)
      : JabIq(jt),
      m_utc(jt.m_utc),
      m_timezone(jt.m_timezone),
      m_display(jt.m_display){}
    virtual ~JabTime(){}

    virtual const char*const Uri()const{
      return "xmpp:xmlns=jabber:iq:time";
    }
    
    ///Set utc time
    virtual void Utc(const std::string &utc){m_utc=utc;}
    ///Set Time zone
    virtual void TimeZone(const std::string &tz){m_timezone=tz;}
    ///Set display
    virtual void Display(const std::string &dsp){m_display=dsp;}

    ///Get utc time
    virtual std::string Utc()const{return m_utc;}
    ///Get timezone
    virtual std::string TimeZone()const{return m_timezone;}
    ///Get display
    virtual std::string Display()const{return m_display;}

    virtual xercesc::DOMNode*const BuildXml();
  protected:
    std::string m_utc;
    std::string m_timezone;
    std::string m_display;
  };
};
#endif
