/********************************************************************
 * OAjabber library
 * Contains objects to aid in Jabber client development and extend the
 * basic functionality of jabber.
 *
 * Copyright (C) 2001 Justin Kirby
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Justin Kirby <justink@rochester.rr.com>
 *
 *******************************************************************/
#include "framework/first.h"
#include "openaether.hpp"
#include <boost/smart_ptr.hpp>
#include <xercesc/util/XMLString.hpp>
#include <xercesc/dom/DOMElement.hpp>
#include "oapr/oapr_exception.hpp"
#include "oapr/oapr_exc_utils.hpp"
#include "framework/xmlutil.hpp"
#include "framework/xerces_xmlns.hpp"
#include "framework/dom_ptr.hpp"
#include "framework/xmlstr.hpp"
#include "xstrm/jabjid.h"
#include "jabber/ijabver.h"

XERCES_CPP_NAMESPACE_USE
namespace jabber
{

  DOMNode*const JabVersion::BuildXml()
  {
    JabJid to,from;
    std::string id,type;
    std::string version,name,os;
    boost::scoped_array<XMLCh> tagName,tagValue,attrName,attrValue;

    DOMDocument *doc=oa::XMLUtils().Document(DocUri(),"oa");

    tagName.reset(XMLString::transcode("iq"));
    oa::dom_ptr<DOMElement> iqElem(doc->createElement(tagName.get()));

    /* the to attribute is required
     * if not present throw logic exception
     */
    to=To();
    if(to.IsNull()){
      OAPR_THROW(oapr::logic_error,"Require attribute to missing");
    }
    attrName.reset(XMLString::transcode("to"));
    attrValue.reset(XMLString::transcode(to.Jid().c_str()));
    iqElem->setAttribute(attrName.get(),attrValue.get());

    /* the type must be get,result or error
     * set to get if not result or error
     */
    type=Type();
    if(type.empty() || type=="get"){
      type="get";
    }
    attrName.reset(XMLString::transcode("type"));
    attrValue.reset(XMLString::transcode(type.c_str()));
    iqElem->setAttribute(attrName.get(),attrValue.get());

    /* from is handled by jabberd
     * but it can be set if here
     */
    from=From();
    if(!from.IsNull()){
      attrName.reset(XMLString::transcode("from"));
      attrValue.reset(XMLString::transcode(from.Jid().c_str()));
      iqElem->setAttribute(attrName.get(),attrValue.get());
    }

    /* create the query element with a namespace
     */
    tagName.reset(XMLString::transcode("query"));
    tagValue.reset(XMLString::transcode(Xmlns().c_str()));
    oa::dom_ptr<DOMElement> queryElem(doc->createElementNS(tagValue.get(),tagName.get()));


    //    attrName.reset(XMLString::transcode("xmlns"));
    attrValue.reset(XMLString::transcode(Xmlns().c_str()));
    /**\todo move this out somewhere to someplace for someone*/
    tagValue.reset(XMLString::transcode("http://www.w3.org/XML/1998/namespace"));
    //    queryElem->setAttributeNS(tagValue.get(),attrName.get(),attrValue.get());
    oa::XercesXmlns()(queryElem.get(),attrValue.get());
    version=Version();
    if(!version.empty()){
      tagName.reset(XMLString::transcode("version"));
      tagValue.reset(XMLString::transcode(version.c_str()));
      oa::dom_ptr<DOMElement> verElem(doc->createElement(tagName.get()));
      oa::dom_ptr<DOMText> verValue(doc->createTextNode(tagValue.get()));    
      verElem->appendChild(verValue.release());
      queryElem->appendChild(verElem.release());
    }

    name=Name();
    if(!name.empty()){
      tagName.reset(XMLString::transcode("name"));
      tagValue.reset(XMLString::transcode(name.c_str()));
      oa::dom_ptr<DOMElement> nameElem(doc->createElement(tagName.get()));
      oa::dom_ptr<DOMText> nameValue(doc->createTextNode(tagValue.get()));
      nameElem->appendChild(nameValue.release());
      queryElem->appendChild(nameElem.release());
    }
    
    os=Os();
    if(!os.empty()){
      tagName.reset(XMLString::transcode("os"));
      tagValue.reset(XMLString::transcode(os.c_str()));
      oa::dom_ptr<DOMElement> osElem(doc->createElement(tagName.get()));
      oa::dom_ptr<DOMText> osValue(doc->createTextNode(tagValue.get()));
      osElem->appendChild(osValue.release());
      queryElem->appendChild(osElem.release());				   
    }
    iqElem->appendChild(queryElem.release());
    /*
    m_node=iqElem.release();
    return m_node.get();
    */
    return iqElem.release();
  }
};
