/* -*- Mode: c++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 * OAjabber library
 * Contains objects to aid in Jabber client development and extend the
 * basic functionality of jabber.
 *
 * Copyright (C) 2001 Justin Kirby
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Justin Kirby <justink@rochester.rr.com>
 *
 *******************************************************************/
#ifndef OAJABBER_IJABVER_H
#define OAJABBER_IJABVER_H
#include "jabber/ijabiq.h"

namespace jabber
{

  class OAJABBER_API JabVersion;

  /** @ingroup gJabPacket
   */

  ///Jabber version
  /**
   * The jabber version packets
   */
  class JabVersion : public JabIq
  {
  public:
    JabVersion()
      : JabIq("version"){}
    JabVersion(const JabVersion &jv)
      : JabIq(jv),
      m_name(jv.m_name),
      m_version(jv.m_version),
      m_os(jv.m_os){}
    virtual ~JabVersion(){}

    virtual const char*const Uri()const{
      return "xmpp:xmlns=jabber:iq:version";
    }
    ///Set the name
    virtual void Name(const std::string &name){m_name=name;}
    ///Set the version
    virtual void Version(const std::string &version){m_version=version;}
    ///Set the OS
    virtual void Os(const std::string &os){m_os=os;}

    ///get the name
    virtual std::string Name()const{return m_name;}
    ///get the version
    virtual std::string Version()const{return m_version;}
    ///get the Os
    virtual std::string Os()const{return m_os;}

    virtual xercesc::DOMNode*const BuildXml();
  protected:
    std::string m_name;
    std::string m_version;
    std::string m_os;
  };
};
#endif
