/* -*- Mode: c++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 * OAjabber library
 * Contains objects to aid in Jabber client development and extend the
 * basic functionality of jabber.
 *
 * Copyright (C) 2001 Justin Kirby
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Justin Kirby <justink@rochester.rr.com>
 *
 *******************************************************************/
#include <iostream>
#include "framework/first.h"
#include "openaether.hpp"
#include "jabber/jabbldiq.h"
#include "jabber/ijabiq.h"
#include "framework/xmlstr.hpp"
#include <xercesc/dom/DOMElement.hpp>

XERCES_CPP_NAMESPACE_USE
namespace jabber
{
  bool BuildIq::CanBuild(DOMNode *const nd)const
  {
    oa::xmlstr<XMLCh,char> iqName("iq");
    if(nd->getNodeType()==DOMNode::ELEMENT_NODE &&
       !XMLString::compareString(iqName,nd->getLocalName())){
      /* if everyting is empty then simply return false */
      if(m_to.empty() && m_from.empty() && m_id.empty() && m_type.empty()){
        return false;
      }else{
        return true;
      }
    }
    return false;
  }
  JabPacket* BuildIq::Build(DOMNode*const nd)
  {
    if(!m_id.empty()){
      oa::xmlstr<XMLCh,char> idName("id"),idValue(m_id.c_str());
      const XMLCh* ndid=((DOMElement*)nd)->getAttribute(idName);
      if(!XMLString::compareString(ndid,idValue)){
        return BuildPacket(nd);
      }
    }else if(!m_type.empty()){
      oa::xmlstr<XMLCh,char> tName("type"),tValue(m_type.c_str());
      const XMLCh* ndt=((DOMElement*)nd)->getAttribute(tName);
      if(!XMLString::compareString(ndt,tValue)){
        return BuildPacket(nd);
      }
    }else if(!m_from.empty()){
      oa::xmlstr<XMLCh,char> fromName("from"),fromValue(m_from.c_str());
      const XMLCh* ndfrom=((DOMElement*)nd)->getAttribute(fromName);
      if(!XMLString::compareString(ndfrom,fromValue)){
        return BuildPacket(nd);
      }
    }else if(!m_to.empty()){
      oa::xmlstr<XMLCh,char> toName("to"),toValue(m_to.c_str());
      const XMLCh* ndto=((DOMElement*)nd)->getAttribute(toName);
      if(!XMLString::compareString(ndto,toValue)){
        return BuildPacket(nd);
      }
    }
    return 0;
  }
  JabPacket* BuildIq::BuildPacket(DOMNode*const nd)
  {
    std::auto_ptr<JabIq> rv(new JabIq("iq"));
    DOMElement*const iq=((DOMElement*const)nd);
    oa::xmlstr<char,XMLCh> attrval;
    oa::xmlstr<XMLCh,char> toName("to")
      ,fromName("from")
      ,idName("id")
      ,typeName("type");

    if(iq->hasAttribute(idName)){
      attrval.set(iq->getAttribute(idName));
      rv->Id(attrval.get());
    }
    if(iq->hasAttribute(toName)){
      attrval.set(iq->getAttribute(toName));
      JabJid to(attrval.get());
      rv->To(to);
    }
    if(iq->hasAttribute(fromName)){
      attrval.set(iq->getAttribute(fromName));
      JabJid from(attrval.get());
      rv->From(from);
    }
    if(iq->hasAttribute(typeName)){
      attrval.set(iq->getAttribute(typeName));
      rv->Type(attrval.get());
      if(rv->Type()=="error"){
        JabError err;
        if(GetError(err,nd)){
          rv->Error(err);
        }
      }
    }
    return rv.release();
  }
};
