/* -*- Mode: c++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 * OAjabber library
 * Contains objects to aid in Jabber client development and extend the
 * basic functionality of jabber.
 *
 * Copyright (C) 2001 Justin Kirby
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Justin Kirby <justink@rochester.rr.com>
 *
 *******************************************************************/
#ifndef OAJABBER_JABBLDIQ_H
#define OAJABBER_JABBLDIQ_H

#include "boost/smart_ptr.hpp"
#include "xstrm/jabbuild.h"
#include "jabber/oajabber_exports.h"

namespace jabber
{
  /** @ingroup gJabBuild
   * @{
   */

  ///Build JabIq object
  /**
   * Creates a JabIq object.
   * Probably most useful as a base class. There is
   * very little point for it otherwise.
   */
  class OAJABBER_API BuildIq;
  class BuildIq : public JabBuild
  {
  public:
    BuildIq(){}
    BuildIq(const BuildIq& bi)
      : JabBuild(bi)
        ,m_to(bi.m_to)
        ,m_from(bi.m_from)
        ,m_id(bi.m_id)
        ,m_type(bi.m_type){}
    virtual ~BuildIq(){}
    virtual std::string ID()const{
      return "jabber::BuildIq";
    }
    
    virtual void To(const std::string &to){m_to=to;}
    virtual void From(const std::string &from){m_from=from;}
    virtual void Id(const std::string &id){m_id=id;}
    virtual void Type(const std::string &type){m_type=type;}

    virtual JabBuild* Clone(){
      return new BuildIq(*this);
    }
    virtual bool CanBuild(xercesc::DOMNode *const nd)const;
    virtual JabPacket* Build(xercesc::DOMNode*const nd);
  private:
    JabPacket* BuildPacket(xercesc::DOMNode*const nd);    
  private:
    std::string m_to;
    std::string m_from;
    std::string m_id;
    std::string m_type;
  };
  /** @} */
};
#endif

