/********************************************************************
 * OAjabber library
 * Contains objects to aid in Jabber client development and extend the
 * basic functionality of jabber.
 *
 * Copyright (C) 2002 Justin Kirby
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Justin Kirby <justink@rochester.rr.com>
 *
 *******************************************************************/
#include "framework/first.h"
#include "openaether.hpp"
#include <boost/smart_ptr.hpp>
#include <xercesc/util/XMLString.hpp>
#if !defined WIN32_VC6
/**@todo find a better solution */
#include "oapu/oapr_logstream.hpp"
#endif
#include "framework/xmlutil.hpp"
#include "jabber/ijablast.h"
#include "jabber/jabbldlast.h"

XERCES_CPP_NAMESPACE_USE
namespace jabber
{
  bool BuildLast::CanBuild(DOMNode *const nd)const
  {
    /* requirements to qualify as jep012 are:
     * - nd name is iq
     * - nd has children
     * - child name is query and xmlns is jabber:iq:last
     */
    boost::scoped_array<XMLCh> iqName(XMLString::transcode("iq"));
    boost::scoped_array<XMLCh> queryName(XMLString::transcode("query"));
    boost::scoped_array<XMLCh> uri(XMLString::transcode("jabber:iq:last"));
    if(nd->hasChildNodes() &&
       !XMLString::compareString(nd->getNodeName(),iqName.get())){
      DOMNode*query=nd->getFirstChild();
      if(!XMLString::compareString(query->getNodeName(),queryName.get()) &&
	 !XMLString::compareString(uri.get(),query->getNamespaceURI())){
	return true;
      }
    }
    return false;
  }
  JabPacket* BuildLast::Build(DOMNode *const nd)
  {
    if(!nd){return 0;}
    std::auto_ptr<JabLast> plast(new JabLast);

    DOMNode *iq = nd;
#if !defined WIN32_VC6
/**@todo find a better solution */
    oapr::logstrm<<OAPR_LOG("Building Last",oapr::Log_Info);
    if(oapr::loglvl>=oapr::Log_Debug){
      oapr::logstrm<<oa::XMLUtils().Print(nd);
    }    
#endif
    std::string to,from,id,type;
    GetTo(to,iq);
    GetFrom(from,iq);
    GetId(id,iq);
    GetType(type,iq);

    /* since we are only getting a single attribute
     * we just do it here
     */
    /* get the query child*/
    DOMNode *query = iq->getFirstChild();
    if(!query || query->getNodeType()!=DOMNode::ELEMENT_NODE){
      return 0;
    }
    std::string seconds;
    boost::scoped_array<XMLCh> attrName(XMLString::transcode("seconds"));
    if(((DOMElement*)query)->hasAttribute(attrName.get())){
      oa::XMLUtils().DomstrToStd(seconds,((DOMElement*)query)->getAttribute(attrName.get()));
    }

    if(!seconds.empty()){
      plast->Seconds(seconds);
    }
    plast->To(to);
    plast->From(from);
    plast->Id(id);
    plast->Type(type);
    return plast.release();
  }
};
