/********************************************************************
 * OAjabber library
 * Contains objects to aid in Jabber client development and extend the
 * basic functionality of jabber.
 *
 * Copyright (C) 2001 Justin Kirby
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Justin Kirby <justink@rochester.rr.com>
 *
 *******************************************************************/
#include "framework/first.h"
#include "openaether.hpp"
#include <boost/smart_ptr.hpp>
#include <xercesc/util/XMLString.hpp>
#if !defined WIN32_VC6
/**@todo find a better solution */
#include "oapu/oapr_logstream.hpp"
#endif
#include "framework/xmlutil.hpp"
#include "jabber/jabbldmsg.h"

XERCES_CPP_NAMESPACE_USE
namespace jabber
{
  bool BuildMessage::CanBuild(DOMNode *const nd)const
  {
    /* only requirement is that the node be named message*/
    boost::scoped_array<XMLCh> name(XMLString::transcode("message"));
    if(nd->getNodeType()==DOMNode::ELEMENT_NODE &&
       !XMLString::compareString(name.get(),nd->getNodeName())){
      return true;
    }
    return false;
  }
  JabPacket* BuildMessage::Build(DOMNode *const nd)
  {
    if(!nd){return 0;}

    std::auto_ptr<JabMsg> pmessage(new JabMsg);
    DOMNode*msg=nd;
#if !defined WIN32_VC6
/**@todo find a better solution */
    oapr::logstrm<<OAPR_LOG("Building Message",oapr::Log_Info);
    if(oapr::loglvl>=oapr::Log_Debug){
      oapr::logstrm<<oa::XMLUtils().Print(nd);
    }
#endif

    JabError error;
    if(GetError(error,msg)){
      pmessage->Error(error);
    }
    const XMLCh* msgName = ((DOMElement*)msg)->getTagName();
    std::string strMsgName;
    oa::XMLUtils().DomstrToStd(strMsgName,msgName);
    if(strMsgName != "message"){
      return 0;
    }
    std::string to,from,id,type,subject,body,thread;

    GetTo(to,msg);
    GetFrom(from,msg);
    GetId(id,msg);
    GetType(type,msg);
    GetSubject(subject,msg);
    GetBody(body,msg);
    GetThread(thread,msg);
    pmessage->Id(id);
    pmessage->From(from);
    pmessage->To(to);
    pmessage->Type(type);
    pmessage->Subject(subject);
    pmessage->Body(body);
    pmessage->Thread(thread);

    return pmessage.release();
  }
  bool BuildMessage::GetSubject(std::string &sbj, DOMNode *const nd)
  {
    boost::scoped_array<XMLCh> tagName(XMLString::transcode("subject"));
    DOMNodeList* subjects = 
      ((DOMElement*)nd)->getElementsByTagName(tagName.get());
    if(subjects->getLength()<0){
      return false;
    }
    //first one is all we care about for now
    DOMNode *subject = subjects->item(0);
    if(!subject || !subject->hasChildNodes()){
      return false;
    }
    DOMNode* txt = subject->getFirstChild();
    if(!txt){
      return false;
    }

    const XMLCh* domSubj = txt->getNodeValue();

    oa::XMLUtils().DomstrToStd(sbj,domSubj);
    return true;
  }

  bool BuildMessage::GetBody(std::string &body,DOMNode*const nd)
  {
    boost::scoped_array<XMLCh> tagName(XMLString::transcode("body"));
    DOMNodeList* bodies = 
      ((DOMElement*)nd)->getElementsByTagName(tagName.get());
    if(bodies->getLength()<0){
      return false;
    }
    DOMNode* ndBody = bodies->item(0);
    if(!ndBody || !ndBody->hasChildNodes()){
      return false;
    }
    DOMNode* txt = ndBody->getFirstChild();
    if(!txt){
      return false;
    }
    const XMLCh* domBody = txt->getNodeValue();
    oa::XMLUtils().DomstrToStd(body,domBody);
    return true;
  }

  bool BuildMessage::GetThread(std::string &thread,DOMNode *const nd)
  {
    boost::scoped_array<XMLCh> tagName(XMLString::transcode("thread"));
    DOMNodeList* threads = 
      ((DOMElement*)nd)->getElementsByTagName(tagName.get());
    if(threads->getLength()<0){
      return false;
    }
    //only interested (and should only be) one thread elem
    DOMNode* ndThread = threads->item(0);
    if(!ndThread || !ndThread->hasChildNodes()){
      return false;
    }
    DOMNode* txt = ndThread->getFirstChild();
    if(!txt){
      return false;
    }
    const XMLCh* domThread = txt->getNodeValue();
    oa::XMLUtils().DomstrToStd(thread, domThread);
    return true;
  }
};
