/* -*- Mode: c++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 * OAjabber library
 * Contains objects to aid in Jabber client development and extend the
 * basic functionality of jabber.
 *
 * Copyright (C) 2001 Justin Kirby
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Justin Kirby <justink@rochester.rr.com>
 *
 *******************************************************************/
/**
 * Changelog
 * 2002-07-30
 *  Forgot to include jabbuild.h for JabBuild
 */
#ifndef OAJABBER_JABBLDMSG_H
#define OAJABBER_JABBLDMSG_H


#include "jabber/oajabber_exports.h"
#include "xstrm/jabbuild.h"
#include "jabber/ijabmsg.h"

namespace jabber
{
  class OAJABBER_API BuildMessage;

  /** @ingroup gJabBuild
   */

  ///Builds JabMsg object
  /**
   * Creates a JabMsg object.
   * Attempts to handle all features that
   * would be expected of a message object.
   * With the notable exception of X.
   * However, all features are as base as possible.
   */
  class BuildMessage : public JabBuild
  {
  public:
    ///ctor
    BuildMessage(){}
    BuildMessage(const BuildMessage& bm)
      : JabBuild(bm){}

    ///dtor
    virtual ~BuildMessage(){}
    
    virtual std::string ID()const{
      return "jabber::BuildMessage";
    }
    virtual JabBuild* Clone(){
      return new BuildMessage(*this);
    }
    virtual bool CanBuild(xercesc::DOMNode *const nd)const;
    ///Build the JabMsg
    virtual JabPacket* Build(xercesc::DOMNode *const nd);
  protected:

    ///Get the subject
    /**
     * find the value of subject
     * @param sbj subject value
     * @param nd DOMNode
     */
    virtual bool GetSubject(std::string &sbj,xercesc::DOMNode*const nd);

    ///Get the Body
    /**
     * Get the message body
     * @param body body value
     * @param nd xercesc::DOMNode
     * @return true on success, false otherwise
     */
    virtual bool GetBody(std::string &body,xercesc::DOMNode*const nd);

    ///Get the thread
    /**
     * Get the message thread
     * @param thread value
     * @param nd DOMNode
     * @return true on success, false otherwise
     */
    virtual bool GetThread(std::string &thread,xercesc::DOMNode*const nd);
  };

};
#endif
