/********************************************************************
 * OAjabber library
 * Contains objects to aid in Jabber client development and extend the
 * basic functionality of jabber.
 *
 * Copyright (C) 2001 Justin Kirby
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Justin Kirby <justink@rochester.rr.com>
 *
 *******************************************************************/
#include "framework/first.h"
#include "openaether.hpp"
#include <boost/smart_ptr.hpp>
#include <xercesc/util/XMLString.hpp>
#if !defined WIN32_VC6
/**@todo find a better solution */
#include "oapu/oapr_logstream.hpp"
#endif
#include "framework/xmlutil.hpp"
#include "jabber/jabbldpres.h"

XERCES_CPP_NAMESPACE_USE
namespace jabber
{
  bool BuildPresence::CanBuild(DOMNode *const nd)const
  {
    /* if this nodes name is presence, then we can do this*/
    boost::scoped_array<XMLCh> name(XMLString::transcode("presence"));
    if(nd->getNodeType()==DOMNode::ELEMENT_NODE &&
       !XMLString::compareString(name.get(),nd->getLocalName())){
      return true;
    }    
    return false;
  }
  JabPacket* BuildPresence::Build(DOMNode *const nd)
  {
    /* make sure that the packet is reset*/
    std::auto_ptr<JabPresence> ppresence(new JabPresence);
    //new dom stuff
    if(!nd){return 0;}
#if !defined WIN32_VC6
/**@todo find a better solution */
    oapr::logstrm<<OAPR_LOG("Building Presense",oapr::Log_Info);
    if(oapr::loglvl>=oapr::Log_Debug){
      oapr::logstrm<<oa::XMLUtils().Print(nd);
    }
#endif
    DOMNode*pres=nd;
    JabError error;
    if(GetError(error,pres)){
      ppresence->Error(error);
    }
    const XMLCh* presName = ((DOMElement*)pres)->getTagName();
    std::string strPresName;
    oa::XMLUtils().DomstrToStd(strPresName,presName);
    if(strPresName != "presence"){
      return 0;
    }

    std::string from,to,id,type,show,status,priority;
    GetFrom(from,pres);
    GetTo(to,pres);
    GetId(id,pres);
    GetType(type,pres);
    GetShow(show,pres);
    GetStatus(status,pres);
    GetPriority(priority,pres);

    ppresence->Id(id);
    ppresence->To(to);
    ppresence->From(from);
    ppresence->Type(type);
    ppresence->Show(show);
    ppresence->Status(status);
    ppresence->Priority(priority);

    return ppresence.release();
  }
  bool BuildPresence::GetShow(std::string &show,DOMNode *const nd)
  {
    boost::scoped_array<XMLCh> tagName(XMLString::transcode("show"));
    DOMNodeList *shows = 
      ((DOMElement*)nd)->getElementsByTagName(tagName.get());
    if(shows->getLength()<0){
      return false;
    }
    //first one is all we care about for now
    DOMNode* ndShow = shows->item(0);
    if(!ndShow || !ndShow->hasChildNodes()){
      return false;
    }
    DOMNode* txt = ndShow->getFirstChild();
    if(!txt){
      return false;
    }

    const XMLCh* domSubj = txt->getNodeValue();
    oa::XMLUtils().DomstrToStd(show,domSubj);
    return true;
  }
  bool BuildPresence::GetStatus(std::string &stat,DOMNode*const nd)
  {
    boost::scoped_array<XMLCh> tagName(XMLString::transcode("status"));
    DOMNodeList* statuses = 
      ((DOMElement*)nd)->getElementsByTagName(tagName.get());
    if(statuses->getLength()<0){
      return false;
    }
    //first one is all we care about for now
    DOMNode* status  = statuses->item(0);
    if(!status || !status->hasChildNodes()){
      return false;
    }
    DOMNode* txt = status->getFirstChild();
    if(!txt){
      return false;
    }

    const XMLCh* domStat = txt->getNodeValue();
    oa::XMLUtils().DomstrToStd(stat,domStat);
    return true;

  }
  bool BuildPresence::GetPriority(std::string &pri,DOMNode*const nd)
  {
    boost::scoped_array<XMLCh> tagName(XMLString::transcode("priority"));
    DOMNodeList* priorities = 
      ((DOMElement*)nd)->getElementsByTagName(tagName.get());
    if(priorities->getLength()<0){
      return false;
    }
    //first one is all we care about for now
    DOMNode* priority  = priorities->item(0);
    if(!priority || !priority->hasChildNodes()){
      return false;
    }
    DOMNode* txt = priority->getFirstChild();
    if(!txt){
      return false;
    }

    const XMLCh* domPri = txt->getNodeValue();
    oa::XMLUtils().DomstrToStd(pri,domPri);
    return true;
  }
};
