/* -*- Mode: c++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 * OAjabber library
 * Contains objects to aid in Jabber client development and extend the
 * basic functionality of jabber.
 *
 * Copyright (C) 2001 Justin Kirby
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Justin Kirby <justink@rochester.rr.com>
 *
 *******************************************************************/
#ifndef OAJABBER_JABBLDPRES_H
#define OAJABBER_JABBLDPRES_H
#include "jabber/oajabber_exports.h"
#include "xstrm/jabbuild.h"
#include "jabber/ijabpres.h"

namespace jabber
{
  class OAJABBER_API BuildPresence;

  /** @ingroup gJabBuild
   */

  ///Builds JabPresence object.
  /***
   * Builds the JabPresence object from a
   * CmdPresence. If another object is used,
   * the results are undetermined.
   */
  class BuildPresence : public JabBuild
  {
  public:
    ///ctor
    BuildPresence(){}
    BuildPresence(const BuildPresence& bp)
      : JabBuild(bp){}
	
    ///dtor
    virtual ~BuildPresence(){}

    virtual std::string ID()const{
      return "jabber::BuildPresence";
    }
    virtual JabBuild* Clone(){
      return new BuildPresence(*this);
    }
    virtual bool CanBuild(xercesc::DOMNode *const nd)const;
    ///Build for JabPresence
    /**
     * Attempts to instantiate a JabPresence object
     * Uses the data from DOMNode
     * @param nd node to build
     * @return JabPresence packet, null on failure
     */
    virtual JabPacket* Build(xercesc::DOMNode *const nd);
  private:
    ///Get the status
    /**
     * get the status of the presence element if it exists
     * @param stat value of status
     * @param nd DOMNode
     */
    bool GetStatus(std::string &stat,xercesc::DOMNode*const nd);

    ///Get the show
    /**
     * get the show value if it exists
     * @param show value of show
     * @param nd DOMNode
     */
    bool GetShow(std::string &show,xercesc::DOMNode*const nd);
    //Get the priority
    /**
     * extracet the priority from the presence
     * @param pri value of priority
     * @param nd DOMNode
     */
    bool GetPriority(std::string &pri,xercesc::DOMNode*const nd);
  };
};
#endif
