/********************************************************************
 * OAjabber library
 * Contains objects to aid in Jabber client development and extend the
 * basic functionality of jabber.
 *
 * Copyright (C) 2001 Justin Kirby
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Justin Kirby <justink@rochester.rr.com>
 *
 *******************************************************************/
#include "framework/first.h"
#include "openaether.hpp"
#include <boost/smart_ptr.hpp>
#include <xercesc/util/XMLString.hpp>
#include <xercesc/dom/DOMNamedNodeMap.hpp>
#include <xercesc/dom/DOMNodeList.hpp>
#include <xercesc/dom/DOMElement.hpp>
#if !defined WIN32_VC6
/**@todo find a better solution */
#include "oapu/oapr_logstream.hpp"
#endif
#include "framework/xmlutil.hpp"
#include "framework/xmlstr.hpp"
#include "jabber/jabbldregister.h"

XERCES_CPP_NAMESPACE_USE
namespace jabber
{
  bool BuildRegister::CanBuild(DOMNode *const nd)const
  {
    /* what this needs to build
     * - nd named iq and has children
     * - child must be query with xmlns of jabber:iq:register
     */
    oa::xmlstr<XMLCh,char> iqName("iq");
    oa::xmlstr<XMLCh,char> queryName("query");
    oa::xmlstr<XMLCh,char> uri("jabber:iq:register");
    if(nd->hasChildNodes() &&
       !XMLString::compareString(nd->getLocalName(),iqName)){
      DOMNode*query=nd->getFirstChild();
      if(!XMLString::compareString(query->getLocalName(),queryName)&&
	 !XMLString::compareString(query->getNamespaceURI(),uri)){
	return true;
      }
    }
    return false;
  }
  JabPacket* BuildRegister::Build(DOMNode *const nd)
  {
    if(!nd){return 0;/**\todo should this throw?*/}
    DOMNode* reg = nd;
    std::auto_ptr<JabRegister> pregister(new JabRegister);
#if !defined WIN32_VC6
/**@todo find a better solution */
    oapr::logstrm<<OAPR_LOG("Building Register",oapr::Log_Info);
    if(oapr::loglvl>=oapr::Log_Debug){
      oapr::logstrm<<oa::XMLUtils().Print(nd);
    }    
#endif
    JabError error;
    if(GetError(error,reg)){
      pregister->Error(error);
    }

    std::string to,from,id,type;
    GetTo(to,reg);
    GetFrom(from,reg);
    GetId(id,reg);
    GetType(type,reg);

    pregister->Id(id);
    pregister->From(from);
    pregister->To(to);
    pregister->Type(type);

    //determine type:
    //is this a response for required fields?
    if(reg->hasChildNodes() && type=="result"){
      //do required fields response
      BuildRequired(pregister.get(),reg);
    }else{
      //successful response
      BuildResult(pregister.get(),reg);
    }
    return pregister.release();
  }
  bool BuildRegister::BuildRequired(JabRegister*const reg,DOMNode*const nd)
  {
    DOMNode* query = nd->getFirstChild();
    if(!query){
      return false;
    }

    DOMNodeList* reqs = query->getChildNodes();
    DOMNode* req;
    std::string name;
    for(unsigned int i=0; i<reqs->getLength();++i){
      req = reqs->item(i);
      if(!req || req->getNodeType() != DOMNode::ELEMENT_NODE){
	continue;
      }
      //ok now we get the name... and add it to register as required
      const XMLCh* reqName = ((DOMElement*)req)->getTagName();
      //list of exceptions are:
      //<instructions> and <key> which need to be stored
      oa::XMLUtils().DomstrToStd(name,reqName);
      if(name == "instructions" || name == "key"){
	DOMNode *reqText = req->getFirstChild();
	if(reqText){
	  const XMLCh* reqValue = reqText->getNodeValue();
	  std::string strValue;
	  oa::XMLUtils().DomstrToStd(strValue,reqValue);
	  reg->SetValue(name,strValue);	  
	}
      }else{
	reg->AddRequired(name);
      }
    }
    return true;
  }
  bool BuildRegister::BuildResult(JabRegister*const reg,DOMNode*const nd)
  {
    return true;//hmmmmm
  }
};
