/* -*- Mode: c++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 * OAjabber library
 * Contains objects to aid in Jabber client development and extend the
 * basic functionality of jabber.
 *
 * Copyright (C) 2001 Justin Kirby
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Justin Kirby <justink@rochester.rr.com>
 *
 *******************************************************************/
#ifndef OAJABBER_JBBLDTIME_H
#define OAJABBER_JBBLDTIME_H
#include "jabber/oajabber_exports.h"
#include "xstrm/jabbuild.h"
#include "jabber/ijabtime.h"

namespace jabber
{
  class OAJABBER_API BuildTime;

  /** @ingroup gJabBuild
   */
  ///Build a JabTime packet
  /**
   * Builds a JabTime packet from the DOMNode
   */
  class BuildTime : public JabBuild
  {
  public:
    ///ctor
    BuildTime(){}    
    BuildTime(const BuildTime& bt)
      : JabBuild(bt){}
    ///dtor
    virtual ~BuildTime(){}

    virtual std::string ID()const{
      return "jabber::BuildTime";
    }
    virtual JabBuild* Clone(){
      return new BuildTime(*this);
    }
    virtual bool CanBuild(xercesc::DOMNode *const nd)const;
    ///Build the packet
    /**
     * Build the packet
     * @param nd DOMNode
     * @return Pointer to JabTime packet, null on failure
     */
    virtual JabPacket* Build(xercesc::DOMNode *const nd);
  protected:
    ///Get UTC
    /**
     * Get teh UTC value
     * @param utc utc value
     * @param nd DOMNode
     */
    virtual int GetUTC(std::string &utc,xercesc::DOMNode*const nd);

    ///Get Timezone
    /**
     * Get the timezone
     * @param tz timezone value
     * @param nd DOMNode
     */
    virtual int GetTimeZone(std::string &tz,xercesc::DOMNode*const nd);

    ///Get display
    /**
     * Get teh value to display
     * @param disp display value
     * @param nd DOMNode
     */
    virtual int GetDisplay(std::string &disp,xercesc::DOMNode*const nd);
  };
};
#endif
