/* -*- Mode: c++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 * OAjabber library
 * Contains objects to aid in Jabber client development and extend the
 * basic functionality of jabber.
 *
 * Copyright (C) 2001 Justin Kirby
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Justin Kirby <justink@rochester.rr.com>
 *
 *******************************************************************/
#ifndef OAJABBER_JABBLDVER_H
#define OAJABBER_JABBLDVER_H
#include "jabber/oajabber_exports.h"
#include "xstrm/jabbuild.h"
#include "jabber/ijabver.h"

namespace jabber
{
  class OAJABBER_API BuildVersion;
  /** @ingroup gJabBuild
   */

  ///Build version JabPacket
  /**
   * This builds a JabVersion packet
   * from the DOMNode
   */
  class BuildVersion : public JabBuild
  {
  public:
    ///ctor
    BuildVersion(){}
    BuildVersion(const BuildVersion& bv)
      : JabBuild(bv){}
    ///dtor
    virtual ~BuildVersion(){}

    virtual bool CanBuild(xercesc::DOMNode *const nd)const;
    ///Build request
    /**
     * Build the JabVersion packet
     * @param nd DOMNode
     * @return pointer to JabVersion packet, null on failure
     */
    virtual JabPacket* Build(xercesc::DOMNode *const nd);

  protected:

    ///Get the name
    /**
     * Get the name of the end points client
     * @param name name value
     * @param nd DOMNode
     */
    virtual int GetName(std::string &name,xercesc::DOMNode*const nd);

    ///Get the version
    /** 
     * Get teh version of the jid
     * @param ver version value
     * @param nd DOMNode
     */
    virtual int GetVersion(std::string &ver,xercesc::DOMNode*const nd);

    ///Get the os
    /**
     * Get teh operating system
     * @param os os value
     * @param nd DOMNode
     */
    virtual int GetOs(std::string &os,xercesc::DOMNode*const nd);
  };
};
#endif
