/********************************************************************
 * OAjabber library
 * Contains objects to aid in Jabber client development and extend the
 * basic functionality of jabber.
 *
 * Copyright (C) 2001 Justin Kirby
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Justin Kirby <justink@rochester.rr.com>
 *
 *******************************************************************/
#ifndef OAJABBER_JABEVENTPOL_H
#define OAJABBER_JABEVENTPOL_H

#include <pthread.h>

namespace jabber
{

  /** @defgroup gEventPolicies Event Policies
   * Provide basic policies for
   * - synchronization
   * - threads
   * @{
   */

  /** @defgroup gEvenSyncPolicies Event Synchronization Policies
   * Synchronized objects, Sync and ASync
   * These provide synchronization abilities to events.
   * @{
   */

  /**
   * Basic wrapper around the boost conditional.
   * This allows the Event to fire off a signal
   * to any object waiting for such a signal.
   */
  class EventSyncPol
  {
  public:
    EventSyncPol(){}
    EventSyncPol(const EventSyncPol& esp)
      : m_condition(esp.m_condition){}
    virtual ~EventSyncPol(){}

    /**
     * Sets the condition. 
     * Because the boost condition is noncopyable,
     * a pointer must be used. Note that the condition
     * pointed to can *not* go out of scope
     * @param cnd Pointer to the boost condition
     */
    void SetConditional(pthread_cond_t pct){
      m_condition=pct;
    }

    /**
     * Basically a call to the notify_all function
     * of the boost condition. 
     */
    void Signal(){
      pthread_cond_broadcast(&m_condition);
    }
  private:
    pthread_cond_t m_condition;
  };


  /**
   * Event Asynchronize policy
   * This is a simple null object. The default implementation
   * without any modification is asynchronous. Therefore there
   * is really nothing to do
   */
  class EventASyncPol
  {
  public:
    EventASyncPol(){}
    EventASyncPol(const EventASyncPol& easp){}
    virtual ~EventASyncPol(){}
    /**
     * Null signal... it does nothing
     */
    void Signal(){}    
  };
  /** @} */ //end of gEventSyncPolicies

  /** @defgroup gEventThreadPolicies Event Threading Policies
   * @{
   */
  /*
  template<typename event_t>
  class EventThreadPol : public event_t
  {
  public:
    EventThreadPol(){}    
    EventThreadPol(const EventThreadPol& etp)
      :p_event(etp.p_event){}
    virtual ~EvenThreadPol(){}
    void operator()(){
      //static casting to disambiguate
      static_cast<event_t*>(this)->Event(m_jid,p_packet);
    }
  private:
    event_t *p_event;
  };
  */
  /*
  tempalte<typename event_t>
  class EventNullThreadPol
  {
  public:
    EventNullThreadPol(){}
    virtual ~EventNullThreadPol(){}
  };
  */
  /** @} */ //end of gEventThreadPolicies
  /** @} */ //end of gEventPolicies
};
#endif
