/* -*- Mode: c++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 * OAjabber library
 * Contains objects to aid in Jabber client development and extend the
 * basic functionality of jabber.
 *
 * Copyright (C) 2001 Justin Kirby
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Justin Kirby <justink@rochester.rr.com>
 *
 *******************************************************************/
#ifndef OAJABBER_JABEVENTPOLICY_H
#define OAJABBER_JABEVENTPOLICY_H

#include "oajabber/jabevent.h"
#include "oajabber/jabeventpol.h"

namespace jabber
{
  template<
    typename event_t, 
    typename sync_t=EventASyncPol
  >
  class PolicyEvent : public sync_t, 
		      public event_t
  {
  public:
    PolicyEvent()
      : p_event(0){}
    PolicyEvent(const PolicyEvent& pe)
      :	sync_t(pe),
	event_t(pe),
	p_event(pe.p_event){}
    virtual ~PolicyEvent(){}
    virtual std::string Id()const{
      if(p_event){
        return "jabber::PolicyEvent"+p_event->Id();
      }
      return "";
    }
    virtual void SetEvent(event_t* ev){
      p_event=ev;
    }
  
    virtual JabEvent::HANDLE_MODES HandleEvent(const JabJid& jid, 
					       JabPacket *pckt){
      JabEvent::HANDLE_MODES retval;
      if(p_event){
	retval = p_event->HandleEvent(jid,pckt);
      }else{
	return JabEvent::PASS;
      }

      if(retval == JabEvent::CONTINUE || retval == JabEvent::USED){
	Signal();
      }
      return retval;
    }
  private:
    event_t *p_event;    
  };
};

#endif
