/********************************************************************
 * OAjabber library
 * Contains objects to aid in Jabber client development and extend the
 * basic functionality of jabber.
 *
 * Copyright (C) 2001 Justin Kirby
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Justin Kirby <justink@rochester.rr.com>
 *
 *******************************************************************/
#include "framework/first.h"
#include <boost/tokenizer.hpp>
#include "oapr/oapr_exception.hpp"
#include "oapr/oapr_exc_utils.hpp"
#include "jabber/jaburi.h"

/**\bug
 * This is completely STUPID!! move this stuff around somewhere
 * proprietary extentions:
 * iq on the query, this is not allowed by the jep0032
 * id as a valid param for the message query
 * adding disco as valid query
 */
const std::string jabber::JabUri::m_validQuery="message presence subscribe iq disco";
const std::string jabber::JabUri::m_validMessage="body subject thread id";
const std::string jabber::JabUri::m_validPresence="type show status priority";
const std::string jabber::JabUri::m_validSubscribe="subscribe subscribed unsubscribe unsubscribed";

namespace jabber
{
  bool JabUri::operator==(const JabUri& ju)const
  {
    if(&ju==this){return true;}
    if(m_jid==ju.m_jid &&
       m_query==ju.m_query){
      cont_type::const_iterator pos=m_parameters.begin();
      cont_type::const_iterator fnd;
      for(;pos!=m_parameters.end();++pos){
	fnd=ju.m_parameters.find(pos->first);
	if(fnd==ju.m_parameters.end()){
	  return false;
	}
	if(fnd->second!=pos->second){
	  return false;
	}
      }
      return true;
    }
    return false;
  }
  bool JabUri::operator<(const JabJid& jj)const
  {
    if(m_jid<jj.Jid()){
      return true;
    }
    return false;
  }
  std::string JabUri::Parameter(const std::string &param)const
  {
    cont_type::const_iterator fnd=m_parameters.find(param);
    if(fnd==m_parameters.end()){
      OAPR_THROW(oapr::runtime_error,"invalid parameter name");      
    }
    return fnd->second;
  }
    
  bool JabUri::IsNull()const
  {
    if(m_jid.IsNull() &&
       m_query.empty()){
      return true;
    }
    return false;
  }
  bool JabUri::IsValid()const
  {
    /* - *must* have a valid jid
     * - query can only be message, presence subscribe
     */
    if(!m_jid.IsValid()){
      return false;
    }
    return (IsValidQuery(m_query));
  }
  
  std::string JabUri::BuildUri()const
  {
    if(!IsValid()){
      OAPR_THROW(oapr::runtime_error,"invalid uri");
    }
    std::string uri("xmpp:");
    uri+=m_jid.Jid();

    /* now we add the query if there*/
    if(!m_query.empty()){
      uri+="?"+m_query;
      /* now add the parameters, but only if its not empty*/
      if(!m_parameters.empty()){
	cont_type::const_iterator pos;
	for(pos=m_parameters.begin();pos!=m_parameters.end();++pos){
	  uri+="&"+pos->first+"="+pos->second;
	}
      }
    }    
    return uri;    
  }
  void JabUri::ParseUri(const std::string& uri)
  {
    ParseJid(uri);
    ParseQuery(uri);
    ParseParams(uri);
  }
  bool JabUri::IsValidQuery(const std::string &qry)const
  {
    std::string::size_type found=JabUri::m_validQuery.find(qry);
    if(found == std::string::npos){
      return false;
    }else{
      return true;
    }
  }
  bool JabUri::IsValidParam(const std::string &param)const
  {
    /**\todo actually implement this :) */
    return true;
  }
  bool JabUri::ParseJid(const std::string &uri)
  {
    /* find the : and ?
     * ex: xmpp:foo@bar.com/there?message
     */
    typedef boost::tokenizer<boost::char_separator<char> > tokenizer;
    boost::char_separator<char> sep(":?");
    tokenizer tokens(uri,sep);
    tokenizer::iterator jiditer=++tokens.begin();
    if(jiditer == tokens.end()){
      OAPR_THROW(oapr::invalid_arg,"invalid uri, does not contain a JID");
    }

    m_jid.Jid(*jiditer);    
    return true;
  }
  bool JabUri::ParseQuery(const std::string &uri)
  {
    /* find the ? and &
     */
    typedef boost::tokenizer<boost::char_separator<char> > tokenizer;
    boost::char_separator<char> sep("?&");
    tokenizer tokens(uri,sep);
    tokenizer::iterator qryiter=++tokens.begin();
    if(qryiter == tokens.end()){
      return false;
    }
    m_query=*qryiter;
    return true;
  }
  bool JabUri::ParseParams(const std::string &uri)
  {
    /* - find &param=value
     */
    std::string param,value;
    typedef boost::tokenizer<boost::char_separator<char> > tokenizer;
    boost::char_separator<char> sep("&=");
    tokenizer tokens(uri,sep);
    tokenizer::iterator pos;
    for(pos=++tokens.begin();pos!=tokens.end();++pos){
      param=*pos;
      /* increment and do end check*/
      if( ++pos==tokens.end()){
	break;
      }
      value=*pos;
      m_parameters[param]=value;
    }
    return true;
  }
};
