/* -*- Mode: c++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 * OAjabber library
 * Contains objects to aid in Jabber client development and extend the
 * basic functionality of jabber.
 *
 * Copyright (C) 2001 Justin Kirby
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Justin Kirby <justink@rochester.rr.com>
 *
 *******************************************************************/
#ifndef OAJABBER_JABURI_H
#define OAJABBER_JABURI_H
#include <map>
#include "xstrm/jabjid.h"
#include "jabber/oajabber_exports.h"

namespace jabber
{
	class OAJABBER_API JabUri;
  class JabUri
  {
  public:
    JabUri(){}
    JabUri(const JabUri& ju)
      : m_jid(ju.m_jid),
        m_query(ju.m_query),
        m_parameters(ju.m_parameters)
    {}
    ~JabUri(){}

    bool operator==(const JabUri& ju)const;

    bool operator==(const JabJid& jj)const{
      return (jj==m_jid);
    }
    bool operator<(const JabUri& ju)const{
      if(&ju==this){return false;}
      return (Uri()<ju.Uri());
    }
    bool operator<(const JabJid& jj)const;

    std::string Uri()const{
      return BuildUri();
    }
    JabJid Jid()const{return m_jid;}
    std::string Query()const{return m_query;}
    std::string Parameter(const std::string &param)const;

    bool HasParameter(const std::string &param)const{
      return (m_parameters.find(param)!=m_parameters.end());
    }
    
    void Uri(const std::string &uri){
      ParseUri(uri);
    }
    void Jid(const JabJid& jj){m_jid=jj;}      
    void Query(const std::string& qry){m_query=qry;}
    void Parameter(const std::string &param, const std::string& value){
      m_parameters[param]=value;
    }

    bool IsNull()const;
    bool IsValid()const;
  private:
    std::string BuildUri()const;
    void ParseUri(const std::string& uri);
    bool IsValidQuery(const std::string &qry)const;
    bool IsValidParam(const std::string &param)const;
    bool ParseJid(const std::string &uri);
    bool ParseQuery(const std::string &uri);
    bool ParseParams(const std::string &uri);
  private:
    JabJid m_jid;
    std::string m_query;
    typedef std::map<std::string,std::string> cont_type;
    cont_type m_parameters;

    static const std::string m_validQuery;
    static const std::string m_validMessage;
    static const std::string m_validPresence;
    static const std::string m_validSubscribe;
    
  };
};
#endif
