/* -*- Mode: c++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 * OAjabber library
 * Contains objects to aid in Jabber client development and extend the
 * basic functionality of jabber.
 *
 * Copyright (C) 2004 Justin Kirby
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Justin Kirby <justink@rochester.rr.com>
 *
 *******************************************************************/
#ifndef OAJABBER_REGISTER_HPP
#define OAJABBER_REGISTER_HPP
#include "boost/shared_ptr.hpp"
#include "oapr/oapr_io.hpp"
#include "xstrm/jabevent.h"
#include "xstrm/jabsession.h"
#include "jabber/oajabber_exports.h"

namespace jabber
{
	class OAJABBER_API RegisterMgr;
  class RegisterMgr : public JabEvent
  {
  public:
    enum Register_State{
      Register_Undefined=0,
      Register_Init,
      Register_Connected,
      Register_Stream,
      Register_Query,
      Register_Request,
      Register_Created,
      Register_Error
    };
    typedef boost::shared_ptr<oapr::basic_io> io_ptr;
    typedef boost::shared_ptr<JabSession> session_ptr;
    typedef boost::shared_ptr<JabEvent> event_ptr;
  public:
    RegisterMgr(const JabJid& jid, const std::string& key, bool sync)
      : m_jid(jid)
        ,m_key(key)
        ,m_sync(sync)
        ,m_state(Register_Undefined)
        ,m_sig(oapr::pool_mgr::instance()->unique())
        ,m_mutex(oapr::pool_mgr::instance()->unique())
        ,m_hasError(false)
    {
    }
    RegisterMgr(const RegisterMgr& rm)
      : m_io(rm.m_io)
        ,m_session(rm.m_session)
        ,m_callback(rm.m_callback)
        ,m_wrapper(rm.m_wrapper)
        ,m_jid(rm.m_jid)
        ,m_key(rm.m_key)
        ,m_sync(rm.m_sync)
        ,m_state(rm.m_state)
        ,m_sig(oapr::pool_mgr::instance()->unique())
        ,m_mutex(oapr::pool_mgr::instance()->unique())
        ,m_hasError(rm.m_hasError)
        ,m_error(rm.m_error)
    {
    }
    ~RegisterMgr();

    Register_State State()const{
      /**@todo do we need a lock here?*/
      return m_state;
    }
    std::string Id()const{ return "jabber::RegisterMgr";}

    bool HasError()const{ return m_hasError; }
    JabError Error()const{ return m_error; }

    void Create(const io_ptr& ip, const event_ptr& ep);
    void Destroy();


    session_ptr Session()const{
      return m_session;
    }
    
  protected:
    JabEvent::HANDLE_MODES Exec(const JabJid& jid, JabPacket* pckt);
    JabEvent::HANDLE_MODES Query(const JabJid& jid, JabPacket* pckt);
    JabEvent::HANDLE_MODES Request(const JabJid& jid, JabPacket* pckt);
    JabEvent::HANDLE_MODES Created(const JabJid& jid, JabPacket* pckt);
    JabEvent::HANDLE_MODES Error(const JabJid& jid, JabPacket* pckt);
  private:
    void Connect();
    void Stream();
  private:
    io_ptr m_io;
    session_ptr m_session;
    event_ptr m_callback;
    event_ptr m_wrapper;
    JabJid m_jid;
    std::string m_key;
    bool m_sync;
    Register_State m_state;
    oapr::condition m_sig;
    oapr::mutex m_mutex;
    bool m_hasError;
    JabError m_error;
  };
};
#endif
