/* -*- Mode: c++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 * OAjabber library
 * Contains objects to aid in Jabber client development and extend the
 * basic functionality of jabber.
 *
 * Copyright (C) 2004 Justin Kirby
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Justin Kirby <justink@rochester.rr.com>
 *
 *******************************************************************/
#include "openaether.hpp"
#include <iostream>
#include "jabber/register.h"
#include "oapr/oapr_app.hpp"
#include "oapr/oapr_exception.hpp"
#include "oapr/oapr_exc_utils.hpp"
#include "oapr/oapr_pool_mgr.hpp"
#include "oapr/oapr_io_socketinfo.hpp"
#include "oapr/oapr_io_socket.hpp"
#include "jabber/ijabstream.h"

XERCES_CPP_NAMESPACE_USE

const int g_port=5222;

using jabber::JabJid;
using jabber::RegisterMgr;
using jabber::JabEvent;
using jabber::JabPacket;

class EchoEvent : public jabber::JabEvent
{
public:
  EchoEvent(){};
  ~EchoEvent(){}
  std::string Id()const{return "EchoEvent";}
protected:
  JabEvent::HANDLE_MODES Exec(const JabJid& jid, JabPacket* pckt)
  {
    std::cerr<<"\n\n"<<jid.Jid()<<"\n"<<pckt->xml()<<"\n";
    return JabEvent::CONTINUE;
  }
};

bool test_stream_fail()
{
  try{
    jabber::JabStream strm("jabber:client");
    std::cerr<<strm.xml();
  }catch(const oapr::exception_base& oe){
    std::cerr<<"good fail "<<oe.what()<<"\n";
    return true;
  }catch(const std::exception& e){
    std::cerr<<"bad fail "<<e.what()<<"\n";
    return false;
  }catch(...){
    std::cerr<<"bad fail\n";
    return false;
  }
  return false;  
}
bool test_stream_pass()
{
  try{
    jabber::JabStream strm("jabber:client");
    strm.Host("instant-help.net");

    std::cerr<<"\n\n"<<strm.xml()<<"\n\n";
  }OAPR_CATCH();
  return true;
}

int main(int argc, char*argv[])
{
  try{
    oapr::oapr_app app("reg-test");
    XMLPlatformUtils::Initialize();

    if(!test_stream_fail()){
      std::cerr<<"stream failed to fail\n";
      return 1;
    }
    if(!test_stream_pass()){
      std::cerr<<"stream failed\n";
      return 2;
    }
    {

      JabJid jid("foo@instant-help.net");
      RegisterMgr mgr(jid,"bar",true);
      boost::shared_ptr<EchoEvent> echo(static_cast<EchoEvent*>(0));

      /* establish socket*/
      oapr::pool_mgr::pool_ptr sockpl=oapr::pool_mgr::instance()->unique();
      oapr::sockinfo si(sockpl);
      si.create(jid.Domain().c_str(),APR_UNSPEC,g_port,0);
      boost::shared_ptr<oapr::basic_socket> io(new oapr::basic_socket(sockpl));
      io->create(si.family(),SOCK_STREAM);
      io->connect(si);
      io->option(APR_SO_NONBLOCK,0);


      mgr.Create(io,echo);	  
      //      mgr.Destroy();
    }

    XMLPlatformUtils::Terminate();
  }OAPR_PRINT(&std::cerr,return 1;);
  return 0;
}
