/********************************************************************
 * OpenAether jabDB
 * Transport to allow jabber access to rdbms 
 *
 * Copyright (C) 2002 Justin Kirby
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Justin Kirby <justink@rochester.rr.com>
 *
 *******************************************************************/
#include "framework/first.h"
#include "openaether.hpp"
#include <xercesc/util/XMLString.hpp>
#include <xercesc/dom/DOMElement.hpp>
#include "boost/smart_ptr.hpp"
#include "oapu/oapr_logstream.hpp"
#include "framework/xmlutil.hpp"
#include "jda/builder.hpp"
#include "jda/builder_set.hpp"
#include "jda/builder_get.hpp"

XERCES_CPP_NAMESPACE_USE
namespace oajabdb
{
  bool Builder::CanBuild(DOMNode*const nd)const
  {
    /* requirements for this:
     * - iq that has children
     * - child is database and ns
     */
    boost::scoped_array<XMLCh> iqName(XMLString::transcode("iq"));
    boost::scoped_array<XMLCh> dbName(XMLString::transcode("database"));
    boost::scoped_array<XMLCh> uri(XMLString::transcode("http://openaether.org/projects/jabber_database.html"));


    if(nd->hasChildNodes() &&
       !XMLString::compareString(iqName.get(),nd->getNodeName())){
      DOMNode*db=nd->getFirstChild();

      if(!XMLString::compareString(dbName.get(),db->getNodeName()) &&
	 !XMLString::compareString(uri.get(),db->getNamespaceURI())){
	return true;
      }
    }
    return false;
  }
  jabber::JabPacket* Builder::Build(DOMNode*const nd)
  {
    if(!nd){/**\todo throw config error*/return 0;}

    /* first thing we should do is clean this!! */
    oa::XMLUtils().Clean(nd);
    DOMNode* iq = nd;

    //determine whether it has a <query>
    boost::scoped_array<XMLCh> tagName(XMLString::transcode("query"));
    bool hasQuery = oa::XMLUtils().HasElem(iq,tagName.get());

    //determine the type and build appropriate builder
    std::string type;
    boost::scoped_ptr<jabber::JabBuild> builder;
    GetType(type,iq);

    if(type == "get" && !hasQuery){
      builder.reset(new BuildGet);
    }else if(type == "set" && !hasQuery){
      builder.reset(new BuildSet);
    }else if(hasQuery){
      builder.reset(new BuildQuery);
    }else{
      /**\todo type is result, which we don't send any out yet*/
      return 0;
    }
    p_packet.reset(builder->Build(iq));
    return p_packet.release();
  }
};
