/********************************************************************
 * OpenAether jabDB
 * Transport to allow jabber access to rdbms 
 *
 * Copyright (C) 2002 Justin Kirby
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Justin Kirby <justink@rochester.rr.com>
 *
 *******************************************************************/
#include "framework/first.h"
#include "openaether.hpp"
#include <xercesc/util/XMLString.hpp>
#include <xercesc/dom/DOMElement.hpp>
#include <xercesc/dom/DOMNodeList.hpp>
#include "framework/xmlutil.hpp"
#include "jda/builder_db.hpp"

XERCES_CPP_NAMESPACE_USE
namespace oajabdb
{

  DOMNode*const DbBuild::DatabaseName(DOMNode*const nd,std::string &db)
  {
    if(nd->getNodeType() != DOMNode::ELEMENT_NODE){
      return 0;
    }
    /**\todo should the "database" be brought out into a const/define?*/
    boost::scoped_array<XMLCh> tagName(XMLString::transcode("database"));
    DOMNodeList* dblist = ((DOMElement*)nd)->getElementsByTagName(tagName.get());
    if(dblist->getLength() <= 0){
      return 0;
    }
    //there should only be one.. we will ignore multiples
    DOMNode* dbn = dblist->item(0);
    /**\todo again should this be brought out into a const/define?*/
    boost::scoped_array<XMLCh> attrName(XMLString::transcode("name"));
    const XMLCh* name = ((DOMElement*)dbn)->getAttribute(attrName.get());
    oa::XMLUtils().DomstrToStd(db,name);
    return dbn;
  }
  int DbBuild::TableName(DOMNode*const nd,std::string &tb)
  {
    if(nd->getNodeType() != DOMNode::ELEMENT_NODE){
      return 1;
    }

    boost::scoped_array<XMLCh> attrName(XMLString::transcode("name"));
    const XMLCh* name = ((DOMElement*)nd)->getAttribute(attrName.get());
    oa::XMLUtils().DomstrToStd(tb,name);
    return 0;
  }
  int DbBuild::TableCount(DOMNode*const nd)
  {
    if(nd->getNodeType() !=  DOMNode::ELEMENT_NODE){
      return -1;
    }

    boost::scoped_array<XMLCh> tagName(XMLString::transcode("table"));
    DOMNodeList* tblist = ((DOMElement*)nd)->getElementsByTagName(tagName.get());
    return tblist->getLength();
  }

  int DbBuild::BuildColumns(DOMNode*const nd)
  {
    std::string sname,sval;
    DOMNode *txtnd,*col;
    DOMNodeList* colList;

    colList = nd->getChildNodes();
    unsigned int idx;
    for(idx=0; idx<colList->getLength(); ++idx){
      col = colList->item(idx);

      //make sure we are dealing with elements
      if(col->getNodeType() != DOMNode::ELEMENT_NODE){
#ifdef DEBUG
	short type = col->getNodeType();/* for debug purposes only*/
	type=0;
#endif
	continue;
      }
      /**\todo should this be dynamic cast?*/
      DOMElement* elemCol = ((DOMElement*)col);
      const XMLCh* elemName = elemCol->getTagName();
      std::string strElemName;
      oa::XMLUtils().DomstrToStd(strElemName,elemName);
      if(strElemName != "col"){
	continue;//we only want to deal with col elements
      }
      boost::scoped_array<XMLCh> attrName(XMLString::transcode("name"));
      const XMLCh* name = elemCol->getAttribute(attrName.get());
      const XMLCh* val;
      if(elemCol->hasChildNodes()){
	txtnd = elemCol->getFirstChild();
	if(txtnd->getNodeType() == DOMNode::TEXT_NODE){
	  val = txtnd->getNodeValue();	  
	}else if(txtnd->getNodeType() == DOMNode::CDATA_SECTION_NODE){
	  val = ((DOMCDATASection*)txtnd)->getData();
	}else{
	  val = 0;
	}
      }else{
	val = 0;
      }
	

      oa::XMLUtils xutil;
      xutil.DomstrToStd(sname,name);
      if(val){
	xutil.DomstrToStd(sval,val);
      }else{
	sval.clear();
      }

      p_packet->Cols(sname,sval);
    }
  
    return 0;
  }
  int DbBuild::BuildClause(DOMNode*const nd)
  {
    std::string sname,sop,sconj,sval;
    const XMLCh *name=0,*op=0,*conj=0,*val=0;
    if(nd->getNodeType()!=DOMNode::ELEMENT_NODE){
      return 1;
    }
    DOMElement* table = ((DOMElement*)nd);
    boost::scoped_array<XMLCh> tagName(XMLString::transcode("where"));
    DOMNodeList* whereList = table->getElementsByTagName(tagName.get());
    unsigned int whereIdx;
    for(whereIdx=0;whereIdx<whereList->getLength();++whereIdx){
      DOMNode* clause = whereList->item(whereIdx);

      DOMNodeList* colList = clause->getChildNodes();
      unsigned int colIdx;
      for(colIdx=0; colIdx<colList->getLength();++colIdx){
	//check for name is col
	DOMNode* whereChild = colList->item(colIdx);
	if(whereChild->getNodeType()!=DOMNode::ELEMENT_NODE){
	  continue;
	}
	DOMElement* elemCol = ((DOMElement*)whereChild);
	//determine the name
	const XMLCh* elemColName = elemCol->getTagName();
	std::string strElemColName;
	oa::XMLUtils().DomstrToStd(strElemColName,elemColName);
	if(strElemColName != "col"){
	  continue;
	}
	DOMNode* valnd = elemCol->getFirstChild();
	if(valnd->getNodeType() == DOMNode::TEXT_NODE){
	  boost::scoped_array<XMLCh> an(XMLString::transcode("name"));
	  name = elemCol->getAttribute(an.get());

	  an.reset(XMLString::transcode("op"));
	  op = elemCol->getAttribute(an.get());

	  an.reset(XMLString::transcode("conj"));
	  conj = elemCol->getAttribute(an.get());

	  val = valnd->getNodeValue();
	    
	  oa::XMLUtils xutils;
	  xutils.DomstrToStd(sname,name);
	  xutils.DomstrToStd(sop,op);
	  xutils.DomstrToStd(sconj,conj);
	  xutils.DomstrToStd(sval,val);
	    
	  p_packet->Clause(sname,sop,sconj,sval);
	}
      }
    }
    return 0;
  }  
};
