/********************************************************************
 * OpenAether jabDB
 * Transport to allow jabber access to rdbms 
 *
 * Copyright (C) 2002 Justin Kirby
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Justin Kirby <justink@rochester.rr.com>
 *
 *******************************************************************/
#ifndef JABDB_JABBER_BUILDER_DB_HPP
#define JABDB_JABBER_BUILDER_DB_HPP
#include "jda/oadbjab_exports.hpp"

#include <xercesc/dom/DOMNode.hpp>
#include "xstrm/jabbuild.h"
#include "jda/jabdb_packet.hpp"
namespace oajabdb
{
  class OADBJAB_API DbBuild;
  class DbBuild : public jabber::JabBuild
  {
  public:
    DbBuild(){}
    virtual ~DbBuild(){}
  protected:
    /**
     * Finds the name of the database
     * @param db Name of the database if found
     * @param nd Node to search in for <database>
     * @return zero if found, non-zero otherwise
     */
    virtual xercesc::DOMNode*const DatabaseName(xercesc::DOMNode*const nd,std::string &db);
    /**
     * Finds the name of the table
     * @param tb Name of the table if found
     * @param nd Node to search in for <table>
     * @return zero if found, non-zero otherwise
     */
    virtual int TableName(xercesc::DOMNode*const nd,std::string &tb);
    /**
     * Get the number of Table elements
     * @return number of <table> elements, or -1 if error occurred
     */
    virtual int TableCount(xercesc::DOMNode*const nd);
    /**
     * Get each column and insert into the packet
     * @return Number of columns processed
     */
    virtual int BuildColumns(xercesc::DOMNode*const nd);
    /**
     * Finds and processes the <where> clause if
     * one exists.
     * @return zero if found/processed, non-zero otherwise
     */
    virtual int BuildClause(xercesc::DOMNode*const nd);
  protected:
    std::auto_ptr<PcktJabDb> p_packet;
  };
};
#endif
