/********************************************************************
 * OpenAether jabDB
 * Transport to allow jabber access to rdbms 
 *
 * Copyright (C) 2002 Justin Kirby
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Justin Kirby <justink@rochester.rr.com>
 *
 *******************************************************************/
#include "framework/first.h"
#include "openaether.hpp"
#include "boost/smart_ptr.hpp"
#include "framework/xmlutil.hpp"
#include <xercesc/util/XMLString.hpp>
#include "jda/builder_get.hpp"
#include "jda/builder_schema.hpp"
#include "jda/builder_select.hpp"

#include "jda/packet_cont.hpp"

XERCES_CPP_NAMESPACE_USE
namespace oajabdb
{
  bool BuildGet::CanBuild(DOMNode*const nd)const
  {
    /**\todo implement this*/
    return false;
  }

  jabber::JabPacket* BuildGet::Build(DOMNode*const nd)
  {
    /*we can assume all the pleasantries are over with... 
      e.g. checking for null etc.. so just get on with it
    */

    std::string dbname;

    boost::scoped_ptr<DbBuild> builder;
    int tableCount = TableCount(nd);
    DOMNode* dbnode = DatabaseName(nd,dbname);

    /* its possible to not have any tables, in that case its 
     * definitely a schema request. So just build the schema
     */
    if(tableCount <= 0){
      //note: passing a null node to BuildSchema forces checks
      builder.reset(new BuildSchema);
      PcktJabDb *pckt = dynamic_cast<PcktJabDb*>(builder->Build(0));
      if(pckt){
	std::auto_ptr<PcktCont> cont(new PcktCont);
	cont->Add(pckt);
	cont->Database(dbname);
	p_packet.reset(cont.release());
      }
    }else{
      //create a nodelist of tables
      boost::scoped_array<XMLCh> tagName(XMLString::transcode("table"));
      DOMNodeList* tableList=((DOMElement*)dbnode)->getElementsByTagName(tagName.get());

      //ok so now we need to create a packet container
      std::auto_ptr<PcktCont> cont(new PcktCont);
      for(unsigned int idx=0; idx<tableList->getLength();idx++){
	DOMNode* table = tableList->item(idx);
	// basically if it has <col> elements, then its a select
	// otherwise its a schema request
	boost::scoped_array<XMLCh> tagName(XMLString::transcode("col"));
	bool hasCol = oa::XMLUtils().HasElem(table,tagName.get());
	if(hasCol){
	  builder.reset(new BuildSelect);
	}else{
	  builder.reset(new BuildSchema);
	}
	//can we go up?
	PcktJabDb *pckt=dynamic_cast<PcktJabDb*>(builder->Build(table));
	if(pckt){
	  cont->Add(pckt);	
	}else{
	  /**\todo throw*/
	}
      }
      cont->Database(dbname);
      p_packet.reset(cont.release());
    }
    //now we set the top level info
    std::string to,from,type,id;
    GetTo(to,nd);
    GetFrom(from,nd);
    GetType(type,nd);
    GetId(id,nd);
    p_packet->To(to);
    p_packet->From(from);
    p_packet->Type(type);
    p_packet->Id(id);

    return p_packet.release();
  }
};
