/********************************************************************
 * OpenAether jabDB
 * Transport to allow jabber access to rdbms 
 *
 * Copyright (C) 2002 Justin Kirby
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Justin Kirby <justink@rochester.rr.com>
 *
 *******************************************************************/
#include "framework/first.h"
#include "openaether.hpp"
#include "boost/smart_ptr.hpp"
#include "framework/xmlutil.hpp"
#include <xercesc/util/XMLString.hpp>
#include "jda/builder_set.hpp"
#include "jda/builder_update.hpp"
#include "jda/builder_insert.hpp"
#include "jda/builder_delete.hpp"
#include "jda/packet_cont.hpp"

XERCES_CPP_NAMESPACE_USE
namespace oajabdb
{
  bool BuildSet::CanBuild(DOMNode*const nd)const
  {
    /**\todo implement this*/
    return false;
  }
  jabber::JabPacket* BuildSet::Build(DOMNode*const nd)
  {
    /*
      we can assume that everything is checked already in Builder
      so we do not have to perform null checks, etc...
     */
    std::string dbName;
    boost::scoped_ptr<DbBuild> builder;
    int tableCount = TableCount(nd);
    DOMNode*const dbNode = DatabaseName(nd,dbName);

    std::auto_ptr<PcktCont> cont(new PcktCont);
    if(tableCount <= 0){
      /**\todo send error out*/
    }else{
      boost::scoped_array<XMLCh> tagName(XMLString::transcode("table"));
      DOMNodeList* tableList = ((DOMElement*)dbNode)->getElementsByTagName(tagName.get());

      unsigned int idx;
      for(idx=0;idx<tableList->getLength();++idx){
	DOMNode* table = tableList->item(idx);

	//does the table have col elements?
	bool hasCol = HasColumns(table);
	tagName.reset(XMLString::transcode("where"));
	bool hasWhere = oa::XMLUtils().HasElem(table,tagName.get());
	if(hasCol){
	  if(hasWhere){
	    builder.reset(new BuildUpdate);
	  }else{
	    builder.reset(new BuildInsert);
	  }
	}else{
	  builder.reset(new BuildDelete);
	}
	jabber::JabPacket *jabPacket = builder->Build(table);
	PcktJabDb *dbPacket = dynamic_cast<PcktJabDb*>(jabPacket);
	if(dbPacket){
	  cont->Add(dbPacket);
	}else{
	  /**\todo throw or figure out how to handle this*/
	  delete jabPacket;
	}
      }
      
    }
    cont->Database(dbName);
    p_packet.reset(cont.release());
    std::string to,from,type,id;
    GetTo(to,nd);
    GetFrom(from,nd);
    GetType(type,nd);
    GetId(id,nd);
    p_packet->To(to);
    p_packet->From(from);
    p_packet->Type(type);
    p_packet->Id(id);


    return p_packet.release();
  }


  bool BuildSet::HasColumns(DOMNode*const nd)
  {
    if(nd->getNodeType()!=DOMNode::ELEMENT_NODE){
      return false;
    }

    // can only check the immediate children
    unsigned int idx;
    DOMNodeList *nodeList = nd->getChildNodes();
    for(idx=0; idx<nodeList->getLength(); ++idx){
      DOMNode* child = nodeList->item(idx);
      if(child->getNodeType()==DOMNode::ELEMENT_NODE){
	DOMElement* elem = ((DOMElement*)child);
	const XMLCh* name = elem->getTagName();
	std::string strName;
	oa::XMLUtils().DomstrToStd(strName,name);
	if(strName == "col"){
	  return true;
	}
      }
    }
    return false;
  }
};
