/********************************************************************
 * OpenAether jabDB
 * Transport to allow jabber access to rdbms 
 *
 * Copyright (C) 2002 Justin Kirby
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Justin Kirby <justink@rochester.rr.com>
 *
 *******************************************************************/
#ifndef OAJABDB_JABDB_PACKET_HPP
#define OAJABDB_JABDB_PACKET_HPP

#include <xercesc/dom/DOMNode.hpp>
#include "boost/smart_ptr.hpp"
#include "jabber/ijabiq.h"
#include "jda/jabdb_sql_where.hpp"

namespace oajabdb
{
  class PcktJabDb : public jabber::JabIq
  {
  public:
    typedef std::vector<std::string> colname_type;
    typedef std::vector<std::string> colval_type;

    enum Packet_Category{Category_Schema=0,
			 Category_Sql,
			 Category_Admin,
			 Category_Embed,
			 Category_Error};

    enum Packet_Action{Action_Schema=0,
		       Action_Select,
		       Action_Insert,
		       Action_Update,
		       Action_Delete,
		       Action_Create,
		       Action_Drop,
		       Action_Result,
		       Action_Error};
  public:
    PcktJabDb(std::string clsid)
      : jabber::JabIq(clsid){}

		
    PcktJabDb(const PcktJabDb& pjd)
      : jabber::JabIq(pjd),
	m_database(pjd.m_database),
	m_table(pjd.m_table){}
    virtual ~PcktJabDb(){}

    //generic information derived classes must provide
    virtual Packet_Category Category()const=0;
    virtual Packet_Action Action()const=0;

    virtual const char*const Uri()const{
      return "xmpp:xmlns=http://jabber.org/protocol/jda";
    }
    virtual std::string Xmlns()const{
      /**\todo change if the xmlns ever changes, e.g. jep-0043 is active*/
      return "http://openaether.org/projects/jabber_database.html";
    }

    /**
     * Set the database name
     * @param db The name of the database
     */
    virtual void Database(const std::string &db){
      m_database=db;
    }
    /**
     * Set the table or object name
     * @param tbl The name of the table or object
     */
    virtual void Table(const std::string &tbl){
      m_table = tbl;
    }
    /**
     * Get the database name
     * @return Name of the database
     */
    virtual std::string Database() const{
      return m_database;
    }
    /**
     * Get the table or object name
     * @return Name of the table or object
     */
    virtual std::string Table() const{
      return m_table;
    }
    
    /**
     * Returns a new instance of itself
     */
    virtual PcktJabDb* Clone()=0;


    virtual void Cols(const std::string& col, 
		      const std::string& val)=0;

    virtual unsigned int Length()const=0;      
    virtual std::string Cols(int idx)const=0;
    virtual std::string Vals(int idx)const=0;
    virtual void Clause(const std::string& col,
			const std::string& op,
			const std::string& val,
			const std::string& conj)=0;

    virtual Where Clause()const=0;

  protected:
    virtual xercesc::DOMNode*const BuildTable()=0;
  private:
    std::string m_database;
    std::string m_table;

    /**\bug this creates a really nasty dependency
     * and should be resolved some other way
     * possibly by making the BuildTable() public
     */
    friend class PcktCont;
  };
};

#endif
