/********************************************************************
 * OpenAether jabDB
 * Transport to allow jabber access to rdbms 
 *
 * Copyright (C) 2002 Justin Kirby
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Justin Kirby <justink@rochester.rr.com>
 *
 *******************************************************************/
#include "framework/first.h"
#include "openaether.hpp"
#include <xercesc/util/XMLString.hpp>
#include <xercesc/dom/DOMElement.hpp>
#include "framework/xmlutil.hpp"
#include "framework/xmlstr.hpp"
#include "framework/xerces_xmlns.hpp"
#include "jda/packet_cont.hpp"

using jabber::JabPacket;

XERCES_CPP_NAMESPACE_USE
namespace oajabdb
{
  void PcktCont::To(const std::string &to)
  {
    cont_type::iterator pos;
    for(pos=m_container.begin(); pos!=m_container.end();++pos){
      pos->get()->To(to);
    }
    m_to=to;
  }
  void PcktCont::From(const std::string &from)
  {
    cont_type::iterator pos;
    for(pos=m_container.begin(); pos!=m_container.end();++pos){
      pos->get()->From(from);
    }
    m_from=from;
  }
  void PcktCont::Id(const std::string &id)
  {
    cont_type::iterator pos;
    for(pos=m_container.begin(); pos!=m_container.end();++pos){
      pos->get()->Id(id);
    }
    m_id=id;
  }
  void PcktCont::Type(const std::string &type)
  {
    cont_type::iterator pos;
    for(pos=m_container.begin(); pos!=m_container.end();++pos){
      pos->get()->Type(type);
    }
    m_type=type;
  }
  void PcktCont::Database(const std::string &db)
  {
    cont_type::iterator pos;
    m_database=db;
    for(pos=m_container.begin(); pos!=m_container.end();++pos){
      pos->get()->Database(db);
    }
    m_database=db;
  }
  std::string PcktCont::xml()
  {
    //simple translation of GetXml to a string
    std::string ret;
    DOMNode* node = BuildXml();
    if(node){
      oa::XMLUtils().Print(ret,node);
    }
    return ret;
  }
  DOMNode*const PcktCont::BuildXml()
  {
    /* The following needs to be built
     * <iq id="id" type="type" from="me" to="to">
     *   <database xmlns="xmlns" name="dbname">
     *     children containers go here...
     *   </database>
     * </iq>
     */



    oa::xmlstr<XMLCh,char> tagName,
      tagValue,
      attrName,
      attrValue;

    DOMDocument *doc = oa::XMLUtils().Document(DocUri(),"oa");


    tagName.set("database");
    tagValue.set(Xmlns().c_str());
    oa::dom_ptr<DOMElement> dbElem(doc->createElementNS(tagValue.get(),
							tagName.get()));


    /* forcing the xmlns attribute to exist
     * because of the lame formatter
     */
    oa::XercesXmlns()(dbElem.get(),Xmlns().c_str());

    /*if the type is an error, then build that
     * however, if we have children, then we must let 
     * them have their say
     */
    if(Type()=="error" && m_container.size()<=0){
      oa::dom_ptr<DOMNode> errorElem(BuildError());
      dbElem->appendChild(errorElem.release());
    }else{
      cont_type::iterator pos=m_container.begin();
      for(;pos!=m_container.end();++pos){
	oa::dom_ptr<DOMNode> subNode((*pos)->BuildTable());
	if(subNode.get()){
	   dbElem->appendChild(subNode.release());
	}
      }
    }

    /* create the base iq element
     */
    oa::dom_ptr<DOMNode> iqElem(BuildIq());
    iqElem->appendChild(dbElem.release());
    /*
    m_node=iqElem.release();
    return m_node.get();
    */
    return iqElem.release();
  }
};
