/********************************************************************
 * OpenAether jabDB
 * Transport to allow jabber access to rdbms 
 *
 * Copyright (C) 2002 Justin Kirby
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Justin Kirby <justink@rochester.rr.com>
 *
 *******************************************************************/
#ifndef OAJABDB_JABBER_JABDB_PACKET_CONT_HPP
#define OAJABDB_JABBER_JABDB_PACKET_CONT_HPP

#include "jda/oadbjab_exports.hpp"
#include <xercesc/dom/DOMNode.hpp>
#include "boost/smart_ptr.hpp"
#include "jabber/ijabiq.h"
#include "jda/jabdb_packet.hpp"

namespace oajabdb
{
  class OADBJAB_API PcktCont;
  class PcktCont : public jabber::JabIq
  {
  public:
    typedef boost::shared_ptr<PcktJabDb> elem_type;
    typedef std::vector<elem_type> cont_type;
  public:
    PcktCont()
      : jabber::JabIq("pcktcont"){}
    PcktCont(const PcktCont& pc)
      : jabber::JabIq(pc),
	m_container(pc.m_container),
	m_database(pc.m_database),
	m_xml(pc.m_xml){}
    virtual ~PcktCont(){}
	       
    virtual const char*const Uri()const{
      return "xmpp:xmlns=http://jabber.org/protocol/jda";
    }
    virtual std::string Xmlns(){
      /**\todo change if the xmlns ever changes, e.g. jep-0043 is active*/
      return "http://openaether.org/projects/jabber_database.html";
    }

    using JabIq::To;
    virtual void To(const std::string &to);
    using JabIq::From;
    virtual void From(const std::string &from);
    using JabIq::Id;
    virtual void Id(const std::string &id);
    using JabIq::Type;
    virtual void Type(const std::string &type);

    virtual void Database(const std::string &db);
    virtual std::string Database() const{
      return m_database;
    }

    /**
     * Clones itself
     */
    virtual PcktCont* Clone(){
      return new PcktCont(*this);
    }

    /**
     * The number of children this contains
     * @return the length of the children 
     */
    virtual int Length() const{
      return m_container.size();
    }
    /**
     * Return the pointer to the child at idx
     * There is no bounds checking except for in the vector
     * so the vector will throw an out_of_bounds exception
     * You do <b>not</b> own the pointer, the reason its const
     *
     * @param idx The index to retreive
     * @return Pointer to the packet
     */
    virtual PcktJabDb* const Get(int idx) const{
      return m_container[idx].get();
    }
    /**
     * Add a packet as a child to this
     * 
     */
    virtual void Add(PcktJabDb* pjd){
      if(pjd){
	m_container.push_back(elem_type(pjd));
      }
    }
    
    /**
     * Remove the packet at index
     *
     */
    virtual void Remove(int idx){
      m_container.erase((m_container.begin()+idx));
    }
    virtual std::string xml();
    virtual xercesc::DOMNode*const BuildXml();

  private:
    cont_type m_container;
    std::string m_database;
    xercesc::DOMNode* m_xml;
  };
};

#endif
