/********************************************************************
 * OpenAether jabDB
 * Transport to allow jabber access to rdbms 
 *
 * Copyright (C) 2002 Justin Kirby
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Justin Kirby <justink@rochester.rr.com>
 *
 *******************************************************************/
/*****--== Change Log --== *****
 * 2002-09-20
 * Changed PcktDelete::Category and PcktDelete::Action to return
 * values based off of jabber::JabPacket::m_hasError value. This
 * is mainly to accomodate bug #59
 */

#ifndef OAJABDB_JABDB_PACKET_DELETE_HPP
#define OAJABDB_JABDB_PACKET_DELETE_HPP

#include "jda/oadbjab_exports.hpp"
#include "jda/jabdb_sql_where.hpp"
#include "jda/jabdb_packet.hpp"

namespace oajabdb
{
  class OADBJAB_API PcktJabDb;
  class OADBJAB_API PcktDelete;
  class PcktDelete : public PcktJabDb
  {
  public:
    PcktDelete()
      : PcktJabDb("pcktdelete"){}

    PcktDelete(const PcktDelete& pd)
      : PcktJabDb(pd),
	m_where(pd.m_where){}

    virtual ~PcktDelete(){}

    virtual const char*const Uri()const{
      return "xmpp:xmlns=http://jabber.org/protocol/jda?delete";
    }
    virtual PcktJabDb::Packet_Category Category()const{
      return m_hasError ? PcktJabDb::Category_Error : PcktJabDb::Category_Sql;
    }
    virtual PcktJabDb::Packet_Action Action()const{
      return m_hasError ? PcktJabDb::Action_Error :  PcktJabDb::Action_Delete;
    }


    virtual PcktJabDb* Clone(){
      return new PcktDelete(*this);
    }

    void Cols(const std::string &col, const std::string &val){}
    void Clause(const std::string &col,
		const std::string &op,
		const std::string &val,
		const std::string &conj){
      m_where.m_clause.push_back(m_where.MakeQualifier(col,op,val,conj));
    }

    virtual unsigned int Length()const{
      return 0;
    }
    virtual std::string Cols(int idx)const{
      return std::string("");
    }
    virtual std::string Vals(int idx)const{
      return std::string("");
    }
    Where Clause()const{
      return m_where;
    }
    std::string xml();
    virtual xercesc::DOMNode*const BuildXml();
  protected:
    virtual xercesc::DOMNode*const BuildTable();
    void BuildResult();
    void BuildSet();
  private:
    Where m_where;
  };
};

#endif
