/********************************************************************
 * OpenAether Framework
 * Contains objects to aid in the development of applications
 * mainly focusing on test applications and daemons
 *
 * Copyright (C) 2002 Justin Kirby
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Justin Kirby <justink@rochester.rr.com>
 *
 *******************************************************************/
/*****--== Change Log --== *****
 * 2002-09-20
 * Changed PcktDelete::Category and PcktDelete::Action to return
 * values based off of jabber::JabPacket::m_hasError value. This
 * is mainly to accomodate bug #59
 */
#ifndef OAJABDB_JABDB_PACKET_SCHEMA_HPP
#define OAJABDB_JABDB_PACKET_SCHEMA_HPP

#include "jda/oadbjab_exports.hpp"
#include "jda/jabdb_packet.hpp"

namespace oajabdb
{
  class OADBJAB_API PcktJabDb;
  class OADBJAB_API PcktSchema;
  class PcktSchema : public PcktJabDb
  {
  public:
    struct Column_Desc{
      std::string name;
      std::string type;
      std::string size;
    };
    typedef std::vector<Column_Desc> col_schema;
  public:
    PcktSchema()
      : PcktJabDb("pcktschema"){}

    PcktSchema(const PcktSchema& ps)
      : PcktJabDb(ps),
	m_schema(ps.m_schema),
	m_permission(ps.m_permission)
    {
    }
    virtual ~PcktSchema(){}

    virtual const char*const Uri()const{
      return "xmpp:xmlns=http://jabber.org/protocol/jda?schema";
    }
    virtual PcktJabDb::Packet_Category Category()const{
      return m_hasError ? PcktJabDb::Category_Error : PcktJabDb::Category_Schema;
    }
    virtual PcktJabDb::Packet_Action Action()const{
      return m_hasError ? PcktJabDb::Action_Error : PcktJabDb::Action_Schema;
    }

    virtual PcktJabDb* Clone(){
      return new PcktSchema(*this);
    }


    /**
     */
    virtual void Permission(const std::string &perm){
      m_permission=perm;
    }
    /**
     */
    virtual std::string Permission()const{
      return m_permission;
    }
    /**
     * Simply add the col
     * @param col the name of the column
     * @param val permission of the col
     */
    void Cols(const std::string &col, const std::string &val){
	  /**\note would do initializer list, except vc6 doesnt like it */
      Column_Desc cd;cd.name=col;cd.type=val;
      m_schema.push_back(cd);
    }
    void Cols(const std::string &col, 
	      const std::string &val, 
	      const std::string &sz){
	  /**\note would do initializer list, except vc6 doesnt like it */
      Column_Desc cd;cd.name=col;cd.type=val;cd.size=sz;
      m_schema.push_back(cd);
    }
    
    /**
     * There is no clause in a schema, so this does nothing
     */
    void Clause(const std::string &col,
		const std::string &op,
		const std::string &val,
		const std::string &conj){
      //nothing to be done here
    }
    virtual unsigned int Length()const{
      return m_schema.size();
    }

    /**
     * 
     */
    virtual std::string Cols(int idx)const{
      return m_schema[idx].name;
    }
    /**
     *
     */
    virtual std::string Vals(int idx)const{
      return m_schema[idx].type;
    }
    /**
     *
     */
    virtual std::string Size(int idx)const{
      return m_schema[idx].size;
    }
    /**
     * There is no clause in schema, so the Where is empty
     */
    Where Clause()const{
      return Where();
    }
    std::string xml();
    virtual xercesc::DOMNode*const BuildXml();

  protected:
    virtual xercesc::DOMNode*const BuildTable();
    bool BuildGet();
    bool BuildResult();

  protected:
    col_schema m_schema;
    std::string m_permission;
  };
};

#endif
