/********************************************************************
 * OpenAether jabDB
 * Transport to allow jabber access to rdbms 
 *
 * Copyright (C) 2002 Justin Kirby
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Justin Kirby <justink@rochester.rr.com>
 *
 *******************************************************************/
#include "framework/first.h"
#include "openaether.hpp"
#include <xercesc/util/XMLString.hpp>
#include "oapr/oapr_exception.hpp"
#include "oapr/oapr_exc_utils.hpp"
#include "framework/xmlutil.hpp"
#include "framework/xmlstr.hpp"
#include "framework/xerces_xmlns.hpp"
#include "jda/packet_select.hpp"

XERCES_CPP_NAMESPACE_USE
namespace oajabdb
{
  std::string PcktSelect::xml()
  {
    /*
     * ok we have to loop through all the col/val and build up the xml
     * this is going to be a pain in the asdf
     *
     * xml looks like:
     * <table name="tablename">
     *   <col name="colname">value</col>
     *   ....
     * </table>
     */
    oa::dom_ptr<DOMNode> built(BuildXml());
    std::string ret;
    if(built.get()){
      oa::XMLUtils().Print(ret,built.get());
    }
    return ret;
  }
  DOMNode*const PcktSelect::BuildXml()
  {
    /* if this is called, assume client or other implementation*/
    oa::xmlstr<XMLCh,char> tagName("database"),
      attrName("xmlns"),
      tagValue,
      attrValue(Xmlns().c_str());

    DOMDocument *doc=oa::XMLUtils().Document(DocUri(),"oa");
    /* build the table elem before iq
     * this way we get the type set if an error occurs
     */
    oa::dom_ptr<DOMNode> tableElem(BuildTable());

    /* create the database element
     * force it into the namespace
     */
    tagValue.set(Xmlns().c_str());
    oa::dom_ptr<DOMElement> dbElem(doc->createElementNS(tagValue.get(),
							tagName.get()));


    /* since the Formatter sucks, manually adding the 
     * xmlns attribute
     */
    oa::XercesXmlns()(dbElem.get(),Xmlns().c_str());

    /* the name attribute is required on the database
     * element. throw a logic error if its not there
     */
    std::string dbname=Database();
    if(dbname.empty()){
      OAPR_THROW(oapr::logic_error,"Required database attribute name is empty");
    }

    attrName.set("name");
    attrValue.set(dbname.c_str());
    dbElem->setAttribute(attrName.get(),attrValue.get());



    /* now we build the iq element */
    oa::dom_ptr<DOMNode> iqElem(BuildIq());

    /* append children in proper order*/
    dbElem->appendChild(tableElem.release());
    iqElem->appendChild(dbElem.release());
    /*
    m_node=iqElem.release();
    return m_node.get();
    */
    return iqElem.release();
  }
  DOMNode*const PcktSelect::BuildTable()
  {

    oa::xmlstr<XMLCh,char> tagName,
      tagValue,
      attrName,
      attrValue;

    DOMDocument *doc=oa::XMLUtils().Document(DocUri(),"oa");

    /* create the table element
     */
    tagName.set("table");
    oa::dom_ptr<DOMElement> tableElem(doc->createElement(tagName.get()));

    /* add the name attribute, 
     * this is required, so throw if empty
     */
    std::string name=Table();
    if(name.empty()){
      OAPR_THROW(oapr::logic_error,"Required table attribute name is empty");
    }

    attrName.set("name");
    attrValue.set(name.c_str());
    tableElem->setAttribute(attrName.get(),attrValue.get());


    /* get the type
     * - only allowed values are get, result, error
     * - throwing if type is set or anything else
     */
    std::string type=Type();
    if(type=="get" || type=="result"){
      /* iterator through all the columns 
       * and append each empty col element to the table
       * note: the only diff of get<>result is that result
       * needs a value as a child element of the col
       */
      colval_type::iterator pos=m_colval.begin();
      tagName.set("col");
      attrName.set("name");

      for(;pos!=m_colval.end();++pos){
	/* create the col element*/
	oa::dom_ptr<DOMElement> colElem(doc->createElement(tagName.get()));

	/* the name attribute is REQUIRED*/
	if(pos->name.empty()){
	  OAPR_THROW(oapr::logic_error,"Required col attribute name is empty");
	}

	attrValue.set(pos->name.c_str());
	colElem->setAttribute(attrName.get(),attrValue.get());

	/* check to see if result type
	 * if so, create and append the value as a text node
	 */
	if(type=="result"){
	  tagValue.set(pos->value.c_str());
	  oa::dom_ptr<DOMText> colValue(doc->createTextNode(tagValue.get()));
	  colElem->appendChild(colValue.release());
	}

	/* add to the table element */
	tableElem->appendChild(colElem.release());

	/* now do the where clause.
	 * note that only the get builds a where clause
	 * so checking for that type
	 */
	if(type=="get"){
	  /* create the where element
	   */
	  tagName.set("where");
	  oa::dom_ptr<DOMElement> whereElem(doc->createElement(tagName.get()));
	  /**\todo there needs to be more error checking here*/
	  /* create the columns and append to where
	   */
	  /**\todo this needs to be moved out into the base class*/
	  Where::clause_type::iterator pos = m_where.m_clause.begin();
	  for(;pos!=m_where.m_clause.end();++pos){

	    /* create the column element*/
	    tagName.set("col");
	    oa::dom_ptr<DOMElement> colElem(doc->createElement(tagName.get()));
	    
	    /* ad the name attribute*/
	    attrName.set("name");
	    attrValue.set(pos->col.c_str());
	    colElem->setAttribute(attrName.get(),attrValue.get());

	    /* add the op attribute*/
	    attrName.set("op");
	    attrValue.set(pos->op.c_str());
	    colElem->setAttribute(attrName.get(),attrValue.get());
	
	    /* if a conjunction exists, add it*/
	    if(!pos->conj.empty()){
	      attrName.set("conj");
	      attrValue.set(pos->conj.c_str());
	      colElem->setAttribute(attrName.get(),attrValue.get());
	    }

	    /* add the value as a text node*/
	    tagValue.set(pos->val.c_str());
	    oa::dom_ptr<DOMText> colValue(doc->createTextNode(tagValue.get()));
	    colElem->appendChild(colValue.release());
	    /* add this to the where elem*/
	    whereElem->appendChild(colElem.release());
	  }
	  tableElem->appendChild(whereElem.release());
	}
      }
    }else if(type=="error"){
      /* this is empty, cause we do error checking after, 
       * just using as a lame filter
       */
    }else{
      /* type is either set or some other non-allowed value, so throw*/
      OAPR_THROW(oapr::logic_error,"Type is other than allowed values, get,result,error");
    }
    if(HasError()){
      /* force type to error*/
      Type("error");
      oa::dom_ptr<DOMNode> errorElem(BuildError());
      tableElem->appendChild(errorElem.release());
    }

    return tableElem.release();
  }
};
