/********************************************************************
 * OpenAether jabDB
 * Transport to allow jabber access to rdbms 
 *
 * Copyright (C) 2002 Justin Kirby
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Justin Kirby <justink@rochester.rr.com>
 *
 *******************************************************************/
/*****--== Change Log --== *****
 * 2002-09-20
 * Changed PcktSelect::Category and PcktSelect::Action to return
 * values based off of jabber::JabPacket::m_hasError value. This
 * is mainly to accomodate bug #59
 */
#ifndef OAJABDB_JABDB_PACKET_SELECT_HPP
#define OAJABDB_JABDB_PACKET_SELECT_HPP

#include "jda/oadbjab_exports.hpp"
#include <stdexcept>
#include "jda/jabdb_packet.hpp"

namespace oajabdb
{
  /**
   * Contains the data from a select packet
   */
  class OADBJAB_API PcktJabDb;
  class OADBJAB_API PcktSelect;
  class PcktSelect : public PcktJabDb
  {
  public:
    struct Column_Value{
      std::string name;
      std::string value;
    };
    typedef std::vector<Column_Value> colval_type;
  public:
    PcktSelect()
      : PcktJabDb("pcktselect"){}
    PcktSelect(const PcktSelect& ps)
      : PcktJabDb(ps),
	m_where(ps.m_where),
	m_colval(ps.m_colval){}
    virtual ~PcktSelect(){}

    virtual const char*const Uri()const{
      return "xmpp:xmlns=http://jabber.org/protocol/jda?select";
    }
    virtual PcktJabDb::Packet_Category Category()const{
      return m_hasError ? PcktJabDb::Category_Error : PcktJabDb::Category_Sql;
    }
    virtual PcktJabDb::Packet_Action Action()const{
      return m_hasError ? PcktJabDb::Action_Error : PcktJabDb::Action_Select;
    }
    
    virtual PcktJabDb* Clone(){
      return new PcktSelect(*this);
    }
    void Cols(const std::string &col, const std::string &val){
		/**\note would do initializer list, except vc6 doesnt like it */
      Column_Value cv;cv.name=col;cv.value=val;
      m_colval.push_back(cv);
    }

    void Clause(const std::string &col,
		const std::string &op,
		const std::string &val,
		const std::string &conj){

      m_where.m_clause.push_back(m_where.MakeQualifier(col,op,val,conj));
    }

    virtual unsigned int Length()const{
      return m_colval.size();
    }
    virtual std::string Cols(int idx)const{
      return m_colval[idx].name;
    }
    virtual std::string Vals(int idx)const{
      return m_colval[idx].value;
    }
    Where Clause()const{
      return m_where;
    }
    std::string xml();
    virtual xercesc::DOMNode*const BuildXml();
  protected:
    virtual xercesc::DOMNode*const BuildTable();
    bool BuildResult();
    bool BuildGet();
    /**
     * Throws a logic exception of level EXC_Error
     */
    bool BuildSet();
  protected:
    Where m_where;
    colval_type m_colval;
  };
};

#endif
