/* -*- Mode: c++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 * OAMessage Event library
 * Contains objects to aid in Jabber client development and extend the
 * basic functionality of jabber.
 *
 * Copyright (C) 2003 Justin Kirby
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Justin Kirby <justink@rochester.rr.com>
 *
 *******************************************************************/
#include "framework/first.h"
#include "openaether.hpp"
#include "msgevent/jabbldmsgevent.hpp"
#include <xercesc/util/XMLString.hpp>
#include <xercesc/dom/DOMNodeList.hpp>
#include "framework/xmlstr.hpp"
#include "msgevent/jabmsgevent.hpp"

XERCES_CPP_NAMESPACE_USE
namespace jabber
{
  bool BuildMsgEvent::CanBuild(DOMNode *const nd)const
  {
    /* The things we need are:
     * nd must be <message>
     * must have x child in jabber:x:event namespace
     * x must have children
     */
    oa::xmlstr<XMLCh,char> msgName("message");
    if(nd->getNodeType()==DOMNode::ELEMENT_NODE &&
       !XMLString::compareString(msgName,nd->getNodeName()) &&
       nd->hasChildNodes()){
      /* we have a message with child nodes
       * now loop trhough the child nodes until we find an x
       */
      oa::xmlstr<XMLCh,char> xmlns("jabber:x:event"),locName("x");
      DOMNodeList *msgList = nd->getChildNodes();
      for(unsigned int i=0; i<msgList->getLength();++i){
	DOMNode *child = msgList->item(i);
	if(!XMLString::compareString(xmlns,child->getNamespaceURI()) &&
	   !XMLString::compareString(locName,child->getLocalName()) &&
	   child->hasChildNodes()){
	  return true;
	}
      }      
    }
    return false;
  }
  JabPacket* BuildMsgEvent::Build(DOMNode *const nd)
  {
    /* if we are asked to build, then we have <x> in jabber:x:event*/
    /*
    std::auto_ptr<JabPacket> rv(0);
    DOMNode*const x=FindX(nd);
    rv.reset(BuildX(x));
    if(!rv.get()){return 0;}
    */
    /* now we do the typical to,from,id,type,et al*/
    /*
    std::string to,from,id,type;
    GetTo(to,nd);
    GetFrom(from,nd);
    GetId(id,nd);
    GetType(type,nd);

    JabJid tojid,fromjid;
    tojid.Jid(to);fromjid.Jid(from);
    rv->To(tojid);
    rv->From(fromjid);
    rv->Id(id);
    rv->Type(type);
    
    return rv.release();
    */
    return 0;
  }
  DOMNode*const BuildMsgEvent::FindX(DOMNode *const nd)
  {
    /* should we do checks?*/
    oa::xmlstr<XMLCh,char> xname("x"),uri("jabber:x:event");
    DOMNodeList *msgList = nd->getChildNodes();
    for(unsigned int i=0;i<msgList->getLength();++i){
      DOMNode*child=msgList->item(i);
      if(!XMLString::compareString(xname,child->getNodeName()) &&
         !XMLString::compareString(uri,child->getNamespaceURI())){
        return child;
      }
    }
    return 0;
  }
  JabX* BuildMsgEvent::BuildX(DOMNode *const xnd)
  {
    DOMNode*nd = FindX(xnd);
    if(!nd || !nd->hasChildNodes()){return 0;}

    oa::xmlstr<XMLCh,char> offline("offline"),
      delivered("delivered"),
      displayed("displayed"),
      composing("composing"),
      id("id");
    std::auto_ptr<JabMsgEvent> rv(new JabMsgEvent);
    /* so we walk along the x children and build accordingly*/
    DOMNodeList *xlist=nd->getChildNodes();
    for(unsigned int i=0;i<xlist->getLength();++i){
      DOMNode *xchild=xlist->item(i);
      if(!XMLString::compareString(xchild->getNodeName(),offline)){
        rv->SetOption(JabMsgEvent::Event_Offline);
      }else if(!XMLString::compareString(xchild->getNodeName(),delivered)){
        rv->SetOption(JabMsgEvent::Event_Delivered);
      }else if(!XMLString::compareString(xchild->getNodeName(),displayed)){
        rv->SetOption(JabMsgEvent::Event_Displayed);
      }else if(!XMLString::compareString(xchild->getNodeName(),composing)){
        rv->SetOption(JabMsgEvent::Event_Composing);
      }else if(!XMLString::compareString(xchild->getNodeName(),id)){
        DOMNode *txt = xchild->getFirstChild();
        oa::xmlstr<char,XMLCh> idValue(txt->getNodeValue());
        rv->MsgId(idValue.get());
      }
    }
    
    return rv.release();
  }
};
