/* -*- Mode: c++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 * OAMessage Event library
 * Contains objects to aid in Jabber client development and extend the
 * basic functionality of jabber.
 *
 * Copyright (C) 2003 Justin Kirby
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Justin Kirby <justink@rochester.rr.com>
 *
 *******************************************************************/
#include "framework/first.h"
#include "openaether.hpp"
#include "msgevent/jabmsgevent.hpp"
#include "framework/xmlutil.hpp"
#include "framework/xmlstr.hpp"
#include "framework/dom_ptr.hpp"

XERCES_CPP_NAMESPACE_USE
namespace jabber
{
  JabX::clone_type JabMsgEvent::Clone()
  {
    JabX::clone_type rv(new JabMsgEvent(*this));
    return rv;
  }

  std::string JabMsgEvent::xml()
  {
    std::string ret;
    oa::dom_ptr<DOMNode> built(BuildXml());
    if(built.get()){
      oa::XMLUtils().Print(ret,built.get());
    }
    return ret;
  }
  DOMNode*const JabMsgEvent::BuildXml()
  {
    return BuildX();
    /*
    DOMNode *rv=0,*xNode=0;

    xNode = BuildX();// we *always* build this nomatter what

    if(!Attached()){
      rv=BuildHeader();
      rv->appendChild(xNode);
    }else{
      rv=xNode;
    }


    return rv;
    */
  }
  /*
  DOMNode*const JabMsgEvent::BuildHeader()
  {

    oa::xmlstr<XMLCh,char> elemName("message"),
      attrName,attrValue;
    */
    /* we build the message header */
    /*
    DOMDocument *doc=oa::XMLUtils().Document(DocUri(),"oa");
    oa::dom_ptr<DOMElement> msgElem(doc->createElement(elemName));
    */

    /* do from if there*/
    /*
    if(!From().IsNull()){
      attrName.set("from");
      attrValue.set(From().Jid().c_str());
      msgElem->setAttribute(attrName,attrValue);      
    }

    if(!To().IsNull()){
      attrName.set("to");
      attrValue.set(To().Jid().c_str());
      msgElem->setAttribute(attrName,attrValue);
    }

    if(!Id().empty()){
      attrName.set("id");
      attrValue.set(Id().c_str());
      msgElem->setAttribute(attrName,attrValue);
    }

    if(Type().empty()){
      Type("normal");
    }

    attrName.set("type");
    attrValue.set(Type().c_str());
    msgElem->setAttribute(attrName,attrValue);
    */
    /* we don't deal with errors... is this a problem here?*/
    /*
    return msgElem.release();
  }
    */
  DOMNode*const JabMsgEvent::BuildX()
  {
    oa::xmlstr<XMLCh,char> xName("x"),
      xmlnsName("xmlns"),
      xmlnsValue(Xmlns().c_str()),
      eventName;

    DOMDocument *doc = oa::XMLUtils().Document(DocUri(),"oa");
    oa::dom_ptr<DOMElement> xElem(doc->createElementNS(xmlnsValue,xName));
    //forcing attribute so it prints
    xElem->setAttribute(xmlnsName,xmlnsValue);

    if(m_events & Event_Offline){
      eventName.set("offline");
      oa::dom_ptr<DOMElement> offelem(doc->createElement(eventName));
      xElem->appendChild(offelem.release());
    }
    if(m_events & Event_Delivered){
      eventName.set("delivered");
      oa::dom_ptr<DOMElement> delelem(doc->createElement(eventName));
      xElem->appendChild(delelem.release());
    }
    if(m_events & Event_Displayed){
      eventName.set("displayed");
      oa::dom_ptr<DOMElement> dispelem(doc->createElement(eventName));
      xElem->appendChild(dispelem.release());
    }
    if(m_events & Event_Composing){
      eventName.set("composing");
      oa::dom_ptr<DOMElement> compelem(doc->createElement(eventName));
      xElem->appendChild(compelem.release());
    }

    if(!MsgId().empty()){
      oa::xmlstr<XMLCh,char> idName("id"),
        idValue(MsgId().c_str());
      oa::dom_ptr<DOMElement> idElem(doc->createElement(idName));
      oa::dom_ptr<DOMText> idText(doc->createTextNode(idValue));
      idElem->appendChild(idText.release());
      xElem->appendChild(idElem.release());
    }
    
    return xElem.release();
  }
};
