/* -*- Mode: c++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 * OAMessage Event library
 * Contains objects to aid in Jabber client development and extend the
 * basic functionality of jabber.
 *
 * Copyright (C) 2003 Justin Kirby
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Justin Kirby <justink@rochester.rr.com>
 *
 *******************************************************************/
#ifndef OAJBBER_OAMSGEVENT_JABMSGEVENT_HPP
#define OAJBBER_OAMSGEVENT_JABMSGEVENT_HPP
#include "xstrm/ijabx.h"
#include "msgevent/oamsgevent_exports.hpp"
namespace jabber
{
  class OAMSGEVENT_API JabMsgEvent;
  class JabMsgEvent : public JabX
  {
  public:
    enum Event_Types{Event_Offline=1,
                     Event_Delivered=2,
                     Event_Displayed=4,
                     Event_Composing=8,
                     Event_All=15};
  public:
    JabMsgEvent()
      : JabX(),
        m_events(0){}
    JabMsgEvent(const JabMsgEvent &jme)
      : JabX(jme),
        m_events(jme.m_events),
        m_msgid(jme.m_msgid){}
    virtual ~JabMsgEvent(){}
    virtual void SetOption(JabMsgEvent::Event_Types opts){
      m_events |= opts;
    }
    virtual bool HasOption(JabMsgEvent::Event_Types opt){
      return (m_events & opt);
    }
    virtual void MsgId(const std::string &id){m_msgid=id;}
    virtual std::string MsgId()const{return m_msgid;}

    /* things we have to impl for packet */
    virtual const char*const Uri()const{
      return "xmpp:xmlns=jabber:x:event";
    }
    virtual std::string Xmlns()const{
      return "jabber:x:event";
    }
    virtual void Xmlns(const std::string& x){}
    virtual std::string xml();
    virtual xercesc::DOMNode*const BuildXml();

    /* things we have to impl for jabx*/
    virtual JabX::clone_type Clone();
  protected:
    /*
    virtual xercesc::DOMNode*const BuildHeader();
    */
    virtual xercesc::DOMNode*const BuildX();
  private:
    char m_events;
    std::string m_msgid;
  };
};

#endif
