/* -*- Mode: c++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 * OAjabber pubsub library
 * Contains objects to aid in Jabber client development and extend the
 * basic functionality of jabber.
 *
 * Copyright (C) 2001 Justin Kirby
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Justin Kirby <justink@rochester.rr.com>
 *
 *******************************************************************/
#include "framework/first.h"
#include "openaether.hpp"
#include "pubsub/jabpsaffiliation.hpp"
#include "framework/xmlutil.hpp"
#include "framework/xmlstr.hpp"
#include "framework/dom_ptr.hpp"

XERCES_CPP_NAMESPACE_USE
namespace jabber
{
  DOMNode* const JabPSAffiliation::BuildXml()
  {
    DOMDocument* doc = oa::XMLUtils().Document(DocUri(),"oa");

    oa::xmlstr<XMLCh,char> affName("affiliations");

    oa::dom_ptr<DOMNode> ps(BuildPubSub());
    oa::dom_ptr<DOMNode> iq(BuildIq());
    oa::dom_ptr<DOMElement> aff(doc->createElement(affName));
    
    entity_cont::iterator pos=m_entities.begin();
    for(;pos!=m_entities.end();++pos){
      DOMNode* ent = BuildEnt(doc,*pos);
      if(ent != 0){
        aff->appendChild(ent);
      }
    }

    ps->appendChild(aff.release());
    iq->appendChild(ps.release());

    return iq.release();
  }
  DOMNode*const JabPSAffiliation::BuildEnt(DOMDocument*const doc, 
                                           const PSEntity& pe)
  {

    oa::xmlstr<XMLCh,char> xmlns(Xmlns().c_str()),
      entName("entity"),
      nodeName("node"),
      nodeVal(pe.NodeName().c_str()),
      jidName("jid"),
      jidVal(pe.Jid().Jid().c_str()),
      affName("affiliation"),
      affVal(pe.Affiliation().toString().c_str()),
      subName("subscription"),
      subVal(pe.Subscription().toString().c_str());

    oa::dom_ptr<DOMElement> ent(doc->createElementNS(xmlns,entName));
    ent->setAttribute(nodeName,nodeVal);
    ent->setAttribute(jidName,jidVal);
    ent->setAttribute(affName,affVal);
    ent->setAttribute(subName,subVal);
    return ent.release();
  }
};
