/* -*- Mode: c++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 * OAjabber pubsub library
 * Contains objects to aid in Jabber client development and extend the
 * basic functionality of jabber.
 *
 * Copyright (C) 2001 Justin Kirby
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Justin Kirby <justink@rochester.rr.com>
 *
 *******************************************************************/

#include "pubsub/jabpsdel.hpp"
#include "framework/xmlstr.hpp"
#include "framework/dom_ptr.hpp"
#include "framework/xmlutil.hpp"

namespace jabber
{
  xercesc::DOMNode*const JabPSDel::BuildXml()
  {
    if(m_itemId.empty()){
      /**@todo throw*/
    }
    xercesc::DOMDocument *doc = oa::XMLUtils().Document(DocUri(),"oa");

    oa::xmlstr<XMLCh,char> xmlns(Xmlns().c_str()),
      retName("retract"),
      nodeName("node"),
      nodeVal(NodeName().c_str()),
      itemName("item"),
      idName("id"),
      idVal(m_itemId.c_str());

    oa::dom_ptr<xercesc::DOMElement> ret(doc->createElementNS(xmlns,retName));
    ret->setAttribute(nodeName,nodeVal);

    oa::dom_ptr<xercesc::DOMElement> item(doc->createElementNS(xmlns,itemName));
    item->setAttribute(idName,idVal);
    
    ret->appendChild(item.release());

    oa::dom_ptr<xercesc::DOMNode> iq(BuildIq());
    oa::dom_ptr<xercesc::DOMNode> ps(BuildPubSub());

    ps->appendChild(ret.release());
    iq->appendChild(ps.release());

    return iq.release();		    
  }
};
