/* -*- Mode: c++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 * OAjabber pubsub library
 * Contains objects to aid in Jabber client development and extend the
 * basic functionality of jabber.
 *
 * Copyright (C) 2001 Justin Kirby
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Justin Kirby <justink@rochester.rr.com>
 *
 *******************************************************************/
#include "framework/first.h"
#include "openaether.hpp"
#include "pubsub/jabpsentities.hpp"
#include "framework/xmlutil.hpp"
#include "framework/xmlstr.hpp"
#include "framework/dom_ptr.hpp"
#include "oapr/oapr_exception.hpp"
#include "oapr/oapr_exc_utils.hpp"

XERCES_CPP_NAMESPACE_USE
namespace jabber
{
  DOMNode*const JabPSEntities::BuildXml()
  {
    if(NodeName().empty()){
      OAPR_THROW(oapr::logic_error,
			  "sending an entity stanza without node name specified");
    }
    /* note: we can only have entities if its NOT get */
    if(Type()=="get" && !m_entities.empty()){
      OAPR_THROW(oapr::logic_error,"sending a type get with entities");
    }
    /* note: we must have entities for a set or result */
    if((Type() == "set" || Type()=="result") && m_entities.empty()){
      OAPR_THROW(oapr::logic_error,
			  "sending set or result with no entities");
    }
    oa::xmlstr<XMLCh,char> entities("entities"),
      xmlns(Xmlns().c_str()),
      nodeName("node"),
      nodeValue(NodeName().c_str());
      
    DOMDocument *doc = oa::XMLUtils().Document(DocUri(), "oa");

    oa::dom_ptr<DOMElement> entitiesNode(doc->createElementNS(xmlns,entities));
    entitiesNode->setAttribute(nodeName,nodeValue);
    
    /* if we have entities then add them*/
    oa::dom_ptr<DOMNode> entityNode;
    entity_cont::iterator pos=m_entities.begin();
    for(;pos != m_entities.end();++pos){
      entityNode.reset(EntityToXml(*pos));
      if(entityNode.get()){
        entitiesNode->appendChild(entityNode.release());
      }
    }
    
    oa::dom_ptr<DOMNode> psNode(BuildPubSub());
    oa::dom_ptr<DOMNode> psIq(BuildIq());
    psNode->appendChild(entitiesNode.release());
    psIq->appendChild(psNode.release());
    return psIq.release();
  }
  DOMNode*const JabPSEntities::EntityToXml(const PSEntity& pe)
  {
    /* we need all attributes to have a value*/
    if(pe.Jid().IsNull()){return 0;}

    DOMDocument *doc = oa::XMLUtils().Document(DocUri(),"oa");
    
    oa::xmlstr<XMLCh,char> entityName("entity"),
      xmlns(Xmlns().c_str()),
      jidName("jid"),
      affName("affiliation"),
      subName("subscription"),
      jidValue(pe.Jid().Jid().c_str()),
      affValue(pe.Affiliation().toString().c_str()),
      subValue(pe.Subscription().toString().c_str());

    oa::dom_ptr<DOMElement> entity(doc->createElementNS(xmlns,entityName));
    entity->setAttribute(jidName,jidValue);
    entity->setAttribute(affName,affValue);
    entity->setAttribute(subName,subValue);
    return entity.release();
  }
};
