/* -*- Mode: c++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 * OAjabber pubsub library
 * Contains objects to aid in Jabber client development and extend the
 * basic functionality of jabber.
 *
 * Copyright (C) 2001 Justin Kirby
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Justin Kirby <justink@rochester.rr.com>
 *
 *******************************************************************/
#ifndef OAJABBER_PUBSUB_JABPSENTITIES_HPP
#define OAJABBER_PUBSUB_JABPSENTITIES_HPP

#include "pubsub/jabpubsub.hpp"
#include "pubsub/psentity.hpp"
#include <vector>

namespace jabber
{
  /**
   * JabPSEntities represents the entity query
   * It provides ability to edit entity affiliations
   * @ingroup gJabPackets
   */
  class JabPSEntities : public JabPubsub
  {
  public:
    typedef std::vector<PSEntity> entity_cont;
  public:
    JabPSEntities()
      : JabPubsub("jabber::JabPSEntities"){}
    JabPSEntities(const JabPSEntities& jpe)
      : JabPubsub(jpe),
        m_entities(jpe.m_entities){}
    ~JabPSEntities(){}

    const char* const Uri()const{
      return "http://jabber.org/protocol/pubsub?entities";
    }

    virtual entity_cont Entities()const{return m_entities;}
    virtual void Entity(const PSEntity& pe){m_entities.push_back(pe);}

    virtual xercesc::DOMNode*const BuildXml();
  private:
    xercesc::DOMNode*const EntityToXml(const PSEntity& pe);
  private:
    entity_cont m_entities;    
  };
};
#endif
