/* -*- Mode: c++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 * OAjabber pubsub library
 * Contains objects to aid in Jabber client development and extend the
 * basic functionality of jabber.
 *
 * Copyright (C) 2001 Justin Kirby
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Justin Kirby <justink@rochester.rr.com>
 *
 *******************************************************************/

#include "pubsub/jabpsitems.hpp"
#include "framework/xmlstr.hpp"
#include "framework/dom_ptr.hpp"
#include "framework/xmlutil.hpp"
#include "boost/lexical_cast.hpp"

XERCES_CPP_NAMESPACE_USE
namespace jabber
{
  void JabPSItems::AddItem(const PSItem& pi)
  {
    m_items.push_back(pi);
  }
  JabPSItems::item_cont::const_iterator JabPSItems::BeginItem()const
  {
    return m_items.begin();
  }
  JabPSItems::item_cont::const_iterator JabPSItems::EndItem()const
  {
    return m_items.end();
  }
  void JabPSItems::EraseItem(const std::string& id)
  {
    /* note that since this is a vector, only choice is to loop*/
    item_cont::iterator pos=m_items.begin();
    for(;pos!=m_items.end();++pos){
      if(pos->Id() == id){
	m_items.erase(pos);
	break;
      }
    }
    
  }
  xercesc::DOMNode*const JabPSItems::BuildXml()
  {
    xercesc::DOMDocument *doc = oa::XMLUtils().Document(DocUri(),"oa");

    oa::xmlstr<XMLCh,char> xmlns(Xmlns().c_str()),
      itemsName("items"),
      maxName("max_items"),
      maxVal,
      nodeName("node"),
      nodeVal(NodeName().c_str());

    oa::dom_ptr<DOMElement> items(doc->createElementNS(xmlns,itemsName));
    items->setAttribute(nodeName,nodeVal);

    if(m_max > 0){
      try{
	std::string mx = boost::lexical_cast<std::string>(m_max);
	maxVal.set(mx.c_str());

	items->setAttribute(maxName,maxVal);
      }catch(...){
	/**@todo better catch processing*/
      }
    }

    item_cont::iterator pos = m_items.begin();
    for(;pos != m_items.begin(); ++pos){
      DOMNode*const tmpItem = BuildItem(doc,*pos);
      if(tmpItem != 0){
	items->appendChild(tmpItem);
      }
    }

    oa::dom_ptr<DOMNode> ps(BuildPubSub());
    oa::dom_ptr<DOMNode> iq(BuildIq());

    ps->appendChild(items.release());
    iq->appendChild(ps.release());

    return iq.release();
  }
  xercesc::DOMNode*const JabPSItems::BuildItem(xercesc::DOMDocument*const doc,
					       const PSItem& pi)
  {
    oa::xmlstr<XMLCh,char> xmlns(Xmlns().c_str()),
      itemName("item"), 
      idName("id"),
      idVal(pi.Id().c_str());

    oa::dom_ptr<DOMElement> item(doc->createElementNS(xmlns,itemName));
    item->setAttribute(idName,idVal);

    DOMNode*const payload = pi.PayLoad();
    /* we should only be including the payload if this is a result*/
    if(Type() == "result" && payload != 0){
      item->appendChild(payload);
    }
    return item.release();
  }
};
