/* -*- Mode: c++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 * OAjabber pubsub library
 * Contains objects to aid in Jabber client development and extend the
 * basic functionality of jabber.
 *
 * Copyright (C) 2001 Justin Kirby
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Justin Kirby <justink@rochester.rr.com>
 *
 *******************************************************************/
#ifndef OAJABBER_PUBSUB_JABPSITEMS_HPP
#define OAJABBER_PUBSUB_JABPSITEMS_HPP

#include <vector>
#include "pubsub/jabpubsub.hpp"
#include "pubsub/psitem.hpp"

namespace jabber
{
  class JabPSItems : public JabPubsub
  {
  public:
    typedef std::vector<PSItem> item_cont;
  public:
    JabPSItems()
      : JabPubsub("jabber::JabPSItems")
	,m_max(0){}
    JabPSItems(const JabPSItems& jpi)
      : JabPubsub(jpi)
	,m_items(jpi.m_items)
	,m_max(jpi.m_max){}
    ~JabPSItems(){}

    void Max(unsigned int mx){
      m_max=mx;
    }
    unsigned int Max()const{
      return m_max;
    }
    void AddItem(const PSItem& pi);
    item_cont::const_iterator BeginItem()const;
    item_cont::const_iterator EndItem()const;
    void EraseItem(const std::string& id);

    virtual xercesc::DOMNode*const BuildXml();
  private:
    xercesc::DOMNode*const BuildItem(xercesc::DOMDocument*const doc,
				     const PSItem& pi);
  private:
    item_cont m_items;
    unsigned int m_max;
  };
};
#endif
