/* -*- Mode: c++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 * OAjabber pubsub library
 * Contains objects to aid in Jabber client development and extend the
 * basic functionality of jabber.
 *
 * Copyright (C) 2001 Justin Kirby
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Justin Kirby <justink@rochester.rr.com>
 *
 *******************************************************************/
#include "framework/first.h"
#include "openaether.hpp"
#include "pubsub/jabpssub.hpp"
#include "framework/xmlutil.hpp"
#include "framework/xmlstr.hpp"
#include "framework/dom_ptr.hpp"
#include "oapr/oapr_exception.hpp"
#include "oapr/oapr_exc_utils.hpp"

XERCES_CPP_NAMESPACE_USE
namespace jabber
{
  void JabPSSub::Subscribe(const JabJid &jj)
  {
    m_subType=ST_Sub;
    m_jid=jj;
  }
  void JabPSSub::Unsubscribe(const JabJid &jj)
  {
    m_subType=ST_Unsub;
    m_jid=jj;
  }
  void JabPSSub::Entity(const PSEntity& pse)
  {
    m_subType=ST_Entity;
    m_entity=pse;
  }
  PSEntity JabPSSub::Entity()const{
    return m_entity;
  }

  DOMNode*const JabPSSub::BuildXml()
  {
    if(m_subType==ST_Null){
      OAPR_THROW(oapr::logic_error, "Can not send in null state");
    }

    DOMDocument *doc = oa::XMLUtils().Document(DocUri(),"oa");

    oa::dom_ptr<DOMNode> tmp;
    if(m_subType == ST_Sub){
      tmp = BuildSub(doc);
    }else if(m_subType == ST_Entity){
      tmp = BuildEntity(doc);
    }else if(m_subType == ST_Unsub){
      tmp = BuildUnsub(doc);
    }else{
      /**@todo throw invalid type*/
    }

    oa::dom_ptr<DOMNode> iq(BuildIq());
    oa::dom_ptr<DOMNode> ps(BuildPubSub());
    ps->appendChild(tmp.release());
    iq->appendChild(ps.release());

    return iq.release();
  }
  xercesc::DOMNode*const JabPSSub::BuildSub(DOMDocument*const doc)
  {
    oa::xmlstr<XMLCh,char> xmlns(Xmlns().c_str()),
      subName("subscribe"),
      nodeName("node"),
      nodeVal(NodeName().c_str()),
      jidName("jid"),
      jidVal(m_jid.Jid().c_str());

    oa::dom_ptr<DOMElement> sub(doc->createElementNS(xmlns,subName));
    sub->setAttribute(nodeName,nodeVal);
    sub->setAttribute(jidName,jidVal);
    return sub.release();
    
  }
  xercesc::DOMNode*const JabPSSub::BuildUnsub(DOMDocument*const doc)
  {
    oa::xmlstr<XMLCh,char> xmlns(Xmlns().c_str()),
       usubName("unsubscribe"),
       nodeName("node"),
       nodeVal(NodeName().c_str()),
       jidName("jid"),
       jidVal(m_jid.Jid().c_str());

    oa::dom_ptr<DOMElement> usub(doc->createElementNS(xmlns,usubName));
    usub->setAttribute(nodeName,nodeVal);
    usub->setAttribute(jidName,jidVal);
    return usub.release();
  }
  xercesc::DOMNode*const JabPSSub::BuildEntity(DOMDocument*const doc)
  {
    oa::xmlstr<XMLCh,char> xmlns(Xmlns().c_str()),
      entName("entity"),
      nodeName("node"),
      nodeVal(m_entity.NodeName().c_str()),
      jidName("jid"),
      jidVal(m_entity.Jid().Jid().c_str()),
      affName("affiliation"),
      affVal(m_entity.Affiliation().toString().c_str()),
      subName("subscription"),
      subVal(m_entity.Subscription().toString().c_str());

    oa::dom_ptr<DOMElement> ent(doc->createElementNS(xmlns,entName));

    ent->setAttribute(nodeName,nodeVal);
    ent->setAttribute(jidName,jidVal);
    ent->setAttribute(affName,affVal);
    ent->setAttribute(subName,subVal);

    return ent.release();      
  }
};
