/* -*- Mode: c++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 * OAjabber pubsub library
 * Contains objects to aid in Jabber client development and extend the
 * basic functionality of jabber.
 *
 * Copyright (C) 2001 Justin Kirby
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Justin Kirby <justink@rochester.rr.com>
 *
 *******************************************************************/
#ifndef OAJABBER_PUBSUB_JABPSSUB_HPP
#define OAJABBER_PUBSUB_JABPSSUB_HPP
#include <xercesc/dom/DOMDocument.hpp>
#include "pubsub/jabpubsub.hpp"
#include "pubsub/psentity.hpp"

namespace jabber
{
  /**
   * JabPSSub manages subscriptions to a specific node
   * @ingroup gJabPacket
   */

  class JabPSSub : public JabPubsub
  {
  public:
    JabPSSub()
      : JabPubsub("jabber::JabPSSub"),
        m_subType(JabPSSub::ST_Null){}
    JabPSSub(const JabPSSub& jps)
      : JabPubsub(jps),
        m_entity(jps.m_entity),
        m_jid(jps.m_jid),
        m_subType(jps.m_subType){}
    ~JabPSSub(){}

    const char* const Uri()const{
      return "http://jabber.org/protocol/pubsub?sub";
    }

    virtual void Subscribe(const JabJid &jj);

    virtual void Unsubscribe(const JabJid &jj);

    virtual void Entity(const PSEntity& pse);
    virtual PSEntity Entity()const;


    virtual xercesc::DOMNode*const BuildXml();

  private:
    xercesc::DOMNode*const BuildSub(xercesc::DOMDocument*const doc);
    xercesc::DOMNode*const BuildUnsub(xercesc::DOMDocument*const doc);
    xercesc::DOMNode*const BuildEntity(xercesc::DOMDocument*const doc);
    
  private:
    enum Sub_Type{ST_Null=0,ST_Sub,ST_Unsub,ST_Entity};
    PSEntity m_entity;
    JabJid m_jid;
    Sub_Type m_subType;
  };
};
#endif
