/* -*- Mode: c++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 * OAjabber pubsub library
 * Contains objects to aid in Jabber client development and extend the
 * basic functionality of jabber.
 *
 * Copyright (C) 2001 Justin Kirby
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Justin Kirby <justink@rochester.rr.com>
 *
 *******************************************************************/
#ifndef OAJABBER_PUBSUB_JABPUBSUB_HPP
#define OAJABBER_PUBSUB_JABPUBSUB_HPP

#include "jabber/ijabiq.h"

namespace jabber
{
  /**
   * Base for the pubsub class hierarchy
   * Has the nodename attribute that all others need
   * also has the pubsub node creation for xml as a
   * convenience
   * @ingroup gJabPacket
   */
  class JabPubsub : public JabIq
  {
  public:
    JabPubsub(const std::string &clsid)
      : JabIq(clsid)
    {}
    JabPubsub(const JabPubsub& jps)
      : JabIq(jps),
        m_nodeName(jps.m_nodeName){}
    virtual ~JabPubsub(){}

    virtual void Xmlns(const std::string &x){}
    virtual std::string Xmlns()const {
      return "http://jabber.org/protocol/pubsub";
    }
    /**
     * Set the name of the node
     * @param nn nodename 
     */
    virtual void NodeName(const std::string &nn){
      m_nodeName=nn;
    }
    /**
     * Get the name of the node
     * @returns returns the name of the node
     */
    virtual std::string NodeName()const{return m_nodeName;}

  protected:
    /**
     * Builds the pubsub wrapper element for the iq stanza
     * all pubsub stanzas need this.
     * @return Pointer to the pubsub element 
     */
    xercesc::DOMNode*const BuildPubSub();
  private:
    std::string m_nodeName;/**< name of the node*/
  };
};

#endif
