/* -*- Mode: c++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 * OAjabber pubsub library
 * Contains objects to aid in Jabber client development and extend the
 * basic functionality of jabber.
 *
 * Copyright (C) 2001 Justin Kirby
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Justin Kirby <justink@rochester.rr.com>
 *
 *******************************************************************/
#ifndef OAJABBER_PUBSUB_PSAFFILIATION_HPP
#define OAJABBER_PUBSUB_PSAFFILIATION_HPP
#include <string>
namespace jabber
{


  /**
   * Encapsulates the affiliation type. Essentially
   * creating an enum with a conversion to string.
   *
   * \addtogroup EnumTypes
   */
  class PSAffiliation
  {
  public:

    /**
     * enum for affiliation types. 
     * see jep0060 for more specific information.
     */
    enum Affiliation_Type{AT_None=0,/**< no affiliation */
                          AT_Outcast,/**< is set to outcast */
                          AT_Publisher,/**< is a publisher */
                          AT_Owner};/**< is the owner */
  public:
    PSAffiliation(Affiliation_Type at)
      : m_type(at){}
    PSAffiliation(const PSAffiliation& psa)
      : m_type(psa.m_type){}
    ~PSAffiliation(){}
    /**
     * converts the enum to the equivalent string value.
     * the string values are determined by the attribute
     * values in jep0060 for affiliations.
     * @return A string form of the enum, defaults to none
     */
    std::string toString()const;

    /**
     * The obligatory comparison operator
     */
    bool operator==(const PSAffiliation& psa){
      if(this == &psa){return true;}
      if(m_type==psa.m_type){return true;}
      return false;
    }
  private:
    Affiliation_Type m_type;
  };
};
#endif
