/* -*- Mode: c++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 * OAjabber pubsub library
 * Contains objects to aid in Jabber client development and extend the
 * basic functionality of jabber.
 *
 * Copyright (C) 2001 Justin Kirby
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Justin Kirby <justink@rochester.rr.com>
 *
 *******************************************************************/
#ifndef OAJABBER_PUBSUB_PSSUBSCRIPTION_HPP
#define OAJABBER_PUBSUB_PSSUBSCRIPTION_HPP
#include <string>

namespace jabber
{
  /**
   * Encapsulates the subscription type. Essentially
   * creating an enum that can be converted to a string
   *
   * @addtogroup EnumTypes
   */
  class PSSubscription
  {
  public:
    /**
     * Types for subscription to a node.
     * see jep0060 for exact definitions
     */
    enum Subscription_Type{ST_None,/**< no subscription */
                           ST_Pending, /**< pending sub */
                           ST_Unconfigured, /**< unconfigured subscription */
                           ST_Subscribed};/**< subscribed*/
  public:
    PSSubscription(Subscription_Type st)
      : m_sub(st){}
    PSSubscription(const PSSubscription& pss)
      : m_sub(pss.m_sub){}
    ~PSSubscription(){}

    std::string toString()const;
  private:
    Subscription_Type m_sub;
  };
};
#endif
