/********************************************************************
 * OAsac
 * Is a simple component which provides generic acl functionality to
 * the jabber newtork
 *
 * Copyright (C) 2002 Justin Kirby
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Justin Kirby <justink@rochester.rr.com>
 *
 *******************************************************************/
#include "framework/first.h"
#include "openaether.hpp"
#include <memory>
#include "sac/jabbldsac.hpp"
#include "sac/jabsac.hpp"
#include "framework/xmlstr.hpp"
#include <xercesc/dom/DOMElement.hpp>
#define URI_SAC "http://jabber.org/protocol/sac"

XERCES_CPP_NAMESPACE_USE
namespace jabber
{
  bool BuildSac::CanBuild(DOMNode*const nd)const
  {
    if(!nd){return false;}//<-- paranoia protection

    /* the determining factor is:
     * - element name is acl
     * - namespace is http://jabber.org/protocol/sac
     */

    oa::xmlstr<XMLCh,char> aclName("acl"),
      aclUri(URI_SAC);
    DOMNode*aclNode = nd->getFirstChild();
    if(aclNode &&
       aclNode->getNodeType()==DOMNode::ELEMENT_NODE &&
       !XMLString::compareString(aclNode->getNodeName(),aclName.get()) &&
       !XMLString::compareString(aclNode->getNamespaceURI(),aclUri.get())){
      return true;
    }
    return false;
  }
  JabPacket* BuildSac::Build(DOMNode*const nd)
  {
    std::auto_ptr<JabSac> rv(new JabSac);
    std::string to,from,id,type,
      actor,oper,target,respName;
    JabError jaberror;

    GetTo(to,nd);
    GetFrom(from,nd);
    GetId(id,nd);
    GetType(type,nd);

    DOMNode*aclNode = nd->getFirstChild();
    if(!aclNode){
      return 0;/**\todo throw*/
    }

    /* be paranoid and verify that the element name is acl*/
    oa::xmlstr<XMLCh,char> aclName("acl");
    if(aclNode->getNodeType()==DOMNode::ELEMENT_NODE &&
       !XMLString::compareString(aclNode->getNodeName(),aclName.get())){

      DOMElement *aclElem = (DOMElement*)aclNode;
      oa::xmlstr<XMLCh,char> actorName("actor"),
	operName("oper"),
	targetName("target");

      /* get the actor attribute */
      if(aclElem->hasAttribute(actorName.get())){
	oa::xmlstr<char,XMLCh> actorValue;
	actorValue.set(aclElem->getAttribute(actorName.get()));
	actor=actorValue.get();
      }
      /* get the oper attribute*/
      if(aclElem->hasAttribute(operName.get())){
	oa::xmlstr<char,XMLCh> operValue;
	operValue.set(aclElem->getAttribute(operName.get()));
	oper=operValue.get();
      }
      /* get teh target attribute */
      if(aclElem->hasAttribute(targetName.get())){
	oa::xmlstr<char,XMLCh> targetValue;
	targetValue.set(aclElem->getAttribute(targetName.get()));
	target=targetValue.get();
      }


      if(GetError(jaberror,nd)){
	rv->Error(jaberror);
      }
      /* if we have child get it*/
      if(aclElem->hasChildNodes()){
	DOMNode* childNode = aclElem->getFirstChild();
	if(childNode){
	  oa::xmlstr<char,XMLCh> allowed(childNode->getNodeName());
	  respName = allowed.get();
	}
      }

      JabJid fromJid,toJid;
      fromJid.Jid(from);
      toJid.Jid(to);

      rv->To(toJid);
      rv->From(fromJid);
      rv->Id(id);
      rv->Type(type);

      rv->Actor(actor);
      rv->Oper(oper);
      rv->Target(target);

      if(respName == "allowed"){
	rv->Allowed(true);
      }else{
	rv->Allowed(false);
      }
      return rv.release();
    }
    return 0;
  }
};
