/********************************************************************
 * OAsac
 * Is a simple component which provides generic acl functionality to
 * the jabber newtork
 *
 * Copyright (C) 2002 Justin Kirby
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Justin Kirby <justink@rochester.rr.com>
 *
 *******************************************************************/
#include "framework/first.h"
#include "openaether.hpp"
#include "sac/jabsac.hpp"
#include "framework/xmlutil.hpp"
#include "framework/xmlstr.hpp"
#include "framework/xerces_xmlns.hpp"

XERCES_CPP_NAMESPACE_USE
namespace jabber
{
  DOMNode*const JabSac::BuildXml()
  {
    DOMDocument*doc = oa::XMLUtils().Document(DocUri(),"oa");
    oa::dom_ptr<DOMElement> aclElem;
    {
      oa::xmlstr<XMLCh,char> aclName("acl"),
	aclUri(Xmlns().c_str()),
	xmlnsName("xmlns");

      aclElem.reset(doc->createElementNS(aclUri.get(),aclName.get()));

      /* add the attr so it prints*/
      oa::XercesXmlns()(aclElem.get(),aclUri);
    }
    /* now we add all the attributes*/
    {
      oa::xmlstr<XMLCh,char> actorName("actor"),
	operName("oper"),
	targetName("target"),
	actorValue(m_actor.c_str()),
	operValue(m_oper.c_str()),
	targetValue(m_target.c_str());

      aclElem->setAttribute(actorName.get(),actorValue.get());
      aclElem->setAttribute(operName.get(),operValue.get());
      aclElem->setAttribute(targetName.get(),targetValue.get());
    }

    /* if we have an error, process it*/
    if(HasError()){
      Type("error");//making sure we are of the type
      DOMNode* errorNode = BuildError();
      aclElem->appendChild(errorNode);
    }else if(Type()=="result"){
      oa::xmlstr<XMLCh,char> allowedName;
      if(m_allowed){
	allowedName.set("allowed");
      }else{
	allowedName.set("denied");
      }
      DOMElement* allowedElem = doc->createElement(allowedName.get());
      aclElem->appendChild(allowedElem);
    }

    oa::dom_ptr<DOMNode> iqNode(BuildIq());

    iqNode->appendChild(aclElem.release());
    /*
    m_node.reset(iqNode.release());
    return m_node.get();
    */
    return iqNode.release();
  }
};
