/********************************************************************
 * OAsac
 * Is a simple component which provides generic acl functionality to
 * the jabber newtork
 *
 * Copyright (C) 2002 Justin Kirby
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Justin Kirby <justink@rochester.rr.com>
 *
 *******************************************************************/
#include "framework/first.h"
#include "openaether.hpp"
#include <algorithm>
#include "sac/jabsacquery.hpp"
#include "framework/xmlutil.hpp"
#include "framework/xmlstr.hpp"
#include "framework/xerces_xmlns.hpp"

XERCES_CPP_NAMESPACE_USE
namespace jabber
{

  void JabSacQuery::Remove(const std::string &uri)
  {
    cont_type::iterator pos=std::remove(m_opers.begin(),
					m_opers.end(),
					uri);
    m_opers.erase(pos,m_opers.end());
  }
  JabSacQuery::rv_type JabSacQuery::Get()const
  {
    rv_type rv;
    std::copy(m_opers.begin(),m_opers.end(),std::back_inserter(rv));
    return rv;
  }

  DOMNode*const JabSacQuery::BuildXml()
  {
    DOMDocument*doc = oa::XMLUtils().Document(DocUri(),"oa");
    DOMElement* queryElem=0;
    {
      oa::xmlstr<XMLCh,char> queryName("query"),
	queryUri(Xmlns().c_str()),
	xmlnsName("xmlns");

      queryElem = doc->createElementNS(queryUri.get(),queryName.get());
      oa::XercesXmlns()(queryElem,queryUri);

    }

    if(HasError()){
      Type("error");
      oa::dom_ptr<DOMNode> errorNode(BuildError());
      queryElem->appendChild(errorNode.release());
    }else if(Type()=="result"){
      oa::dom_ptr<DOMElement> operElem;
      oa::xmlstr<XMLCh,char> operName("oper"),
	uriName("uri"),
	uriValue;

      cont_type::iterator pos = m_opers.begin();
      for(;pos!=m_opers.end();++pos){
	if(pos->empty()){continue;}
	uriValue.set(pos->c_str());
	operElem.reset(doc->createElement(operName.get()));
	operElem->setAttribute(uriName.get(),uriValue.get());
	queryElem->appendChild(operElem.release());
      }
    }
    oa::dom_ptr<DOMNode> iqElem(BuildIq());
    iqElem->appendChild(queryElem);
    return iqElem.release();
    /*
    m_node.reset(BuildIq());
    m_node->appendChild(queryElem);
    return m_node.get();
    */
  }
}
