/* -*- Mode: c++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 * XData library
 * Contains objects to aid in Jabber client development and extend the
 * basic functionality of jabber.
 *
 * Copyright (C) 2003 Justin Kirby
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Justin Kirby <justink@rochester.rr.com>
 *
 *******************************************************************/
#include "framework/first.h"
#include "openaether.hpp"
#include "framework/xmlstr.hpp"
#include "framework/dom_ptr.hpp"
#include "framework/xmlutil.hpp"
#include "xdata/field.hpp"

XERCES_CPP_NAMESPACE_USE
namespace jabber
{
  DOMElement*const Field::BuildField()
  {
    DOMDocument*doc=oa::XMLUtils().Document(DocUri(),"oa");
    oa::xmlstr<XMLCh,char> fieldName("field"),
      xmlns(Xmlns().c_str());

    oa::dom_ptr<DOMElement> field(doc->createElementNS(xmlns,fieldName));
    //handle type attribute
    {
      oa::xmlstr<XMLCh,char> typeName("type"),
        typeValue(FieldType().toString().c_str());
      field->setAttribute(typeName,typeValue);
    }
    //handle label attribute, this is not required
    if(!Label().empty()){
      oa::xmlstr<XMLCh,char> labelName("label"),
        labelValue(Label().c_str());
      field->setAttribute(labelName,labelValue);
    }

    //handle var attribute,not required
    if(!Var().empty()){
      oa::xmlstr<XMLCh,char> varName("var"),
        varValue(Var().c_str());
      field->setAttribute(varName,varValue);
    }

    /* is this required? */
    if(Required()){
      oa::xmlstr<XMLCh,char> reqName("required");
      oa::dom_ptr<DOMElement> req(doc->createElementNS(xmlns,reqName));
      field->appendChild(req.release());
    }

    return field.release();

  }
};
