/* -*- Mode: c++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 * XData library
 * Contains objects to aid in Jabber client development and extend the
 * basic functionality of jabber.
 *
 * Copyright (C) 2003 Justin Kirby
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Justin Kirby <justink@rochester.rr.com>
 *
 *******************************************************************/
#ifndef OAJABBER_OAXDATA_FIELD_HPP
#define OAJABBER_OAXDATA_FIELD_HPP
#include <string>
#include <vector>
#include "xstrm/xmlinfo.h"
#include "xdata/fieldtype.hpp"
#include "xdata/fieldoption.hpp"
#include "xdata/fieldvalidate.hpp"

namespace jabber
{
  /**
   * Basic interface for all xdata fields.
   */
  class Field : public jabber::XmlInfo
  {
  public:
    typedef std::vector<std::string> value_cont;
    typedef std::vector<XFieldOption> opt_cont;
    typedef std::vector<XFieldType> type_cont;
  public:
    Field()
      : XmlInfo(){}
    
    Field(const Field& f)
      : XmlInfo(f){}

    virtual ~Field(){}

    /**@name XmlInfo*/
    //@{
    //note: we don't impl uri, cause its class specific
    virtual std::string Xmlns()const{
      return "jabber:x:data";
    }
    virtual void Xmlns(const std::string& ns){}
    //@}

    /**@name xdata field info */
    //@{
    /**@name getters */
    //@{
    virtual std::string Var()const=0;
    virtual void Var(const std::string &v)=0;

    virtual XFieldType FieldType()const=0;
    virtual void FieldType(const XFieldType& t)=0;

    virtual std::string Label()const=0;
    virtual void Label(const std::string &lbl)=0;

    virtual std::string Description()const=0;
    virtual void Description(const std::string &desc)=0;
    //@}
    //@}

    virtual bool Required()const=0;
    virtual void Required(bool req)=0;
    
    virtual void Value(const std::string &val)=0;
    virtual std::string Value()const=0;
    virtual value_cont::const_iterator ValuesBegin()const=0;
    virtual value_cont::const_iterator ValuesEnd()const=0;
    virtual void RemoveValue(const std::string &val)=0;

    virtual opt_cont::const_iterator OptionsBegin()const=0;
    virtual opt_cont::const_iterator OptionsEnd()const=0;
    virtual void AddOption(const std::string &lbl, const std::string &val)=0;
    virtual void AddOption(const XFieldOption& opt)=0;
    virtual void RemoveOption(const XFieldOption &opt)=0;

    virtual FieldValidate Validate()const=0;
    virtual void Validate(const FieldValidate& fv)=0;
  protected:
    virtual xercesc::DOMElement*const BuildField();

  };
};


#endif
