/* -*- Mode: c++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 * XData library
 * Contains objects to aid in Jabber client development and extend the
 * basic functionality of jabber.
 *
 * Copyright (C) 2003 Justin Kirby
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Justin Kirby <justink@rochester.rr.com>
 *
 *******************************************************************/
#include "framework/first.h"
#include "openaether.hpp"
#include "framework/xmlutil.hpp"
#include "framework/xmlstr.hpp"
#include "framework/dom_ptr.hpp"
#include "xdata/fieldlistmulti.hpp"

XERCES_CPP_NAMESPACE_USE
namespace jabber
{
  void FieldListMulti::RemoveValue(const std::string &val)
  {
    m_values.erase(std::remove_if(m_values.begin(),m_values.end(),
				  std::bind2nd(std::equal_to<std::string>(),val)),
		   m_values.end());
  }
  void FieldListMulti::RemoveOption(const XFieldOption &opt)
  {
    m_options.erase(std::remove_if(m_options.begin(),m_options.end(),
				   std::bind2nd(std::equal_to<XFieldOption>(),opt)),
		    m_options.end());
				   
  }
  DOMNode*const FieldListMulti::BuildXml()
  {
    DOMDocument*doc=oa::XMLUtils().Document(DocUri(),"oa");
    oa::dom_ptr<DOMElement> field(BuildField());

    oa::xmlstr<XMLCh,char> xmlns(Xmlns().c_str());

    opt_cont::iterator pos=m_options.begin();
    for(;pos!=m_options.end();++pos){
      field->appendChild(pos->BuildXml(doc,xmlns));
    }

    value_cont::iterator vpos=m_values.begin();
    for(;vpos!=m_values.end();++vpos){
      field->appendChild(BuildValue(*vpos));
    }
    
    return field.release();
  }
  DOMNode* FieldListMulti::BuildValue(const std::string& val)
  {
    DOMDocument*doc=oa::XMLUtils().Document(DocUri(),"oa");
    oa::xmlstr<XMLCh,char> xmlns(Xmlns().c_str()),
      valName("value"),
      valVal(val.c_str());

    oa::dom_ptr<DOMElement> valElem(doc->createElementNS(xmlns,valName));
    oa::dom_ptr<DOMText> txt(doc->createTextNode(valVal));
    valElem->appendChild(txt.release());
    return valElem.release();
  }

};
