/* -*- Mode: c++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 * XData library
 * Contains objects to aid in Jabber client development and extend the
 * basic functionality of jabber.
 *
 * Copyright (C) 2003 Justin Kirby
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Justin Kirby <justink@rochester.rr.com>
 *
 *******************************************************************/
#include "framework/first.h"
#include "openaether.hpp"
#include "oapr/oapr_exception.hpp"
#include "framework/xmlutil.hpp"
#include "framework/xmlstr.hpp"
#include "framework/dom_ptr.hpp"
#include "xdata/fieldlistsingle.hpp"

XERCES_CPP_NAMESPACE_USE
namespace jabber
{
  void FieldListSingle::RemoveOption(const XFieldOption &opt)
  {
    m_options.erase(std::remove_if(m_options.end(),m_options.begin(),
                                   std::bind2nd(std::equal_to<XFieldOption>(),opt)));
  }
  DOMNode*const FieldListSingle::BuildXml()
  {
    try{
      DOMDocument*doc=oa::XMLUtils().Document(DocUri(),"oa");

      oa::xmlstr<XMLCh,char> fieldName("field"),
        xmlns(Xmlns().c_str());

      oa::dom_ptr<DOMElement> field(BuildField());
      oa::dom_ptr<DOMDocumentFragment> fieldFrag(doc->createDocumentFragment());

      opt_cont::const_iterator pos=OptionsBegin();
      for(;pos!=OptionsEnd();++pos){
        fieldFrag->appendChild(pos->BuildXml(doc,xmlns));
      }
      field->appendChild(fieldFrag.release());
      return field.release();
    }OAPR_CATCH();
  }
};
