/* -*- Mode: c++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 * XData library
 * Contains objects to aid in Jabber client development and extend the
 * basic functionality of jabber.
 *
 * Copyright (C) 2003 Justin Kirby
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Justin Kirby <justink@rochester.rr.com>
 *
 *******************************************************************/
#ifndef OAJABBER_OAXDATA_FIELDSINGLE_HPP
#define OAJABBER_OAXDATA_FIELDSINGLE_HPP

#include "xdata/field.hpp"

namespace jabber
{
  class FieldListSingle : public Field
  {
  public:
    FieldListSingle()
      : m_type(XFieldType::List_Single)
        ,m_required(false){}
    FieldListSingle(const FieldListSingle& fs)
      : Field(fs)
        ,m_var(fs.m_var)
        ,m_type(fs.m_type)
        ,m_label(fs.m_label)
        ,m_description(fs.m_description)
        ,m_required(fs.m_required)
        ,m_value(fs.m_value)
        ,m_options(fs.m_options){}
    virtual ~FieldListSingle(){}

    virtual std::string Var()const{return m_var;}
    virtual void Var(const std::string &v){m_var =v;}

    virtual XFieldType FieldType()const{return m_type;}
    virtual void FieldType(const XFieldType& t){}

    virtual std::string Label()const{return m_label;}
    virtual void Label(const std::string &lbl){m_label=lbl;}

    virtual std::string Description()const{return m_description;}
    virtual void Description(const std::string &desc){m_description=desc;}

    virtual bool Required()const{return m_required;}
    virtual void Required(bool req){m_required=req;}
    
    virtual std::string Value()const{return m_value;}
    virtual void Value(const std::string &val){m_value=val;}

    virtual value_cont::const_iterator ValuesBegin()const{
      return value_cont::const_iterator(0);
    }
    virtual value_cont::const_iterator ValuesEnd()const{
      return value_cont::const_iterator(0);
    }
    virtual void RemoveValue(const std::string &val){}

    virtual opt_cont::const_iterator OptionsBegin()const{
      return m_options.begin();
    }
    virtual opt_cont::const_iterator OptionsEnd()const{
      return m_options.end();
    }
    virtual void AddOption(const std::string &lbl, const std::string &val){
      XFieldOption opt(lbl,val);
      m_options.push_back(opt);
    }
    virtual void AddOption(const XFieldOption& opt){
      m_options.push_back(opt);
    }
    virtual void RemoveOption(const XFieldOption &opt);
    virtual xercesc::DOMNode*const BuildXml();
  private:
    std::string m_var;
    XFieldType m_type;
    std::string m_label;
    std::string m_description;
    bool m_required;
    std::string m_value;
    std::vector<XFieldOption> m_options;
  };
};

#endif
