/* -*- Mode: c++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 * XData library
 * Contains objects to aid in Jabber client development and extend the
 * basic functionality of jabber.
 *
 * Copyright (C) 2003 Justin Kirby
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Justin Kirby <justink@rochester.rr.com>
 *
 *******************************************************************/
#include "framework/first.h"
#include "openaether.hpp"
#include "oapr/oapr_exception.hpp"
#include "oapr/oapr_exc_utils.hpp"
#include "framework/xmlutil.hpp"
#include "framework/xmlstr.hpp"
#include "framework/dom_ptr.hpp"

#include "xdata/fieldoption.hpp"

XERCES_CPP_NAMESPACE_USE
namespace jabber
{
  DOMElement*const XFieldOption::BuildXml(DOMDocument*const doc,
                                          const XMLCh*const xmlns)const
  {
    oa::xmlstr<XMLCh,char> optName("option"),
      labelName("label"),
      valueName("value");

    /** this is required, it makes no sense to have an option without 
     * either of these
     */
    if(m_label.empty() || m_value.empty()){
      OAPR_THROW(oapr::invalid_state,"Label or value is empty");
    }
    oa::xmlstr<XMLCh,char> labelValue(m_label.c_str()),
      valueValue(m_value.c_str());

    oa::dom_ptr<DOMElement> opt(doc->createElementNS(xmlns,optName));
    opt->setAttribute(labelName,labelValue);

    oa::dom_ptr<DOMElement> val(doc->createElement(valueName));

    oa::dom_ptr<DOMText> vtxt(doc->createTextNode(valueValue));
    val->appendChild(vtxt.release());
    opt->appendChild(val.release());
    return opt.release();
  }
};
