/* -*- Mode: c++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 * XData library
 * Contains objects to aid in Jabber client development and extend the
 * basic functionality of jabber.
 *
 * Copyright (C) 2003 Justin Kirby
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Justin Kirby <justink@rochester.rr.com>
 *
 *******************************************************************/
#include "framework/first.h"
#include "openaether.hpp"
#include "framework/xmlutil.hpp"
#include "framework/dom_ptr.hpp"
#include "framework/xmlstr.hpp"
#include "xdata/fieldtextmulti.hpp"

XERCES_CPP_NAMESPACE_USE
namespace jabber
{
  void FieldTextMulti::Value(const std::string &val)
  {
    m_values.push_back(val);
  }
  std::string FieldTextMulti::Value()const
  {
    if(m_values.size()>0){
      return m_values[0];
    }
    return "";
  }
  void FieldTextMulti::RemoveValue(const std::string &val)
  {
    value_cont::iterator rm=std::remove_if(m_values.begin(),m_values.end(),
                               std::bind2nd(std::equal_to<std::string>(),val));
    m_values.erase(rm,m_values.end());

  }
  std::string FieldTextMulti::xml()
  {
    return "";
  }
  DOMNode*const FieldTextMulti::BuildXml()
  {
    DOMDocument *doc=oa::XMLUtils().Document(DocUri(),"oa");
    oa::dom_ptr<DOMElement> field(BuildField());



    oa::xmlstr<XMLCh,char> valueName("value"),valueValue;
    oa::dom_ptr<DOMElement> val;
    oa::dom_ptr<DOMText> valText;
    value_cont::iterator pos=m_values.begin();
    for(;pos!=m_values.end();++pos){
      valueValue.set(pos->c_str());
      val.reset(doc->createElement(valueName));
      valText.reset(doc->createTextNode(valueValue));
      val->appendChild(valText.release());
      field->appendChild(val.release());
    }
    return field.release();
  }

};
