/* -*- Mode: c++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 * XData library
 * Contains objects to aid in Jabber client development and extend the
 * basic functionality of jabber.
 *
 * Copyright (C) 2003 Justin Kirby
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Justin Kirby <justink@rochester.rr.com>
 *
 *******************************************************************/
#ifndef OAJABBER_OAXDATA_FIELDTEXTMULTI_HPP
#define OAJABBER_OAXDATA_FIELDTEXTMULTI_HPP

#include "xdata/field.hpp"

namespace jabber
{
  class FieldTextMulti : public Field
  {
  public:
    FieldTextMulti()
      : Field(){}
    FieldTextMulti(const FieldTextMulti& ftm)
      : Field(ftm),
        m_var(ftm.m_var),
        m_label(ftm.m_label),
        m_desc(ftm.m_desc),
        m_required(ftm.m_required),
        m_values(ftm.m_values){}
    ~FieldTextMulti(){}

    virtual std::string Var()const{return m_var;}
    virtual void Var(const std::string &v){m_var=v;}

    virtual XFieldType FieldType()const{
      return XFieldType(XFieldType::Text_Multi);
    }
    virtual void FieldType(const XFieldType& t){}

    virtual std::string Label()const{return m_label;}
    virtual void Label(const std::string &lbl){m_label=lbl;}

    virtual std::string Description()const{return m_desc;}
    virtual void Description(const std::string &desc){m_desc=desc;}

    virtual bool Required()const{return m_required;}
    virtual void Required(bool req){m_required=req;}
    
    virtual void Value(const std::string &val);
    virtual std::string Value()const;
    virtual value_cont::const_iterator ValuesBegin()const{
      return m_values.begin();
    }
    virtual value_cont::const_iterator ValuesEnd()const{
      return m_values.end();
    }
    virtual void RemoveValue(const std::string &val);

    virtual opt_cont::const_iterator OptionsBegin()const{
      return opt_cont::const_iterator(0);
    }
    virtual opt_cont::const_iterator OptionsEnd()const{
      return opt_cont::const_iterator(0);
    }
    virtual void AddOption(const std::string &lbl, const std::string &val){}
    virtual void AddOption(const XFieldOption& opt){}
    virtual void RemoveOption(const XFieldOption &opt){}

    virtual const char*const Uri()const{
      return "xmpp:xmlns=jabber:x:data?text-multi";
    }
    virtual std::string xml();
    virtual xercesc::DOMNode*const BuildXml();

  private:
    std::string m_var;
    std::string m_label;
    std::string m_desc;
    bool m_required;

    value_cont m_values;
  };
}
#endif
