/* -*- Mode: c++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 * XData library
 * Contains objects to aid in Jabber client development and extend the
 * basic functionality of jabber.
 *
 * Copyright (C) 2003 Justin Kirby
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Justin Kirby <justink@rochester.rr.com>
 *
 *******************************************************************/
#ifndef OAJABBER_OAXDATA_FORMITEM_HPP
#define OAJABBER_OAXDATA_FORMITEM_HPP
#include <vector>
#include "boost/shared_ptr.hpp"
#include "xdata/field.hpp"


namespace jabber
{
  class FormItem : public jabber::XmlInfo
  {
  public:
    typedef Field field_type;
    typedef boost::shared_ptr<field_type> field_ptr;
    typedef std::vector<field_ptr> field_cont;
  public:
    FormItem()
      : XmlInfo(){}
    FormItem(const FormItem& fi)
      : XmlInfo(fi)
        ,m_fields(fi.m_fields){}
    ~FormItem(){}

    /**@name XmlInfo*/
    //@{
    virtual std::string Xmlns()const{
      return "jabber:x:data";
    }
    virtual void Xmlns(const std::string& ns){}
    //@}
    virtual const char*const Uri()const{
      return "xmlns:xmpp=jabber:x:data?fixed";
    }
    field_cont& Fields(){
      return m_fields;
    }
    virtual xercesc::DOMNode*const BuildXml();
  private:
    field_cont m_fields;
  };
};
#endif
