/* -*- Mode: c++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 * XData library
 * Contains objects to aid in Jabber client development and extend the
 * basic functionality of jabber.
 *
 * Copyright (C) 2003 Justin Kirby
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Justin Kirby <justink@rochester.rr.com>
 *
 *******************************************************************/
#include "framework/first.h"
#include "openaether.hpp"
#include "framework/xerces_xmlns.hpp"
#include "framework/xmlutil.hpp"
#include "framework/xmlstr.hpp"
#include "framework/dom_ptr.hpp"
#include "xdata/jabxdata.hpp"


XERCES_CPP_NAMESPACE_USE
namespace jabber
{
  std::string JabData::xml()
  {
    return "";
  }
  DOMNode*const JabData::BuildXml()
  {
    DOMDocument *doc = oa::XMLUtils().Document(DocUri(),"oa");
    oa::xmlstr<XMLCh,char> xName("x"),
      xns(Xmlns().c_str()),
      typeValue,
      typeName("type");
      

    oa::dom_ptr<DOMElement> x(doc->createElementNS(xns,xName));
    typeValue.set(m_form.Type().toString().c_str());
    x->setAttribute(typeName,typeValue);

    /****
     * dealing with title element
     */
    oa::dom_ptr<DOMElement> title;
    oa::xmlstr<XMLCh,char> titleName("title"),
      titleValue;
    //if the there is no title and its of type form insert an empty elem
    if(m_form.Title().empty() &&
       m_form.Type() == XFormType::Form){
      //insert an empty title element
      title.reset(doc->createElement(titleName));
      x->appendChild(title.release());
    }else if(!m_form.Title().empty()){
      //if we do have a title, add it no matter what
      titleValue.set(m_form.Title().c_str());
      title.reset(doc->createElement(titleName));
      oa::dom_ptr<DOMText> titleText(doc->createTextNode(titleValue));
      title->appendChild(titleText.release());
      x->appendChild(title.release());
    }

    /********
     * deal with instructions element
     */

    /* only insert an instruction node if its of type form */
    if(m_form.Type() == XFormType::Form){
      oa::xmlstr<XMLCh,char> instName("instructions");      
      oa::dom_ptr<DOMElement> inst(doc->createElement(instName));
      if(!m_form.Instructions().empty()){
        oa::xmlstr<XMLCh,char> instValue(m_form.Instructions().c_str());
        oa::dom_ptr<DOMText> instText(doc->createTextNode(instValue));
        inst->appendChild(instText.release());
      }
      x->appendChild(inst.release());
    }
    /******
     * if this is of type form then only deal with fields
     * or if there are no reported or items avail, just dump
     * fields
     */
    if(m_form.Type() == XFormType::Form ||
       (m_form.Reported().size()==0)){

      XDataForm::field_cont &fields=m_form.Fields();
      XDataForm::field_cont::iterator pos=fields.begin();
      oa::dom_ptr<DOMDocumentFragment> fieldFrag(doc->createDocumentFragment());
      for(;pos!=fields.end();++pos){
        fieldFrag->appendChild((*pos)->BuildXml());
      }
      x->appendChild(fieldFrag.release());
    }else{
      /** apparently we have reported or items most likely both
       * but guaruntee that if we only have reported we throw
       * out with no items
       */
      if(m_form.Items().size()){
        /**@todo throw out since this is unresolvable*/
      }
      x->appendChild(BuildReported());
      x->appendChild(BuildItems());
    }
    return x.release();
  }
  DOMNode*const JabData::BuildReported()
  {
    DOMDocument *doc=oa::XMLUtils().Document(DocUri(),"oa");
    oa::xmlstr<XMLCh,char> reportedName("reported");

    oa::dom_ptr<DOMElement>  reportedNode(doc->createElement(reportedName));

    oa::dom_ptr<DOMDocumentFragment> reportFrag(doc->createDocumentFragment());
    XDataForm::field_cont &reported=m_form.Reported();
    XDataForm::field_cont::iterator pos=reported.begin();
    for(;pos!=reported.end();++pos){
      reportFrag->appendChild((*pos)->BuildXml());
    }
    reportedNode->appendChild(reportFrag.release());
    return reportedNode.release();
  }
  DOMNode*const JabData::BuildItems()
  {
    DOMDocument *doc=oa::XMLUtils().Document(DocUri(),"oa");
    oa::xmlstr<XMLCh,char> itemname("item");
    oa::dom_ptr<DOMDocumentFragment> itemsFrag(doc->createDocumentFragment());

    XDataForm::item_cont &items=m_form.Items();
    XDataForm::item_cont::iterator pos=items.begin();
    for(;pos!=items.end();++pos){
      itemsFrag->appendChild((*pos)->BuildXml());
    }

    return itemsFrag.release();
  }
};
