/* -*- Mode: c++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 * XData library
 * Contains objects to aid in Jabber client development and extend the
 * basic functionality of jabber.
 *
 * Copyright (C) 2003 Justin Kirby
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Justin Kirby <justink@rochester.rr.com>
 *
 *******************************************************************/
#ifndef OAJABBER_OAXDATA_XDATAFORM_HPP
#define OAJABBER_OAXDATA_XDATAFORM_HPP
#include <boost/shared_ptr.hpp>
#include "xdata/field.hpp"
#include "xdata/formtype.hpp"
#include "xdata/formitem.hpp"

namespace jabber
{
  class XDataForm
  {
  public:
    typedef Field field_type;
    typedef boost::shared_ptr<field_type> field_ptr;
    typedef std::vector<field_ptr> field_cont;
    typedef FormItem item_type;
    typedef boost::shared_ptr<item_type> item_ptr;
    typedef std::vector<item_ptr> item_cont;

  public:
    XDataForm()
      : m_type(XFormType::Form)
    {}
    XDataForm(const XDataForm& df)
      : m_title(df.m_title)
        ,m_instructions(df.m_instructions)
        ,m_type(df.m_type)
        ,m_fields(df.m_fields)
        ,m_reported(df.m_reported)
        ,m_items(df.m_items)
    {}
    virtual ~XDataForm(){}

    std::string Title()const{ return m_title; }
    void Title(const std::string& ttl){ m_title=ttl; }

    std::string Instructions()const{ return m_instructions; }
    void Instructions(const std::string& ins){ m_instructions; }

    XFormType Type()const{ return m_type; }
    void Type(const XFormType& xft){ m_type=xft; }

    field_cont& Fields(){return m_fields;}
    field_cont& Reported(){return m_reported;}
    item_cont& Items(){return m_items;}
  private:
    std::string m_title;
    std::string m_instructions;
    XFormType m_type;
    field_cont m_fields;
    field_cont m_reported;
    item_cont m_items;

  };
};

#endif
