/********************************************************************
 * OAjabber library
 * Contains objects to aid in Jabber client development and extend the
 * basic functionality of jabber.
 *
 * Copyright (C) 2002 Justin Kirby
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Justin Kirby <justink@rochester.rr.com>
 *
 *******************************************************************/
#ifndef OAJABBER_IJABERROR_H
#define OAJABBER_IJABERROR_H

#include "xstrm/oaxstrm_exports.h"
#include <string>

namespace jabber
{
  class OAXSTRM_API JabError;
  /** @ingroup gJabPacket
   */

  ///A jabber error
  /**
   * The central jabber error class. 
   * Assign to a packet to send
   */
  class JabError
  {
  public:
    JabError(){}
    JabError(const JabError &je)
      : m_code(je.m_code),
	m_message(je.m_message){}

    virtual ~JabError(){}
    
    JabError& operator=(const JabError& je){
      m_code = je.m_code;
      m_message = je.m_message;
      return *this;
    }

    ///Retrieve the numerical code
    std::string Code() const{
      return m_code;
    }

    ///Retrieve the error message
    std::string Message() const{
      return m_message;
    }

    ///Set the code
    void Code(const std::string &code){
      m_code = code;
    }

    ///Set the error message
    void Message(const std::string &msg){
      m_message = msg;
    }
  protected:
    std::string m_code;
    std::string m_message;
  };
};

#endif
