/* -*- Mode: c++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 * OAjabber library
 * Contains objects to aid in Jabber client development and extend the
 * basic functionality of jabber.
 *
 * Copyright (C) 2003 Justin Kirby
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Justin Kirby <justink@rochester.rr.com>
 *
 *******************************************************************/
#ifndef OAJABBER_IJABX_H
#define OAJABBER_IJABX_H
#include "xstrm/oaxstrm_exports.h"
#include "xstrm/xmlinfo.h"
#include <boost/smart_ptr.hpp>
namespace jabber
{
	class OAXSTRM_API JabX;
  class JabX : public XmlInfo
  {
  public:
    typedef boost::shared_ptr<JabX> clone_type;
  public:
    /** @name Contructors */
    //@{
    JabX()
      : XmlInfo(),
        m_attached(false){}
    JabX(const JabX& x)
      : XmlInfo(x),
        m_attached(x.m_attached){}
    //@}
    virtual ~JabX(){}

    virtual clone_type Clone()=0;


    /**
     * Indicate whether this X was attached to a
     * stanza.
     * @param attached true indicates this is attached, false
     * indicates its standalone
     */
    void Attached(bool attached){m_attached=attached;}

    /**
     * Determine if this is attached
     * @return Returns true if attached to a stanza, false otherwise
     */
    bool Attached()const{return m_attached;}      
  private:
    bool m_attached;
  };
};

#endif
