/* -*- Mode: c++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 * OAjabber library
 * Contains objects to aid in Jabber client development and extend the
 * basic functionality of jabber.
 *
 * Copyright (C) 2001 Justin Kirby
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Justin Kirby <justink@rochester.rr.com>
 *
 *******************************************************************/
#ifndef OAJABBER_JABAUTH_H
#define OAJABBER_JABAUTH_H
#include "xstrm/jabpacket.h"
#include "xstrm/jabbldauth.h"
#include "framework/thread_thread.hpp"

namespace jabber
{
  class OAXSTRM_API JabAuth;
  class OAXSTRM_API JabAuthClient;
  class OAXSTRM_API JabAuthTrans;

  class JabAuth : public JabPacket
  {
  public:
    JabAuth(std::string classid) : JabPacket("stream:stream",classid){}
    JabAuth(const JabAuth & ja) : JabPacket(ja){}
    virtual ~JabAuth(){}
    using JabPacket::Id;
    virtual std::string Id()const;
    virtual std::string Domain() const=0;
    virtual std::string Namespace() const=0;
    virtual std::string Stream() const=0;
    virtual std::string Jid() const=0;
    virtual std::string Pwd() const=0;
    virtual void Jid(std::string jid)=0;
    virtual void Pwd(std::string pwd)=0;
    virtual JabAuth* Clone()=0;
    virtual BuildAuth* MakeBuilder()=0;
    virtual std::string xml(xercesc::DOMNode*const nd);
    virtual std::string xml();
    using JabPacket::BuildXml;
    virtual xercesc::DOMNode*const BuildXml(xercesc::DOMNode*const nd)=0;
  };

  class JabAuthClient : public JabAuth
  {
  public:
    JabAuthClient(std::string jid="", std::string pwd="");
    JabAuthClient(const JabAuthClient &jc);
    virtual ~JabAuthClient(){}
    virtual const char*const Uri()const{
      return "xmpp:xmlns=jabber:iq:auth";
    }
    std::string Xmlns()const{
      return "jabber:iq:auth";
    }

    std::string Domain() const{return m_domain;}
    std::string Namespace() const{return m_namespace;}
    std::string Stream() const;//
    std::string Jid() const{return m_jid;}
    std::string Pwd() const{return m_pwd;}
    void Jid(std::string jid);//
    void Pwd(std::string pwd){
      m_pwd=pwd;
    }

    virtual JabAuth* Clone(){
      return new JabAuthClient(*this);
    }
    virtual BuildAuth* MakeBuilder(){
      return new BuildAuthClient;
    }
    virtual xercesc::DOMNode*const BuildXml();
    virtual xercesc::DOMNode*const BuildXml(xercesc::DOMNode*const nd);

  protected:
    std::string m_domain;
    std::string m_jid;
  private:
    std::string m_pwd;
    std::string m_namespace;
  };

  class JabAuthTrans : public JabAuth
  {
  public:
    JabAuthTrans(std::string srv, std::string pwd);
    JabAuthTrans(const JabAuthTrans& jt);
    virtual ~JabAuthTrans(){}
    virtual const char*const Uri()const{
      return "xmpp:jabber:handshake";
    }
    virtual std::string Xmlns()const{
      return "jabber:iq:accept";
    }

    virtual std::string Domain() const{return m_server;}
    virtual std::string Namespace() const{return m_namespace;}
    virtual std::string Stream() const;
    virtual std::string Jid() const{return m_server;}
    virtual std::string Pwd() const{return m_secret;}
    virtual void Jid(std::string jid){m_server=jid;}
    virtual void Pwd(std::string pwd){m_secret=pwd;}

    //    virtual std::string xml();
    virtual xercesc::DOMNode*const BuildXml();
    virtual xercesc::DOMNode*const BuildXml(xercesc::DOMNode*const nd);
    virtual JabAuth* Clone(){
      return new JabAuthTrans(*this);
    }
    virtual BuildAuth* MakeBuilder(){
      return new BuildAuthTrans;
    }
  private:
    bool m_done;
    std::string m_server;
    std::string m_secret;
    std::string m_namespace;
    typedef oa::oa_thread::lock_type lock_type;
    oa::oa_thread::mutex m_mutex;
  };
};
#endif
