/********************************************************************
 * OAjabber library
 * Contains objects to aid in Jabber client development and extend the
 * basic functionality of jabber.
 *
 * Copyright (C) 2001 Justin Kirby
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Justin Kirby <justink@rochester.rr.com>
 *
 *******************************************************************/
#include "framework/first.h"
#include "openaether.hpp"
#include <xercesc/dom/DOMNodeList.hpp>
#include "xstrm/jabberdom.h"
#include "framework/xmlstr.hpp"

XERCES_CPP_NAMESPACE_USE
namespace jabber
{
  void JabberDom::PopPacket()
  {
    DOMNode *packet=m_packets.front();
    if(packet){
      m_packets.pop_front();
    }
  }
  void JabberDom::startDocument() {
    DomBuilder::startDocument();
  }
  void JabberDom::startElement(const XMLCh *const uri, 
			       const XMLCh *const localname,
			       const XMLCh *const qname,
			       const Attributes& attributes){
    DomBuilder::startElement(uri,localname,qname,attributes);
    ++m_counter;
  }

  void JabberDom::characters(const XMLCh* const chars,
			     const unsigned int length){
    DomBuilder::characters(chars,length);   
  }
  
  void JabberDom::endElement(const XMLCh *const uri,
			     const XMLCh *const localname,
			     const XMLCh *const qname){
    DomBuilder::endElement(uri,localname,qname);
    --m_counter;
    if(m_counter==1){
      SetPacket();
      CacheLimit();
    }
  }
  void JabberDom::endDocument(){
    DomBuilder::endDocument();
    /**\todo kill the stream here*/
  }

  void JabberDom::SetPacket()
  {
    DOMElement* doc = getElement();
    if(!doc){return;}

    if(doc->hasChildNodes()){
      m_packets.push_back(doc->getLastChild());
    }
  }
  void JabberDom::CacheLimit()
  {
    DOMElement *doc = getElement();
    DOMNodeList *ndList=doc->getChildNodes();
    if(ndList->getLength()>m_cacheSize){
      for(unsigned int i=0; i<ndList->getLength();++i){
	DOMNode*nd=ndList->item(i);
	if(CanRemove(nd)){
	  nd = doc->removeChild(nd);
	  nd->release();
	}
      }
    }        
  }
  bool JabberDom::CanRemove(DOMNode*const nd)
  {
    cont_type::iterator pos=m_packets.begin();
    for(;pos!=m_packets.end();++pos){
      if((*pos)->isSameNode(nd)){
	return false;
      }
    }
    return true;
  }
}
