/* -*- Mode: c++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 * OAjabber library
 * Contains objects to aid in Jabber client development and extend the
 * basic functionality of jabber.
 *
 * Copyright (C) 2001 Justin Kirby
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Justin Kirby <justink@rochester.rr.com>
 *
 *******************************************************************/
#ifndef OAJABBER_JABBERDOM_H
#define OAJABBER_JABBERDOM_H
#include <list>
#include "framework/dombuilder.hpp"


namespace jabber
{
  class JabberDom : public oa::DomBuilder
  {
  public:
    JabberDom()
      : DomBuilder(),
        m_cacheSize(0),
        m_counter(0)
    {}
    virtual ~JabberDom(){}

    virtual void SetCache(unsigned int sz){
      m_cacheSize=sz;
    }
    virtual bool GetLevel()const{
      return (m_counter!=0);
    }
    virtual xercesc::DOMNode* NextPacket()const{
      if(m_packets.empty()){return 0;}
      return m_packets.front();
    }
    virtual void PopPacket();

    /* we are only going to redo the necessary
     * functions from dombuilder
     */
    virtual void startDocument();
    virtual void startElement(const XMLCh *const uri, 
			      const XMLCh *const localname,
			      const XMLCh *const qname,
			      const xercesc::Attributes& attributes);

    virtual void characters(const XMLCh* const chars,
			    const unsigned int length);

    virtual void endElement(const XMLCh *const uri,
			    const XMLCh *const localname,
			    const XMLCh *const qname);
    virtual void endDocument();
  protected:
    virtual void SetPacket();
    virtual void CacheLimit();
    virtual bool CanRemove(xercesc::DOMNode*const nd);
  private:
    typedef std::list<xercesc::DOMNode*> cont_type;
    unsigned int m_cacheSize;
    unsigned int m_counter;
    cont_type m_packets;
  };
};
#endif
