/* -*- Mode: c++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 * OAjabber library
 * Contains objects to aid in Jabber client development and extend the
 * basic functionality of jabber.
 *
 * Copyright (C) 2001 Justin Kirby
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Justin Kirby <justink@rochester.rr.com>
 *
 *******************************************************************/
#include "framework/first.h"
#include "openaether.hpp"
#include <boost/smart_ptr.hpp>
#include <xercesc/util/XMLString.hpp>
#include <xercesc/dom/DOMElement.hpp>
#include "xstrm/jabbldauth.h"
#include "framework/xmlstr.hpp"
#include "framework/debug_xml.hpp"

XERCES_CPP_NAMESPACE_USE
namespace jabber
{
  bool BuildAuthClient::CanBuild(DOMNode *const nd)const
  {
    /* either:
     * - it is an iq of type result with no children
     * - it is an iq of type error with error child
     * and id of m_id
     */
    oapr::debug::write("TRYING to build");
    oapr::debug::write(oa::debug_xml(nd));
    if(m_used){return false;}/* only build once!*/
    if(!m_auth && !m_id.empty()){return false;}
    if(nd->getNodeType()!=DOMNode::ELEMENT_NODE){return false;}    
    oa::xmlstr<XMLCh,char> iqName("iq")
      ,idName("id")
      ,idValue(m_id.c_str());

    if(!XMLString::compareString(iqName,nd->getLocalName()) &&
       ((DOMElement*)nd)->hasAttribute(idName)){
      const XMLCh* atVal=((DOMElement*)nd)->getAttribute(idName);
      if(!XMLString::compareString(idValue,atVal)){
        oapr::debug::write("CAN BUILD AUTH");
        return true;
      }
    }				
    return false;
  }
  JabPacket* BuildAuthClient::Build(DOMNode *const nd)
  {
    /* ok we said we can build this... don't bother checking*/
    boost::scoped_array<XMLCh> typeName(XMLString::transcode("type")),
      typeResult(XMLString::transcode("result")),
      typeValue;
    if(nd->getNodeType()==DOMNode::ELEMENT_NODE){
      const XMLCh* typeValue = ((DOMElement*)nd)->getAttribute(typeName.get());
      if(!XMLString::compareString(typeResult.get(),typeValue)){
        *m_auth=Auth_Succeeded;
      }else{
        *m_auth=Auth_Failed;
      }	
    }
    m_used=true;
    return 0;
  }

  bool BuildAuthTrans::CanBuild(DOMNode *const nd)const
  {
    if(m_used){return false;}/* only build once!*/
    boost::scoped_array<XMLCh> hand(XMLString::transcode("handshake"));
    if(!XMLString::compareString(hand.get(),nd->getNodeName())&&
       !nd->hasChildNodes()){
      return true;
    }
    /**\todo put stream:error check here*/    
    return false;
  }
  JabPacket* BuildAuthTrans::Build(DOMNode *const nd)
  {
    boost::scoped_array<XMLCh> hand(XMLString::transcode("handshake"));
    if(!XMLString::compareString(hand.get(),nd->getNodeName())&&
       !nd->hasChildNodes()){
      *m_auth=Auth_Succeeded;      
    }else{
      *m_auth=Auth_Failed;
    }
    m_used=true;
    return 0;
  }


};
