/* -*- Mode: c++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 * OAjabber library
 * Contains objects to aid in Jabber client development and extend the
 * basic functionality of jabber.
 *
 * Copyright (C) 2001 Justin Kirby
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Justin Kirby <justink@rochester.rr.com>
 *
 *******************************************************************/
#ifndef OAJABBER_JABBLDAUTH_H
#define OAJABBER_JABBLDAUTH_H
#include "xstrm/oaxstrm_exports.h"
#include "xstrm/jabbuild.h"

namespace jabber
{
	class OAXSTRM_API BuildAuth;
  class BuildAuth : public JabBuild
  {
  public:
    /* this is placed here to accomodate all classes that need it*/
    enum AUTH_STATE{Auth_Unknown=0,Auth_Failed,Auth_Succeeded};
  public:
    BuildAuth()
      : JabBuild(),
        m_auth(0),
        m_used(false){}
    BuildAuth(const BuildAuth& ba)
      : JabBuild(ba),
        m_auth(ba.m_auth),
        m_id(ba.m_id),
        m_used(ba.m_used){}
    virtual ~BuildAuth(){}
    virtual std::string ID()const{
      return "jabber::BuildAuth";
    }


    virtual void SetToken(AUTH_STATE *auth){m_auth=auth;}
    virtual void SetId(const std::string& id){m_id=id;}
  protected:
    AUTH_STATE *m_auth;
    std::string m_id;
    bool m_used;
  };

  class BuildAuthClient : public BuildAuth
  {
  public:
    BuildAuthClient()
      : BuildAuth(){}
    BuildAuthClient(const BuildAuthClient& bac)
      : BuildAuth(bac){}
    virtual ~BuildAuthClient(){}
    virtual JabBuild* Clone(){
      return new BuildAuthClient(*this);
    }
    virtual bool CanBuild(xercesc::DOMNode *const nd)const;
    virtual JabPacket* Build(xercesc::DOMNode *const nd);
  };
  class BuildAuthTrans : public BuildAuth
  {
  public:
    BuildAuthTrans()
      : BuildAuth(){}
    BuildAuthTrans(const BuildAuthTrans& bat)
      : BuildAuth(bat){}
    virtual ~BuildAuthTrans(){}
    virtual JabBuild* Clone(){
      return new BuildAuthTrans(*this);
    }
    virtual bool CanBuild(xercesc::DOMNode *const nd)const;
    virtual JabPacket* Build(xercesc::DOMNode *const nd);
  };
};

#endif
