/********************************************************************
 * OAjabber library
 * Contains objects to aid in Jabber client development and extend the
 * basic functionality of jabber.
 *
 * Copyright (C) 2001 Justin Kirby
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Justin Kirby <justink@rochester.rr.com>
 *
 *******************************************************************/
#include "framework/first.h"
#include "openaether.hpp"
#include "boost/smart_ptr.hpp"
#include <xercesc/util/XMLString.hpp>
#include <xercesc/dom/DOMNamedNodeMap.hpp>
#include <xercesc/dom/DOMNodeList.hpp>
#include <xercesc/dom/DOMElement.hpp>
#include "framework/xmlutil.hpp"
#include "framework/xmlstr.hpp"
#include "xstrm/jabbuild.h"


XERCES_CPP_NAMESPACE_USE
namespace jabber
{
  int JabBuild::GetTo(std::string &to, DOMNode*const nd)
  {
    oa::xmlstr<XMLCh,char> attrName("to");
    const XMLCh *attrTo = ((DOMElement *)nd)->getAttribute(attrName);
    if(!attrTo){
      return 1;
    }
    oa::XMLUtils().DomstrToStd(to,attrTo);
    return 0;
  }
  int JabBuild::GetFrom(std::string &to, DOMNode*const nd)
  {
    oa::xmlstr<XMLCh,char> attrName("from");
    const XMLCh* attrFrom = ((DOMElement*)nd)->getAttribute(attrName);
    if(!attrFrom){
      return 1;
    }

    oa::XMLUtils().DomstrToStd(to,attrFrom);
    return 0;
  }
  int JabBuild::GetId(std::string &to, DOMNode*const nd)
  {
    oa::xmlstr<XMLCh,char> attrName("id");
    const XMLCh* attrId = ((DOMElement*)nd)->getAttribute(attrName);
    if(!attrId){
      return 1;
    }
    oa::XMLUtils().DomstrToStd(to,attrId);
    return 0;
  }
  int JabBuild::GetType(std::string &to, DOMNode*const nd)
  {
    oa::xmlstr<XMLCh,char> attrName("type");
    const XMLCh* attrType = ((DOMElement*)nd)->getAttribute(attrName);
    if(!attrType){
      return 1;
    }
    oa::XMLUtils().DomstrToStd(to,attrType);
    return 0;
  }
  bool JabBuild::GetError(JabError &je, DOMNode *const nd)
  {
    boost::scoped_array<XMLCh> xmar(XMLString::transcode("error"));
    DOMNodeList *errors = ((DOMElement*)nd)->getElementsByTagName(xmar.get());
    if(errors->getLength()<=0){
      return false;
    }
    DOMNode* error = errors->item(0);
    //getting the attribute code value
    xmar.reset(XMLString::transcode("code"));
    const XMLCh* code = ((DOMElement*)error)->getAttribute(xmar.get());
    
    DOMNode* errorValue = error->getFirstChild();
    const XMLCh* errorTxt = errorValue->getNodeValue();
    std::string scode,serror;
    oa::XMLUtils().DomstrToStd(scode,code);
    oa::XMLUtils().DomstrToStd(serror,errorTxt);
    je.Code(scode);
    je.Message(serror);
    return true;
  }
  bool JabBuild::GetElemValue(DOMNode*const nd,
			      const XMLCh*const name, 
			      std::string &val)
  {
    DOMNodeList *elems = 
      ((DOMElement*)nd)->getElementsByTagName(name);
    if(elems->getLength()<0){
      return false;
    }
    DOMNode *elem = elems->item(0);
    if(!elem || !elem->hasChildNodes()){
      return false;
    }
    DOMNode *txt = elem->getFirstChild();
    if(!txt){
      return false;
    }
    if(txt->getNodeType() != DOMNode::TEXT_NODE){
      return false;
    }

    const XMLCh *domVal = txt->getNodeValue();
    oa::XMLUtils().DomstrToStd(val,domVal);
    return true;
  }
};//jabber ns
