/* -*- Mode: c++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 * OAjabber library
 * Contains objects to aid in Jabber client development and extend the
 * basic functionality of jabber.
 *
 * Copyright (C) 2001 Justin Kirby
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Justin Kirby <justink@rochester.rr.com>
 *
 *******************************************************************/
#ifndef OAJABBER_JABBUILD_H
#define OAJABBER_JABBUILD_H
#include "xstrm/oaxstrm_exports.h"
#include "xstrm/jabpacket.h"

namespace jabber
{
  class OAXSTRM_API JabBuild;

  /** @defgroup gJabBuild Packet Builders
   * @{
   */
  ///Abstract Base class for builder objects
  /**
   * stores xml/attr name/value 
   * Constructs a specific Jabber object
   */
  class JabBuild
  {
  public:
    ///Your simple ctor
    JabBuild(){}
    JabBuild(const JabBuild& jb){}
    virtual ~JabBuild(){}

    virtual std::string ID()const=0;
    virtual JabBuild* Clone()=0;

    virtual bool IsDeferrable()const{
      return false;
    }

    virtual bool CanBuild(xercesc::DOMNode *const nd)const=0;

    ///Pure Virtual Build
    /**
     * This is used to construct the actual object.
     * The results are cached, must call Result() to obtain
     * the created object
     * @param nd DOMNode to build from
     * @return pointer to jabpacket, null on failure
     */
    virtual JabPacket* Build(xercesc::DOMNode *const nd)=0;
  protected:
    ///Get value of to attribute
    /**
     * Get the value of the to attribute on DOMNode nd
     * if one exists.
     * @param to The value of the to attribute on the DOMNode nd
     *           if it exists.
     * @param nd The DOMNode to look for the attribute
     * @return zero on success, non-zero on failure
     */
    virtual int GetTo(std::string &to, xercesc::DOMNode*const nd);

    ///Get value of from attribute
    /**
     * Get the value of the from attribute on DOMNode nd
     * if one exists.
     * @param from The value of the from attribute on the DOMNode nd
     *           if it exists.
     * @param nd The DOMNode to look for the attribute
     * @return zero on success, non-zero on failure
     */
    virtual int GetFrom(std::string &from, xercesc::DOMNode *const nd);

    ///Get value of id attribute
    /**
     * Get the value of the id attribute on DOMNode nd
     * if one exists.
     * @param id The value of the id attribute on the DOMNode nd
     *           if it exists.
     * @param nd The DOMNode to look for the attribute
     * @return zero on success, non-zero on failure
     */
    virtual int GetId(std::string &id, xercesc::DOMNode *const nd);

    ///Get value of type attribute
    /**
     * Get the value of the type attribute on DOMNode nd
     * if one exists.
     * @param type The value of the type attribute on the DOMNode nd
     *           if it exists.
     * @param nd The DOMNode to look for the attribute
     * @return zero on success, non-zero on failure
     */
    virtual int GetType(std::string &type, xercesc::DOMNode *const nd);

    ///Get the jabber error
    /**
     * Builds a jabber error from the DOMNode if one exists.
     * @param je The JabError built if an error element was
     *           found on the DOMNode nd
     * @param nd The DOMNode to look for an error
     * @return zero on success, non-zero on failure
     */
    virtual bool GetError(JabError &je, xercesc::DOMNode *const nd);

    ///Get value of an element
    /**
     * Get the value of the DOMText node child of
     * DOMNode. 
     * @param nd The DOMNode to extract the value from
     * @param name Name of the DOMNode to do stuff with
     * @param val Value extracted
     * @return true on success, false otherwise
     */
    virtual bool GetElemValue(xercesc::DOMNode *const nd,
			      const XMLCh*const name, 
			      std::string &val);
  };
  /** @}*/
};



#endif
