/* -*- Mode: c++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 * OAjabber library
 * Contains objects to aid in Jabber client development and extend the
 * basic functionality of jabber.
 *
 * Copyright (C) 2001 Justin Kirby
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Justin Kirby <justink@rochester.rr.com>
 *
 *******************************************************************/
#include "framework/first.h"
#include "openaether.hpp" 
#include "xstrm/jabbuilders.h"

XERCES_CPP_NAMESPACE_USE
namespace jabber
{
  
  void Builders::Insert(JabBuild &jb)
  {
    oapr::mlock lck(m_mutex);
    cont_type::iterator fnd = m_builders.find(jb.ID());
    if(fnd == m_builders.end()){
      /* not there yet, so create a sharedptr and isnert*/
      boost::shared_ptr<JabBuild> tmp(jb.Clone());
      m_builders.insert(std::make_pair(jb.ID(),tmp));
    }
  }
  void Builders::InsertX(JabBuildX &jbx)
  {
    oapr::mlock lck(m_mutex);
    contx_type::iterator fnd=m_xbuilders.find(jbx.ID());
    if(fnd == m_xbuilders.end());{
      boost::shared_ptr<JabBuildX> tmp(jbx.CloneX());
      m_xbuilders.insert(std::make_pair(jbx.ID(),tmp));
    }
  }
  void Builders::Flush()
  {
    oapr::mlock lck(m_mutex);
    cont_rm::iterator rpos=m_remove.begin();
    for(;rpos!=m_remove.end(); ++rpos){
      m_builders.erase(*rpos);
    }

    cont_rm::iterator xpos=m_xremove.begin();
    for(;xpos!=m_xremove.end(); ++xpos){
      m_xbuilders.erase(*xpos);
    }

    m_remove.erase(m_remove.begin(),m_remove.end());
    m_xremove.erase(m_xremove.begin(),m_xremove.end());

  }
  Builders::cont_ret Builders::Query(DOMNode *const nd)
  {
    oapr::mlock lck(m_mutex);
    cont_ret retval;
    cont_type::iterator beg=m_builders.begin();
    for(;beg!=m_builders.end();++beg){
      if(beg->second->CanBuild(nd)){
        retval.push_back(beg->second);
      }
    }
    return retval;
  }
  Builders::contx_ret Builders::QueryX(xercesc::DOMNode *const nd)
  {
    oapr::mlock lck(m_mutex);
    contx_ret retval;
    contx_type::iterator beg=m_xbuilders.begin();
    for(;beg!=m_xbuilders.end();++beg){
      if(beg->second->CanBuild(nd)){
        retval.push_back(beg->second);
      }
    }
    return retval;
  }
};
