/* -*- Mode: c++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 * OAjabber library
 * Contains objects to aid in Jabber client development and extend the
 * basic functionality of jabber.
 *
 * Copyright (C) 2001 Justin Kirby
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Justin Kirby <justink@rochester.rr.com>
 *
 *******************************************************************/
#ifndef OAJABBER_JABBUILDERS_H
#define OAJABBER_JABBUILDERS_H

#include <map>
#include <vector>
#include <boost/smart_ptr.hpp>
#include "oapr/oapr_mutex.hpp"
#include "oapr/oapr_pool_mgr.hpp"
#include "xstrm/jabbuild.h"
#include "xstrm/jabbuildx.h"
#include "xstrm/oaxstrm_exports.h"

namespace jabber
{
	class OAXSTRM_API Builders;
  class Builders
  {
  public:
    typedef boost::shared_ptr<JabBuild> elem_type;
    typedef boost::shared_ptr<JabBuildX> elemx_type;
    typedef std::map<std::string,elem_type> cont_type;
    typedef std::map<std::string,elemx_type> contx_type;
    typedef std::vector<elem_type> cont_ret;
    typedef std::vector<elemx_type> contx_ret;
    typedef std::vector<std::string> cont_rm;
  public:
    Builders()
      : m_mutex(oapr::pool_mgr::instance()->unique())
    {}
    ~Builders(){}
    void Insert(JabBuild &jb);
    void Remove(const std::string &id){
      m_remove.push_back(id);
    }

    void InsertX(JabBuildX &jbx);
    void RemoveX(const std::string &id){
      m_xremove.push_back(id);
    }

    void Flush();
    cont_ret Query(xercesc::DOMNode *const nd);
    contx_ret QueryX(xercesc::DOMNode *const nd);
  private:
    cont_type m_builders;
    cont_rm m_remove;
    contx_type m_xbuilders;
    cont_rm m_xremove;
    oapr::mutex m_mutex;
  };
};
#endif
