/********************************************************************
 * OAjabber library
 * Contains objects to aid in Jabber client development and extend the
 * basic functionality of jabber.
 *
 * Copyright (C) 2001 Justin Kirby
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Justin Kirby <justink@rochester.rr.com>
 *
 *******************************************************************/
#include "framework/first.h"
#include "openaether.hpp"
#include <algorithm>
#include "xstrm/jabevent.h"

namespace jabber
{
  void EventChain::Subscribe(const std::string& subj, event_elem ev)
  {
    if(!ev.get()){
      /**\todo throw here!!! */
      return;
    }
    sub_list::iterator exists= Find(subj,ev->Id());
    if(exists == m_subs.end()){
      m_subs.insert(std::make_pair(subj,ev));
    }else if(!ev->ForceUnique() && !exists->second->ForceUnique()){
      m_subs.insert(std::make_pair(subj,ev));
    }
  }
  void EventChain::Unsubscribe(const std::string& subj,const std::string& id)
  {
    sub_list::iterator exists=Find(subj,id);
    if(exists != m_subs.end()){
      m_subs.erase(exists);
    }
  }
  void EventChain::Publish(const JabJid& jid, JabPacket *pckt)
  {
    sub_list::iterator pos=m_subs.lower_bound(pckt->Uri()),
      end=m_subs.upper_bound(pckt->Uri());

    for(;pos!=end;++pos){
      pos->second->HandleEvent(jid,pckt);
    }
  }
  EventChain::sub_list::iterator EventChain::Find(const std::string& subj,
						  const std::string& id)
  {
    sub_list::iterator pos=m_subs.lower_bound(subj);
    sub_list::iterator end=m_subs.upper_bound(subj);
    for(;pos!=end;++pos){
      if(pos->second->Id()==id){
	return pos;
      }
    }
    return m_subs.end();
  }
};
