/* -*- Mode: c++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 * OAjabber library
 * Contains objects to aid in Jabber client development and extend the
 * basic functionality of jabber.
 *
 * Copyright (C) 2001 Justin Kirby
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Justin Kirby <justink@rochester.rr.com>
 *
 *******************************************************************/
#ifndef OAJABBER_JABEVENT_H
#define OAJABBER_JABEVENT_H

#include "xstrm/oaxstrm_exports.h"
#include <vector>
#include <map>
#include <boost/smart_ptr.hpp>
#include "xstrm/jabjid.h"
#include "xstrm/jabpacket.h"

namespace jabber
{
  class OAXSTRM_API JabEvent;
  class OAXSTRM_API EventChain;
  class JabEvent
  {
  public:
    enum HANDLE_MODES{PASS=0,USED,CONTINUE,DELETEME};
  public:
    JabEvent()
      : p_packet(0){}
    JabEvent(const JabEvent& je)
      :m_jid(je.m_jid),
       p_packet(je.p_packet){}
    virtual ~JabEvent(){}
    virtual HANDLE_MODES HandleEvent(const JabJid& jid, JabPacket *pckt){
      return Exec(jid,pckt);
    }
    virtual bool ForceUnique()const{return true;}
    virtual std::string Id()const=0;
  protected:
    virtual HANDLE_MODES Exec(const JabJid& jid, JabPacket *pckt)=0;
  protected:
    JabJid m_jid;
    JabPacket *p_packet;
  };


  class EventChain
  {
  public:
    typedef boost::shared_ptr<JabEvent> event_elem;
    //    typedef std::vector<event_elem> event_cont;
    typedef std::multimap<std::string,event_elem> sub_list;
  public:
    EventChain(){};
    virtual ~EventChain(){}
    virtual void Subscribe(const std::string& subj, event_elem ev);
    virtual void Unsubscribe(const std::string& subj,
                             const std::string& id);
    virtual void Publish(const JabJid& jid, JabPacket *pckt);
  protected:
    sub_list::iterator Find(const std::string& subj, const std::string& id);
  protected:
    sub_list m_subs;    
  };

};
#endif
