/* -*- Mode: c++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 * OAjabber library
 * Contains objects to aid in Jabber client development and extend the
 * basic functionality of jabber.
 *
 * Copyright (C) 2001 Justin Kirby
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Justin Kirby <justink@rochester.rr.com>
 *
 *******************************************************************/
#include "framework/first.h"
#include "openaether.hpp"
#include "xstrm/jabrecvthrd.h"
#include "oapr/oapr_exception.hpp"
#include "oapr/oapr_exc_utils.hpp"
#include "xstrm/jabberdom.h"
#include "xstrm/iojabber.h"
#include "framework/xmlutil.hpp"
#include "framework/debug_xml.hpp"

XERCES_CPP_NAMESPACE_USE
namespace jabber
{
  RecvThread::RecvThread()
    : oapr::thread_class(oapr::pool_mgr::instance()->unique())
    ,m_recvStream(false)
  {}
  void RecvThread::Args(const thrdarg& ta)
  {
    m_strm=ta.stream;
    m_builders=ta.builders;
    m_events=ta.eventchain;
    m_sync=ta.sync;
    m_parser=ta.parser;
    m_owner=ta.owner;

  }
  apr_status_t RecvThread::execute()
  {
    try{
      oapr::debug::write("STARTING THREAD");
      boost::scoped_ptr<InputSource>  insrc(new TcpInputSource(m_strm));
      if(!m_strm->is_open()){
        return APR_BADARG;
      }

      if(!m_parser->parseFirst((*insrc),m_token)){
        /**\bug The best way to handle this it to just return
         * however, reason for termination would be nice
         * or at least some means of signalling that we are 
         * dead
         */
        oapr::debug::write("ENDING THREAD");
        return APR_EINIT;
      }

      while(state()==oapr::tsRunning && m_strm->is_open()){
        if(!m_parser->parseNext(m_token)){//did parse fail?
          //if so return APR_EGENERAL
          oapr::debug::write("ENDING THREAD");
          return APR_EGENERAL;
        }
        if(state()==oapr::tsRunning){
          Process();
        }else{
          oapr::debug::write("ENDING THREAD");
          return APR_SUCCESS;
        }
      }
    }OAPR_PRINT(&std::cerr, oapr::debug::write("ENDING THREAD");return APR_EGENERAL);
    oapr::debug::write("ENDING THREAD");
    return APR_SUCCESS;
  }
  void RecvThread::Process()
  {
    /* unfortunately we have to cast up to the content handler*/
    JabberDom *jdom = dynamic_cast<JabberDom*>(m_parser->getContentHandler());
    DOMNode *domPacket=0;
    boost::scoped_ptr<JabPacket> jabPacket;
    Builders::cont_ret builders;
    Builders::cont_ret::iterator bldPos,bldEnd;

    Builders::contx_ret xbuilders;
    Builders::contx_ret::iterator xbldPos,xbldEnd;

    /* have we dealt with the main stream? */
    if(!m_recvStream){
      domPacket=jdom->getElement();
      if(domPacket){
        m_recvStream=true;
      }
    }else{
      domPacket=jdom->NextPacket();
      if(domPacket){
        jdom->PopPacket();
      }
    }
    while(domPacket){
      /* cleaning*/
      oa::XMLUtils().Clean(domPacket);

      /**debug*/
      oapr::debug::write("[IN]");
      oapr::debug::write(oa::debug_xml(domPacket));

      /* we should flush the builders */
      m_builders->Flush();

      
      builders = m_builders->Query(domPacket);
      bldPos = builders.begin();
      bldEnd = builders.end();
      /* now grab the extensions */
      xbuilders = m_builders->QueryX(domPacket);
      xbldPos = xbuilders.begin();
      xbldPos = xbuilders.end();

      for(;bldPos!=bldEnd;++bldPos){

        jabPacket.reset((*bldPos)->Build(domPacket));

        if(jabPacket.get()){
          /*
          JabJid toJid(jabPacket->To());
          if(toJid.IsNull()){//deal with the case of no to attribute
            toJid.Jid(m_owner);
          }
          */
          JabJid ownerJid;
          ownerJid.Jid(m_owner);
          /* now that the packet is all set, lets add extensions to it*/
          for(;xbldPos!=xbldEnd;++xbldPos){
            JabPacket::x_ptr xtype((*xbldPos)->BuildX(domPacket));
            if(xtype.get() != 0){
              jabPacket->Extension(xtype);
            }
          }
          m_events->Publish(ownerJid,jabPacket.get());
        }
      }
      domPacket=jdom->NextPacket();
      if(domPacket){jdom->PopPacket();}
    }
  }
};
