/* -*- Mode: c++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 * OAjabber library
 * Contains objects to aid in Jabber client development and extend the
 * basic functionality of jabber.
 *
 * Copyright (C) 2001 Justin Kirby
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Justin Kirby <justink@rochester.rr.com>
 *
 *******************************************************************/
#ifndef OAJABBER_JABRECVTHRD_H
#define OAJABBER_JABRECVTHRD_H
#include <boost/smart_ptr.hpp>
#include "oapr/oapr_thread.hpp"
#include "xstrm/jabbuilders.h"
#include "xstrm/jabevent.h"
#include "framework/sigsync.hpp"
#include "framework/thread_thread.hpp"
#include <xercesc/sax2/SAX2XMLReader.hpp>
#include "oapr/oapr_io.hpp"
/* just some randome notes
 * things we need:
 * tcpstrm
 * builders
 * events
 * sigsync
 * sax parser
 * auth jid
 */

namespace jabber
{
  class RecvThread : public oapr::thread_class
  {
  public:
    typedef oapr::basic_io istream_type;
    typedef oapr::basic_io ostream_type;
    typedef istream_type *istream_ptr;
    typedef ostream_type *ostream_ptr;
    typedef Builders* builders_ptr;
    typedef EventChain* eventchain_ptr;
    typedef xercesc::SAX2XMLReader* parser_ptr;
    typedef oa::SigSync<oa::oa_thread>* sync_ptr;
  public:
    class thrdarg
    {
    public:
      thrdarg(){}
      thrdarg(const thrdarg& ta)
        : stream(ta.stream),
          builders(ta.builders),
          eventchain(ta.eventchain),
          parser(ta.parser),
          sync(ta.sync),
          owner(ta.owner){}
      ~thrdarg(){}
      istream_ptr stream;
      builders_ptr builders;
      eventchain_ptr eventchain;
      parser_ptr parser;
      sync_ptr sync;
      std::string owner;
    };
  public:
    RecvThread();
    ~RecvThread(){}
    void Args(const thrdarg& ta);
  protected:
    apr_status_t execute();
    void Process();
  private:
    xercesc::XMLPScanToken m_token;
    istream_ptr m_strm;
    builders_ptr m_builders;
    eventchain_ptr m_events;
    sync_ptr m_sync;
    parser_ptr m_parser;
    std::string m_owner;
    bool m_recvStream;
  };

};
#endif
